<?php
$EVENT_TYPES = array
(
    /**
    * @desc Occurs once per page creation and gives the plugin an opportunity to do
    * any initialization work that needs to be done.
    * Parameters
    *   [None]
    * Returns:
    *   Return true to enable plugin
    */
    'EVENT_PLUGIN_INIT',

    /**
    * @desc Occurs prior to processing a URL action such as URLACTION_VIEWCAT
    * Parameters
    *   action => The action being handled (e.g. URLACTION_VIEWCAT)
    *   id => An associated id
    * Returns:
    *   Return true to continue processing
    */
    'EVENT_EXECUTE_ACTION_PRE',

    /**
    * @desc Handler has an opportunity to choose what page to go to after any
    * iwForm-based form has been submitted successfully.
    * Parameters:
    *   form => The form object reference
    *   link => The link to redirect to (change this if necessary)
    */
    'EVENT_FORM_REDIRECT_NEEDED',
    
    /**
    * @desc Occurs after processing a URL action such as URLACTION_VIEWCAT but
    * before any content has been rendered to the screen.
    * Parameters
    *   action => The action being handled (e.g. URLACTION_VIEWCAT)
    *   id => An associated id
    *   page => A reference to the page object generating the html.  Use
    *       the addContent method of this object to add your own html.
    */
    'EVENT_EXECUTE_ACTION_PRE_CONTENT',
    
    /**
    * @desc Occurs after processing a URL action such as URLACTION_VIEWCAT and
    * after all content has been rendered to the screen.
    * Parameters
    *   action => The action being handled (e.g. URLACTION_VIEWCAT)
    *   id => An associated id
    */
    'EVENT_EXECUTE_ACTION_POST_CONTENT',

    /** 
    * @desc Called after the category has been added to the database
    * Parameters:
    *   object => The object that is being added
    */
    'EVENT_OBJECT_ADDED',

    /** 
    * @desc Called after the category has been edited in the database
    * Parameters:
    *   object => The object that is being edited
    */
    'EVENT_OBJECT_EDITED',
    
    /** 
    * @desc Called just before the object is added to the database. Set continue
    * to false to prevent the object from being added
    * Parameters:
    *   object => The object that is being added
    *   continue => Set this to false to prevent editing of this object.
    */    
    'EVENT_OBJECT_ADDING',
    
    /** 
    * @desc Called just before the object is edited in the database. Set continue
    * to false to prevent the object from being edited
    * Parameters:
    *   object => The object that is being edited
    *   continue => Set this to false to prevent editing of this object.
    */    
    'EVENT_OBJECT_EDITING',    

    /**
    * @desc Called when navigation items are being constructed
    * Parameters:
    *   vertical => True if this is a vertical navigation area or false 
    *                   if it's a horizontal one.
    *   blocks => Update this array of navigation objects with new ones or alter
    *               existing ones.  This is an array of NavBlock objects.       
    */
    'EVENT_NAV_NEEDED',
    
    /**
    * @desc Called when form class names are being iterated.  Add your
    * own class form if you want yours to be included in form processing.
    * Parameters:
    *   forms => Add a form name such 'MyForm' which is the name of a class 
    *       derived from iwForm.
    */
    'EVENT_FORMS_NEEDED',
    
    /**
    * @desc Called when action items are needed.  Usually needed
    * for a console page such as the admin or user front page.
    * Parameters:
    *   
    * 
    */
    'EVENT_FRONTPANEL_LINKS_NEEDED',
    
    /**
    * @desc Called when displaying actions links for a given document
    * Parameters:
    *   actions => The array of actions to allow for a given document - 
    *           You can add a new action by calling shPage::makeActionArray
    *           and adding the result to this actions array.
    */
    'EVENT_DOCUMENT_ACTIONS_NEEDED',
    
    /**
    * @desc Called when displaying a preview of a document.  
    * Parameters
    *   doc     => The document object
    *   content => Populate this string with the preview HTML
    */
    'EVENT_DOCUMENT_PREVIEW_NEEDED',
    
    /**
    * @desc Gives listeners a chance to populate settings array with their
    * own settings/values.
    * Parameters:
    *   settings => An array of settings (name=>value).
    */    
    'EVENT_GET_SETTINGS',
    
    /**
    * @desc Gives listeners a chance to add fields to the global settings form
    * Parameters:
    *   fields => An array of fields (name=>value).
    */
    'EVENT_SETTINGS_FORM_ADD_FIELDS',
    
    /**
    * @desc Gives listeners a chance to apply custom fields in the global settings
    * 
    */
    'EVENT_SETTINGS_FORM_APPLY_FIELDS',
    
    /**
    * @desc Gives user the ability to add alter the display of a document before 
    * being displayed in the library browser *and* allows the user to hide the document
    * entirely.
    * Parameters:
    *   allow_visibility - boolean - set to false to hide the document
    *   doc - shItemDocument - The document object - alter this to change the appearance.  
    */
    'EVENT_BROWSER_PRE_DOCUMENT_ADD',
    
    /**
    * @desc Gives user the ability to alter the display parameters of a document that
    * is about to be displayed in the library browser.
    * Parameters:
    *   info - assoc array - Alter the keys in the array to alter the display
    *   doc - shItemDocument - The document object for information purposes only
    */
    'EVENT_BROWSER_PRE_DOCUMENT_DISPLAY',
       
    /**
    * @desc Listener can add additional content near the area where the file description
    * and title are located.
    */
    'EVENT_DOCUMENT_VIEW_SINGLE_HEADER',

    /**
     * @desc Provides an opportunity for plugins to add their own file types to the list
     *            of supported file types.
     * Parameters:
     *     file_types - An array of file types which the receiver can add their own shFileType derived
     *                classes
     */
    'EVENT_FILE_TYPES_NEEDED',
    
    /**
    * @desc Provides a means for replacing the current language with another
    * Parmaeters:
    *   lang - Modify to alter the language code in use (e.g. 'es' for spanish)
    */
    'EVENT_FILE_LANGUAGE_NEEDED',

	/**
	 * @desc Called during authorization of a single action.  Respondent should
	 * 			set the authorization only if the action matters to them.
	 *	action - The action being authorized
	 *	id - The ID of the item being acted upon (can be zero or empty)
	 * 	auth - Set this to one of AUTH_NOOP, AUTH_ALLOW or AUTH_DENY only if you
	 *			are specifically interested in changing the default authorization policy
	 */
	'EVENT_AUTHORIZATION_NEEDED'
);
  
?>
