<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


/**
 * Contains a list of category object
 */
class shCategoryCollection extends shCollection
{
	
	function buildQuery (iwParameterCollection $params, $countOnly)
	{

		$select = '';
		$query = '';

		if ($countOnly)
		{
			$select = 'select COUNT(DISTINCT(si.category_id)) AS total ';
		}
		else
		{
			$select = 'select DISTINCT(si.category_id),si.parent_id,si.name,'.
						'si.description,si.status ';
		}

		$from = 'sh_categories as si, sh_category_access_map as sam ';
		$query = $select.' FROM '.$from.' WHERE 1 ';

		if ( !$params->have(FILTER_USER) &&
		(!$params->have(FILTER_COMPANY) || !$params->have(FILTER_GROUP)))
		{
			// No user filtering
			$query .= ' AND sam.category_id=si.category_id ';
		}
		else
		{
			$cid = IW_NULL;
			$gid = IW_NULL;				
			if (!$params->have(FILTER_USER))
			{
				// Use the company and group stored in the filter
				$companies = $params->find_value(FILTER_COMPANY);
				$groups = $params->find_value(FILTER_GROUP);
				$cid = $companies[0];
				$gid = $groups[0];
			}
			else
			{
				$users = $params->find_value(FILTER_USER);
				$user = null;
				if (count($users) == 0)
				{
					$user = app()->getLoggedInUserObject();
					if ($user)
					{
						$cid = $user->getCompany()->get(PROP_ID);
						$gid = $user->getGroup()->get(PROP_ID);
					}
				}
				else
				{
                    if (is_object($users[0]))
                    {
                        $user = $users[0];
                    }
                    else
                    {
                        $user = new shUser;
                        $user->set(PROP_USERNAME, $users[0]);
                        if (!$user->load())
                        {
                            $user = null;
                        }
                    }

                    if ($user)
                    {                    
                        $permGroup = $user->getPermissions()->getPermissionGroup();                        
                        // Administrators are not restricted under any circumstances
                        if ($permGroup != USER_TYPE_ADMIN)
                        {
                            if ($permGroup == USER_TYPE_DOMAIN_ADMIN)
                            {
                                $cid = $user->getCompany()->get(PROP_ID);
                                $gid = 0;
                            }
                            else
                            {
                                $cid = $user->getCompany()->get(PROP_ID);
                                $gid = $user->getGroup()->get(PROP_ID);
                            }
                        }                        
					}
					else
					{
						app()->addError (sprintf(app()->loadString('STR_USERNOTFOUND'),$users[0]), ERROR_TYPE_SERIOUS);
					}
				}
			}

			if ($cid == COMPANY_UNREGISTERED)
			{
				// When querying for the "EVERYONE"  company, we don't want to include
				//	those documents that are available to all companies/groups because
				//	that still requires a registered user whereas "EVERYONE" means anyone
				//	(registered or not).
				$query .= '  AND (sam.company_id='.$cid.') '.
						'AND (sam.group_id='.$gid.') AND ' .
						'sam.category_id = si.category_id ';
						$query .= ' AND sam.category_id=si.category_id ';

			}
			else if ($cid != IW_NULL || $gid != IW_NULL)
			{
                if ($gid == 0)
                {
                    // Restrict by org only
                    $query .= '  AND (sam.company_id='.$cid.' OR sam.company_id<=0) '.
                            'AND (sam.category_id = si.category_id)';                        
                }
                else
                {
                    // Restrict by org and group
                    $query .= '  AND (sam.company_id='.$cid.' OR sam.company_id<=0) '.
                            'AND (sam.group_id='.$gid.' OR sam.group_id<=0) AND ' .
                            'sam.category_id = si.category_id ';                        
                }
			}
			else
			{
                // There may be no restrictions if the admin is trying to see categories.
                $query .= ' AND sam.category_id=si.category_id ';
			}
		}

		// Filter by document status
		if ($params->have(FILTER_STATUS))
		{
			$requestedStatuses = $params->find_value(FILTER_STATUS);
			$statuses = array ();
			foreach ($requestedStatuses as $status)
			{
				$statuses[] = '(si.status = '.$status.') ';
			}
			$query .= 'AND ('.implode (' OR ',$statuses).')';
		}

		// Filter by category
		if ($params->have(FILTER_CATEGORY))
		{
			$requestedCategories = $params->find_value(FILTER_CATEGORY);
			$categories = array ();
			foreach ($requestedCategories as $category)
			{
				$categories[] = '(si.parent_id = '.$category.') ';
			}

			if (count ($categories))
			{
				$query .= 'AND ('.implode (' OR ',$categories).') ';
				$query .= 'AND (si.parent_id <> si.category_id)';
			}
		}

		// Filter by keyword (used in search)
		if ($params->have(FILTER_KEYWORDS))
		{
			$statement = array();
			$words = $params->find_value(FILTER_KEYWORDS);
			foreach ($words as $word)
			{
				$statement[] = '(si.name LIKE "%'.$word.'%" || si.description LIKE "%'.$word.'%")';
			}

			if (count($statement)>0)
			{
				$query .= sprintf ('AND (%s)',implode (' OR ', $statement));
			}
		}

		if (!$countOnly)
		{
			$query .= $this->getOrderBy($params);
			$query .= ' '.$this->getOrderDirection($params);
			$query .= ' '.$this->getLimit ($params);
		}
		
		// Note, the FILTER_DOCTYPE has to be applied after the results are returned.
		// Note, the FILTER_INBIN has to be applied after the results are returned.
        app()->flushCacheIfInvalid($query);
		return $query;
	}	
	
	function getRootCategory()
	{
		$cat = new shCategory;
		$cat->set(PROP_ID, 0);
		$cat->load();			
		return $cat;
	}
	
	/**
	 * Supports the following parameters:
	 *	NONE
	 * 	Currently only shows the active categories
	 */
	function load (iwParameterCollection $params,$countOnly=false)
	{
		$this->_params = $params;

		$queryCount = $this->buildQuery ($params, true);		
		$count = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$queryCount);
		if ($count !== false)
		{
			$this->_total = $count[0]['total'];
			if ($countOnly == true)
			{
				return;
			}
		}
		else
		{
			$this->_total = 0;
			return;
		}
		
		$query = $this->buildQuery ($params, false);
		$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);

		if ($rows !== false)	
		{            
			// Now load the data into the object.
			foreach ($rows as $row)
			{                
				$cat = new shCategory;
				$cat->setDBKeyValueArray ($row);
				$this->addItem ($cat);
			}

			// Add the root category if a parent category
			//	is not specified in the filter.   Also, don't include the 
            //  root if a category search is happening (keywords).
			if (!$params->have(FILTER_CATEGORY) && !$params->have(FILTER_KEYWORDS))
			{				
				// Also, if the caller is filtering on user, then
				//	we have to make sure that the user has access
				// 	to the root.  Do this here.
				$rootAccess = true;
				if ($params->have(FILTER_USER))
				{
					$users = $params->find_value(FILTER_USER);
					$user = null;					
					if (count($users) == 0)
					{
						$user = app()->getLoggedInUserObject();
					}
					else
					{
                        if (!is_object($users[0]))
                        {
						    $user = new shUser;
						    if (!$user->loadId($users[0]))
						    {
							    $rootAccess = false;
						    }
                        }
                        else
                        {
                            $user = $users[0];
                        }
					}
					
					if ($rootAccess && $user)
					{
						$rootAccess = ($user->getPermissions()->getPermissionGroup() == 
										USER_TYPE_ADMIN);	
					}
					
				}
				
				if ($rootAccess)
				{
					$this->addItem($this->getRootCategory());
				}
			}

			$cat = new shCategory;
			$this->setIcon ($cat->getItemIcon(true));
			$this->setTitle (str('STR_CAT_ALL'));
			$this->setSubtitle (sprintf(str('STR_CAT_TOTAL'),$this->count()));
		}
		else {
			$this->addErrorObject ($rows, ERROR_TYPE_SERIOUS);
		}        
	}

	/**
	 * Returns a complete category object that has the same id as 
	 *	the given integer as long as the collection contains that
	 *	category.
	 *	@param int $id The id of the category
	 *	@return shCategory The category object found or null if not found
	 */
	function findCategory ($id)
	{
		foreach ($this->items() as $cat)
		{
			if ($cat->get(PROP_ID) == $id)
			{
				return $cat;
			}
		}

		return null;
	}

	/**
	 * Filters not supported 
	 */	
	public function processFilters ()
	{
		// Don't do anything
		return new iwParameterCollection();
	}

	/**
	 * Filters not supported 
	 */	
	public function getFilterHtml ()
	{
		$html = '';
		return $html;
	}

	/**
	 * Sortable fields include: 
	 * 	@arg @c name
	 */
	public function getSortableFields ()
	{

		return array (
				'Name'=>$GLOBALS['TABLE_CATEGORIES']['fields']['CATEGORY_NAME'],
				);
	}

	/**
	 * Delete categories button in the returned html
	 */
	public function getFormButtons ()
	{
		if (app()->isAdminLoggedIn())
		{
			$buttons = '<table cellpadding="2" cellspacing="0"><tr><td colspan="2"><div style="font-weight:bold;">'.str('STR_SELECTED_ITEMS').':</div></td></tr>';
			$buttons .= '<tr><td><input type="submit" name="delCat" value="Send To Trash"/></td></tr>';
			$buttons .= '</table>';
			return $buttons;
		}
	}

	/**
	 * Returns @c shCategory
	 */
	public function getClassName()
	{
		return 'shCategory';
	}

	/**
	 * This gets the information about the contained categories for use 
	 * in select statements or any other simple list of categories.
	 * @param $selectedId The ID of the selected category (0 for root).
	 * @return array The array of categories
	 */
	public function toArray()
	{
		$finalArray = array();
		foreach ($this->items() as $item)
		{
			// Don't add the current category to the list if
			//	it's the same one as the one being edited.  A category
			//	can't be its own parent.
			$finalArray[] = array
			(
					'id'=>$item->get(PROP_ID), 
					'name'=>$item->get(PROP_NAME),
					'value'=>$item->get(PROP_ID) 
			);
		}

		return $finalArray;
	}
	

	public function html ($maxPerPage=10,$currentPage=1,$showHeader=true)
	{
		$html = parent::html($maxPerPage,$currentPage,$showHeader);
		$rename = nx_load_template(array(),'forms/form.ajax.rename.tpl.html');
		$html = $rename.$html;
		return $html;
	}	
}

?>
