<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
/**
 * The nexus specific collection base class provides several static methods for displaying 
 *	different nexus objects in the form of a selection control.  It also provides the 
 *	functionality needed to store parameters specific to all collections of this type.  This
 *	is useful for storing information such as sortby fields, filter info, etc.
 */
abstract class shCollection extends iwObjectCollection
{
	/**
	 * Displaysa a group selection control (no companies).  If
	 *	a company is given, then this will return only those 
	 *	groups in that company.  The name of the submitted value is 
	 *	FORM_FILTER_GROUP.
	 *	
	 *	@param int $companyId The id of the company to draw groups from (0 means all groups)
	 *	@param bool $submitOnSelect TRUE if the control should submit whatever form it's contained
	 *				within once the user changes the selection.
	 *	@return Returns the html to display this control.
	 */
	public function getGroupFilterControl ($companyId=0,$submitOnSelect=true)
	{
		$currentFilter = @$_POST[FORM_FILTER_GROUP];
		if ($currentFilter == '')
		{
			// Get the stored category filter, if any
			$params = $this->getStoredParameters ();
			if ($params->have(FILTER_COMPANY))
			{							
				$requestedCompanies = $params->find_value(FILTER_COMPANY);
				$currentFilter = $requestedCompanies[0];
			}
		}
		
		$html = '<select name="'.FORM_FILTER_GROUP.'" id="filter_group"';
		if ($submitOnSelect)
		{
			$html .= 'onchange="javascript:form.submit();">';	
		}
		else 
		{
			$html .= '>';
		}
		
		$html .= '<option value="0">'.str('STR_GROUP_ALL').'</option>';
		$col = new shGroupCollection;
		
		$params = new iwParameterCollection();
		if ($companyId == 0)
		{
			$params->add (new iwParameter(FILTER,false));
		}
		else 
		{
			$p = $params->add (new iwParameterArray (FILTER_COMPANY));
			$p->addItem_nokey ($companyId);
		}
		$col->load ($params);
		
		foreach ($col->items() as $item)
		{
			$id = $item->get(PROP_ID);
			$name = $item->get(PROP_NAME);
			$option = '<option value="'.$id.'" '.($currentFilter==$id?'SELECTED':'').'>'.$name.'</option>';
			$html .= $option;
		}
		
		$html .= '</select>';
		
		return $html;
	}
	 		
	/**
	 * Displays a company selection control.  The name of the submitted value is 
	 *	FORM_FILTER_COMPANY.
	 *	
	 *	@param bool $submitOnSelect TRUE if the control should submit whatever form it's contained
	 *				within once the user changes the selection.
	 *	@return Returns the html to display this control.
	 */	
	public function getCompanyFilterControl ($submitOnSelect=true)
	{
		$currentFilter = @$_POST[FORM_FILTER_COMPANY];
		if ($currentFilter == '')
		{
			// Get the stored category filter, if any
			$params = $this->getStoredParameters ();
			if ($params->have(FILTER_COMPANY))
			{							
				$requestedCompanies = $params->find_value(FILTER_COMPANY);
				$currentFilter = $requestedCompanies[0];
			}
		}
		
		$html = '<select name="'.FORM_FILTER_COMPANY.'" id="filter_company" ';
		if ($submitOnSelect)
		{
			$html .= 'onchange="javascript:form.submit();">';	
		}
		else 
		{
			$html .= 'onchange="xajax_shAjaxServer.populateCompanyGroups(xajax.getFormValues(\'objectCollectionForm\'));">';
		}
		
		$html .= '<option value="0">All Companies</option>';
		$col = new shCompanyCollection;
		$col->load (new iwParameterCollection());
		foreach ($col->items() as $item)
		{
			$id = $item->get(PROP_ID);
			$name = $item->get(PROP_NAME);
			$option = '<option value="'.$id.'" '.($currentFilter==$id?'SELECTED':'').'>'.$name.'</option>';
			$html .= $option;
		}
		
		$html .= '</select>';
		
		return $html;
	}
	
	
	/**
	 * Displays a category selection control.  The name of the submitted value is 
	 *	FORM_FILTER_CATEGORY.
	 *	
	 *	@param bool $submitOnSelect TRUE if the control should submit whatever form it's contained
	 *				within once the user changes the selection.
	 *	@return Returns the html to display this control.
	 */	
 	public function getCategoryFilterControl($submitOnSelect=true)
 	{
		$currentFilter = @$_POST[FORM_FILTER_CATEGORY];
		if ($currentFilter == '')
		{
			// Get the stored category filter, if any
			$params = $this->getStoredParameters ();
			if ($params->have(FILTER_CATEGORY))
			{							
				$requestedCategories = $params->find_value(FILTER_CATEGORY);
				$currentFilter = $requestedCategories[0];
			}
		}
		
		$html = '<select name="'.FORM_FILTER_CATEGORY.'" id="filter_category"';
		if ($submitOnSelect)
		{
			$html .= 'onchange="javascript:form.submit();">';	
		}
		else 
		{
			$html .= '>';
		}
		
		$html .= '<option value="0">'.str('STR_CAT_ALL').'</option>';
		$catcol = new shCategoryCollection;
		$catcol->load (new iwParameterCollection());
		foreach ($catcol->items() as $cat)
		{
			$id = $cat->get(PROP_ID);
			$name = $cat->get(PROP_NAME);
			$option = '<option value="'.$id.'" '.($currentFilter==$id?'SELECTED':'').'>'.$name.'</option>';
			$html .= $option;
		}
		
		$html .= '</select>';
		
		return $html;
 	 }

	/**
	 * Displays a user selection control.  The name of the submitted value is 
	 *	FORM_FILTER_USER.
	 *	
	 *	@param bool $submitOnSelect TRUE if the control should submit whatever form it's contained
	 *				within once the user changes the selection.
	 *	@return Returns the html to display this control.
	 */	
	public function getUserFilterControl()
	{
		$currentFilter = @$_POST[FORM_FILTER_USER];
		if ($currentFilter == '')
		{
			// Get the stored category filter, if any
			$params = $this->getStoredParameters ();
			
			if ($params->have(FILTER_USER))
			{							
				$requestedUsers = $params->find_value(FILTER_USER);
				$currentFilter = $requestedUsers[0];
			}			
		}
		
		$html = '<select name="'.FORM_FILTER_USER.'" id="filter_user" onchange="javascript:form.submit();">';
		$html .= '<option value="">'.str('STR_USER_ALL').'</option>';
		$col = new shUserCollection;
		$col->load (new iwParameterCollection());
		foreach ($col->items() as $user)
		{
			$id = $user->get(PROP_USERNAME);
			$name = $user->getFullName();
			$option = '<option value="'.$id.'" '.($currentFilter==$id?'SELECTED':'').'>'.$name.'</option>';
			$html .= $option;
		}
		
		$html .= '</select>';
		
		return $html;		
	}
	
	/**
	 * Retrieves the parameter stored for classes of this type.  Parameters
	 *	are stored for the life of a PHP session and are keyed to the 
	 *	classname.  All instances of this object type
	 *	will have the same parameter values throughout the session.
	 *	@return iwParameterCollection The collection of parameters stored
	 *				for this class.
	 */
	public function getStoredParameters()
	{
		$params = new iwParameterCollection();
		
		if (isset ($_SESSION['__Params_'.get_class($this)]))
		{
			$params = $_SESSION['__Params_'.get_class($this)];
		}
		
		return $params;
	}
	
	/**
	 * Store the given parameters object in the session for all
	 *	classes of this type.
	 * 	@param iwParameterCollection $params The parameters to store.
	 */
	public function storeParameters ($params)
	{
		$_SESSION['__Params_'.get_class($this)] = $params;
	}
	
	/**
	 * This will search the given parameters object for the
	 * 	sort direction parameter.  If found, then it is returned,
	 *	otherwise an empty string is returned.  If the sort order
	 *	parameter is set but not the sort direction then 'asc' is
	 *	always returned.
	 *	@param iwParameterCollection $params The parameters to look through.
	 */
	public function getOrderDirection ($params)
 	{
 	 	if ($this->getOrderBy($params)!='')
 	 	{
	 	 	if ($params->have(FILTER_SORTDIR))
	 	 	{
		 	 	return $params->find_value(FILTER_SORTDIR);
	 	 	}
	 	 	
	 	 	return 'asc';
 	 	}
 	 	return '';
 	 }

	/**
	 * This will return the order by parameter in the form of a
	 *	SQL qualifier (as in ORDER BY ....)
	 *	@param iwParameterCollection $params The parameters to look through.
	 *	@return string Returns an empty string if not found
	 */ 
	function getOrderBy ($params)
	{
        if ($params->have(FILTER_SORTBY))
	 	{
         	return ' ORDER BY '.$params->find_value(FILTER_SORTBY);
	 	}
	 	return '';
	}
 
	/**
	 * This will return the limit parameters in the given parameter collection.  Those
	 * 	parameters are FILTER_START and FILTER_PERPAGE.  It will return them in 
	 *		the form of a SQL qualifier ( as in LIMIT [start],[total])
	 *		@param iwParameterCollection $params The parameters to look through
	 *		@return string The LIMIT html or an empty string if one or both of the 
	 *					limit parameters were not found.
	 */	 
	function getLimit ($params)
	{
	 	if ($params->have(FILTER_START) &&
	 		$params->have (FILTER_PERPAGE))
	 	{	 		
		 	return ' LIMIT '.$params->find_value(FILTER_START).','.$params->find_value(FILTER_PERPAGE);
	 	}
	 	return '';
	 }
 }
 
?>
