<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * Contains company objects 
 */
class shCompanyCollection extends shCollection
{
	/**
	 * The following parameters are supported:
	 *	@arg FILTER_INCLUDE_AGGREGATE (bool) - If true this will include 
	 *				the 'All' and 'Everyone' companies. 
	 */
	function load (iwParameterCollection $params,$countOnly=false)
	{
		$this->_params = $params;
		
		$limit = $this->getLimit ($params);		
        $sortInfo = $this->getOrderBy($params);
        $sortInfo .= ' '.$this->getOrderDirection($params);
		
		$queryCount = 'select count(*) as total from '.$GLOBALS['TABLE_COMPANIES']['name'].' where status <> '.COMPANY_STATUS_DELETED;
        app()->flushCacheIfInvalid($queryCount);
		$count = db()->CacheGetOne($queryCount);
                
		$this->_total = $count;			
        
		$query = 'select * from '.$GLOBALS['TABLE_COMPANIES']['name'].' where status <> '.COMPANY_STATUS_DELETED;
		$query .= $limit.' '.$sortInfo;
        
        app()->flushCacheIfInvalid($query);
		$rows = db()->CacheGetAll(DB_CACHE_EXPIRE,$query);  
        		
        if ($rows !== false)	
		{	
            // First add the 'aggregate' companies
            if ($params->have(FILTER_INCLUDE_AGGREGATE))
            {
                $comp = new shCompany;
                $comp->set(PROP_ID, COMPANY_ALL);
                $comp->set(PROP_NAME, str('STR_COMPANY_ALL'));
                $comp->set(PROP_DESCRIPTION, str('STR_COMPANY_ALL_DESC'));
                $this->addItem ($comp);

                // The everyone company referes to those who are not registered 
                //    at all.
                $comp = new shCompany;
                $comp->set(PROP_ID, COMPANY_UNREGISTERED);
                $comp->set(PROP_NAME, str('STR_EVERYONE'));
                $comp->set(PROP_DESCRIPTION, str('STR_EVERYONE_DESC'));
                $this->addItem ($comp);
            }
                    																
			// Now load the data into the object.
			foreach ($rows as $row)
			{
				$comp = new shCompany;
				$comp->setDBKeyValueArray ($row);
				$this->addItem ($comp);
			}		
			
			$this->setIcon (shCompany::getItemIcon(true));
			$this->setTitle (str('STR_COMPANY_ALL'));				
			$this->setSubtitle (sprintf(str('STR_COMPANY_TOTAL'),$this->count()));
							
		}
		else {
			$this->addErrorObject ($rows, ERROR_TYPE_SERIOUS);
		}	
	}

	/**
	 * Filters not supported
	 */
	public function processFilters ()
	{
		// Don't do anything 
		return new iwParameterCollection();
	}
	
	/**
	 * Filters not supported
	 */
	function getFilterHtml ()
	{
		$html = '';
		return $html;
	}

	/**
	 * Supports 'Send To Trash'
	 */
 	function getFormButtons ()
 	{
 				if (app()->isAdminLoggedIn())
		{
 			$buttons = '<table cellpadding="2" cellspacing="0"><tr><td colspan="2"><div style="font-weight:bold;">'.str('STR_SELECTED_ITEMS').':</div></td></tr>';
			$buttons .= '<tr><td><input type="submit" name="delCompany" value="'.str('STR_ACTION_TRASH').'"/></td></tr>';
			$buttons .= '</table>';
 			return $buttons;
		}
 	}	

	/**
	 * Returns @c shCompany
	 */
	function getClassName()
	{
		return 'shCompany';
	}

	/**
	 * The following fields are sortable
	 *	@arg @c name
	 *
	 */
	function getSortableFields ()
	{
		
		return array (
				str('STR_SORT_NAME')=>$GLOBALS['TABLE_COMPANIES']['fields']['COMPANY_NAME'],
				);
	}	
}

?>
