<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shModCollection extends shCollection
{
	function buildQuery (iwParameterCollection $params, $countOnly)
	{
		
		$title = 'Log';
		$select = '';
		$query = '';

		$innerjoin = ' INNER JOIN iw_users AS iwu ON iwm.username=iwu.username ';
		if ($countOnly)
		{
			$select = 'select COUNT(*) AS total ';
		}
		else
		{
			$select = 'select * ';
		}

		if (!$params->have(FILTER_USER))
		{
			// No user filtering
			$query = $select.'from iw_modifications as iwm '.$innerjoin.' WHERE 1 ';
		}
		else
		{
			$requestedUsers = $params->find_value(FILTER_USER);
			$username = $requestedUsers[0];
			$query = $select.'from iw_modifications as iwm WHERE iwm.username="'.$username.'"';
			$title .= '<span style="font-size:11px;font-weight:normal">&nbsp;[Filtered by user]</span>';
		}

		// Filter by document status
		if ($params->have(FILTER_ACTION))
		{
			$requestedActions = $params->find_value(FILTER_ACTION);
			$actions = array ();
			foreach ($requestedActions as $action)
			{
				$actions[] .= '(iwm.action = '.$action.') ';
			}

			$query .= ' AND ('.implode (' OR ',$actions).')';
			$title .= '<span style="font-size:11px;font-weight:normal">&nbsp;'.str('STR_MOD_FILTER_ACTION').'</span>';
		}


		// Filter by modification date
		if ($params->have(FILTER_MODIFY_DATE))
		{
			$dates = $params->find_value(FILTER_MODIFY_DATE);
			if ($dates['start'])
			{
				$startDate = iwUtilities::databaseDateFormat ($dates['start']);
				$query .= ' AND (iwm.date > "'.$startDate.'")';
			}
			if ($dates['end'])
			{
				$endDate = iwUtilities::databaseDateFormat ($dates['end']);
				$query .= ' AND (iwm.date < "'.$endDate.'")';
			}
		}

		// Filter by keyword (used in search)
		if ($params->have(FILTER_OBJECT))
		{
			$object_info = $params->find_value(FILTER_OBJECT);			
			$type = $object_info['type'];
			$id = $object_info['id'];
			$query .= " AND (iwm.subject_id=$id AND iwm.subject_type=$type)";
			$title .= '<span style="font-size:11px;font-weight:normal">&nbsp;'.str('STR_MOD_FILTER_OBJECT').'</span>';
		}

		$query .= $this->getOrderBy($params);
		$query .= ' '.$this->getOrderDirection($params);

		if (!$countOnly)
		{
			$query .= $this->getLimit ($params);
		}

		$this->setTitle ($title);

        app()->flushCacheIfInvalid($query);
                
		return $query;
	}

	function load (iwParameterCollection $params,$countOnly=false)
	{
		$this->_params = $params;
		
		$queryCount = $this->buildQuery ($params, true);
		$count = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$queryCount);
		if ($count == false)	
		{																	
			$this->addError ($count, ERROR_TYPE_SERIOUS);
			return false;
		}
		
		$this->_total = $count[0]['total'];

		$query = $this->buildQuery ($params, false);
		$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($rows !== false)	
		{																	
			// Now load the data into the object.
			foreach ($rows as $row)
			{
				$mod = new shModRecord;

				// If the user information is included, then the mod object
				//	will take care of populating its own user object.
				$mod->setDBKeyValueArray ($row);

				$this->_objects[] = $mod;
                
			}

			$icon = app()->get('template_dir').'/images/history_icon_large.jpg';
			$this->setIcon ($icon);
			$this->setSubtitle (sprintf(str('STR_MOD_SUBTITLE'),$this->count()));
		}
		else 
        {
			$this->addErrorObject ($rows, ERROR_TYPE_SERIOUS);
		}
	}

	function getAdminActions ()
	{
		return array ();
	}

	function getActions ()
	{
		return array ();
	}

	public function processFilters ()
	{
		
		if (!isset ($_GET[URL_FILTERED]))
		{
			// Clear the filters and all remembered parameters
			//	if the 'filtered' param is not given.
			$this->storeParameters(new iwParameterCollection());
		}

		$params = $this->getStoredParameters();

		$user = @$_POST[FORM_FILTER_USER];
		if ($user == '')
		{
			$user = @$_GET[URL_FILTER_USER];
		}

		if ($user != '')
		{
			$p = new iwParameterArray(FILTER_USER);
			$p->addItem_nokey ($user);
			$params->add ($p);			
		}

		$obtype = @$_POST[FORM_FILTER_OBTYPE];
		$obid = @$_POST[FORM_FILTER_OBID];
		if ($obtype == '' && $obid == '')
		{
			$obtype = @$_GET[URL_FILTER_OBTYPE];
			$obid = @$_GET[URL_FILTER_OBID];
		}

		if ($obtype != '' || $obid != '')
		{
			$p = new iwParameterArray(FILTER_OBJECT);
			
			if ($obtype != '')
			{
				$p->addItem ('type',$obtype);
			}

			if ($obid != '')
			{
				$p->addItem ('id',$obid);
			}
			
			$params->add($p);
			
		}

		$this->storeParameters ($params);

		return $params;
	}

	function getFilterHtml ()
	{
		$html = '';
		return $html;
	}

	function allowSorting ()
	{
		return true;
	}

	function itemsAreSelectable () 
	{
		return false;
	}

	function getSortableFields ()
	{

		return array (
				str('STR_SORT_DATE')=>$GLOBALS['TABLE_MOD']['fields']['MOD_DATE'],
				);
	}
	
	function getClassName ()
	{
		return 'shModRecord';
	}
	
}
?>
