<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shPoolItemCollection extends shCollection
{
	function getAdminActions ()
	{
		return array ();
	}
	
	function getActions ()
	{
		return array ();
	}
	
	function setItemByIds ($ids)
	{
		
	}
	
	/**
	 * Loads all the pool items that are in the currently logged in 
	 * user's pool.  If the administrator is logged in then this will
	 * load ALL pool items from all users pools
	 *	@param bool $countOnly Loads only the number of items, not the items themselves
	 *	@return bool Returns true if the load was successful, false otherwise.
	 */
	public function loadUserPool ($countOnly)
	{
		$collectionParams = new iwParameterCollection;
		
		if (app()->isUserLoggedIn())
		{
			if (!app()->isAdminLoggedIn())
			{
				$p = new iwParameterArray(FILTER_USER);
				$p->addItem_nokey (app()->getLoggedInUserObject()->get(PROP_USERNAME));
				$collectionParams->add($p);			
			}
		}
		        
		return $this->load($collectionParams,$countOnly);
	}
	
	function buildQuery (iwParameterCollection $params, $countOnly)
	{                
		$select = '';
		$query = '';
		
		if ($countOnly)
		{
			$select = 'select COUNT(DISTINCT(si.item_id)) AS total ';
		}
		else
		{
			$select = 'select DISTINCT(si.item_id),si.username,si.item_title,si.item_description,'.
						'si.item_type_id,si.item_subtype_id,si.item_type_version,'.
						'si.item_original_filename,si.item_created,'.
						'si.item_status ';
		}
		
		$from = 'sh_item_pool as si ';	
		$query = $select.' FROM '.$from.' WHERE 1 ';
		
		if ($params->have(FILTER_USER))
		{
			$requestedUsers = $params->find_value(FILTER_USER);
			$query .= ' AND (si.username ="'.$requestedUsers[0].'") ';
		}
		

		// Filter by document status
		if ($params->have(FILTER_STATUS))
		{
			$requestedStatuses = $params->find_value(FILTER_STATUS);
			$statuses = array ();
			foreach ($requestedStatuses as $status)
			{
				$statuses[] = '(si.item_status = '.$status.') ';
			}
			$query .= 'AND ('.implode (' OR ',$statuses).')';
		}		
		

		// Filter by modification date
		if ($params->have(FILTER_MODIFY_DATE))
		{
			$dates = $params->find_value(FILTER_MODIFY_DATE);	
			if ($dates['start'])
			{
				$startDate = iwUtilities::databaseDateFormat ($dates['start']);
				$query .= 'AND (item_modified > '.$startDate.')';
			}
			if ($dates['end'])
			{
				$endDate = iwUtilities::databaseDateFormat ($dates['end']);
				$query .= 'AND (item_modified < '.$endDate.')';
			}
		}		
						
		if (!$countOnly)
		{
			$query .= $this->getOrderBy($params);
			$query .= ' '.$this->getOrderDirection($params);
			$query .= ' '.$this->getLimit ($params);
		}
		
		// Note, the FILTER_DOCTYPE has to be applied after the results are returned.		
		// Note, the FILTER_INBIN has to be applied after the results are returned.
        app()->flushCacheIfInvalid($query);
        
		return $query;
	}
	
	function loadFromQuery ($query)
	{
				
		$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($rows !== false)	
		{																	
			// Now load the data into the object.
			foreach ($rows as $row)
			{
				// Check the doctype filter
				$obj = new shPoolItem;
				$obj->setDBKeyValueArray ($row);
				$this->addItem ($obj);
			}
			
			return true;
		}				
		
		return false;
	}
	
	public function load (iwParameterCollection $params,$countOnly=false)
	{
        $this->_params = $params;
        
        if ($params->have(FILTER_SHARED))
        {
            $this->setItems($this->getDocumentsSharedWithUser(app()->getLoggedInUsername()));
            
            $temp = new shPoolItem;
            $this->setIcon ($temp->getItemIcon(true));
            $this->setTitle (str('STR_FILEBOX_SHARED_OTHERS'));                                        
        }        
        else
        {		    											    
		    $queryCount = $this->buildQuery($params, true);
		    $count = app()->getDatabase()->CacheGetOne(DB_CACHE_EXPIRE,$queryCount);
			$this->_total = $count;
                		    
		    $query = $this->buildQuery ($params, false);
		    $rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		    if ($rows !== false)	
		    {																	
			    // Now load the data into the object.
			    foreach ($rows as $row)
			    {
				    // Check the doctype filter
				    $obj = new shPoolItem;
				    $obj->setDBKeyValueArray ($row);
				    if ($params->have(FILTER_DOCTYPE))
				    {
					    $doctypes = $params->find_value(FILTER_DOCTYPE);
					    if (!in_array ($obj->getDocumentType(),$doctypes))
					    {
						    // Go on to the next without adding if the type is not
						    //	in the given array.
						    $this->_total--;
						    continue;
					    }
				    }
								
					array_push($this->_objects,$obj);				    
			    }				
			    
			    $title = 'FileBox';			
			    if (app()->isAdminLoggedIn())
			    {
				    if ($params->have(FILTER_USER))
				    {
					    
					    $users = $params->find_value(FILTER_USER);
					    $title .= '<span style="font-size:11px;font-weight:normal">&nbsp;'.sprintf(str('STR_FILEBOX_FILTER_USER_NAME'),$users[0]).'</span>';				
				    }
			    }
			    
			    $temp = new shPoolItem;
			    $this->setIcon ($temp->getItemIcon(true));
			    $this->setTitle ($title);				
			    
			    // Get the amount of space used and the amount of space available.
			    
			    $analysis = new shFileAnalysis;
			    $analysis->analyze($this);
						    
			    $size = iwUtilities::byteFormat($analysis->totalSizeInBytes);

				$maxsize = -1;
			    $user = app()->getLoggedInUserObject();
			    if ($user)
			    {
				    if (app()->isAdminLoggedIn())
				    {
					    $maxsize = 0;
				    }
				    else 
				    {
					    $maxsize = $user->get(PROP_ITEMPOOL_SIZE);
				    }
			    }
			    
			    $subtitle = '<div>'.sprintf(str('STR_FILEBOX_SPACE_DESC'),$count,$size,$maxsize?$maxsize.' MB': str('STR_NO_LIMIT')).'</div>';
			    
			    $this->setSubtitle ($subtitle);
			    
		    }
		    else {
			    $this->addErrorObject ($rows, ERROR_TYPE_SERIOUS);
		    }		
        }
	}
	
	public function processFilters ()
	{
				
		if (!isset ($_GET[URL_FILTERED]))
		{
			// Clear the filters and all remembered parameters
			//	if the 'filtered' param is not given.
			$this->storeParameters(new iwParameterCollection());
		}
		
		$params = $this->getStoredParameters();				
										
		// When the current user is not admin, then we only allow 
		//	the that person to view pool items that they uploaded.
		if (!app()->isAdminLoggedIn())
		{
			$user = app()->getLoggedInUserObject();
			$p = new iwParameterArray(FILTER_USER);
			$p->addItem_nokey ($user->get(PROP_USERNAME));
			$params->add ($p);
		}
		else
		{
			$user = @$_POST[FORM_FILTER_USER];				
			if ($user)
			{		
				$p = new iwParameterArray(FILTER_USER);
				$p->addItem_nokey ($user);
				$params->add ($p);	
			}		
			else
			{				
				$params = new iwParameterCollection();	
			}
		}
						
		$this->storeParameters ($params);		
				
		return $params;
	}
	
	function getFilterHtml ()
	{
				
		$html = '';

		if (app()->isAdminLoggedIn() && 
			!$this->_params->have(FILTER_SHARED))
		{
			$html .= $this->getUserFilterControl();
		}		
		
		return $html;
	}
				
 	function getFormButtons ()
 	{        
        if ($this->_params->have(FILTER_SHARED))
        {
            // User can't do anything if viewing shared filebox docs from other users.
            return '';
        }
        
 		$html = '
        <script type="text/javascript">function onDelete() {YAHOO.util.Dom.get(\'deleteSelectedFromPool\').value = "1";}</script>
        <input type="hidden" id="deleteSelectedFromPool" name="deleteSelectedFromPool" value=""/>
        <input type="submit" 
            name="submit1" 
            onclick="javascript:areYouSure(\'delete_selected\', \'\', \'objectCollectionForm\',onDelete); return false;" 
            value="Delete Selected"/>';

 		if (app()->isAdminLoggedIn())
 		{
	 		$html .= "<div style='padding-top:8px;'><div style='padding-bottom:3px;font-weight:bold;'>".str('STR_FILEBOX_QUICKADD').":</div>";
	 		$html .= '<input type="submit" name="addSelectedToLibrary" value="'.str('STR_ADD_TO_LIBRARY').'"/>'.'&nbsp;&gt;&nbsp;'; 		
	 		$html .= shCollection::getCompanyFilterControl(false).'&nbsp;&gt;&nbsp;';
	 		$html .= shCollection::getGroupFilterControl(0, false).'&nbsp;&gt;&nbsp;';
	 		$html .= shCollection::getCategoryFilterControl(false).'&nbsp;</div>';
 		} 		
 		
 		return $html;
 	}	

	/**
	 * Retrieves an array of shPoolItem objects that are owned by other users but 
	 * are being shared with the user given in $username.
	 * @return array An array of shPoolItem objects
	 * @param $username string A username
	 */
    protected function getDocumentsSharedWithUser($username)
    {
        $docs = array();
        
        $fileBoxShareDb = $GLOBALS['TABLE_POOL_ACCESS']['name'];
        $fileBoxDb = $GLOBALS['TABLE_ITEM_POOL']['name'];
        
        $query = "select $fileBoxDb.* from $fileBoxDb,$fileBoxShareDb 
                    where $fileBoxShareDb.pool_item_id = $fileBoxDb.item_id 
                    and $fileBoxShareDb.username = '$username'";
             
        app()->flushCacheIfInvalid($query);       
        $results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
        if ($results !== false)
        {
            $documents = array ();
            foreach ($results as $result)
            {
                $doc = new shPoolItem;
                $doc->setDBKeyValueArray($result);
                $docs[] = $doc;
            }        
        }    
        else
        {
            iwUtilities::logDatabaseError();
        }            

        return $docs;
    }
	
	function getClassName()
	{
		return 'shPoolItem';
	}

}
?>
