<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

function sortItemsByNameAsc ($item1, $item2)
{
	return strcasecmp ($item1->get(PROP_NAME),$item2->get(PROP_NAME));
}

function sortItemsByNameDesc ($item1, $item2)
{
	return -(strcasecmp ($item1->get(PROP_NAME),$item2->get(PROP_NAME)));
}

class shTrashCollection extends shCollection
{
	public function getCount()
	{
		$this->_load(new iwParameterCollection(), true);
		return $this->_total;
	}
	
    protected function flushCache()
    {            
        // Make sure we no longer retrieve old trash data.
        $queries = $this->getTrashQueries(false);
        foreach($queries as $query)
        {
            app()->flushCacheIfInvalid($query);
        }
        
        $queries = $this->getTrashQueries(true);
        foreach($queries as $query)
        {
            app()->flushCacheIfInvalid($query);
        }
                    
    }
    
	public function emptyTrash ()
	{
		$failures = array();
		
		$this->load (new iwParameterCollection());
		if ($this->_total > 0)
		{
			foreach ($this->items() as $item)
			{
				$item->enableTrueDeletion(true);
				if (!$item->delete ())
				{
					$failures[] = $item;
				}
			}

            $this->flushCache();
		}
		else
		{
			$this->addError (str('STR_NOITEMSTODELETE'), ERROR_TYPE_WARNING);
		}
				
		return (count($failures) == 0);
	}
    
	protected function getTrashQueries($countOnly)
    {        
        if ($countOnly)
        {
            $select = 'select COUNT(*) AS total';
        }
        else 
        {
            $select = 'select *';
        }
        
        $queries = array();
        
        $queries['shGroup'] = $select.' from sh_groups where status='.GROUP_STATUS_DELETED;
        $queries['shCompany'] = $select.' from sh_companies where status='.COMPANY_STATUS_DELETED;
        $queries['shItemDocument'] = $select.' from sh_items where item_status='.ITEM_STATUS_DELETED;
        $queries['shAttribute'] = $select.' from sh_attributes where status='.ATTRIBUTE_STATUS_DELETED;
        $queries['shCategory'] = $select.' from sh_categories where status='.CATEGORY_STATUS_DELETED;
        $queries['shUser'] = $select.' from iw_users where status='.USER_STATUS_DELETED;

        return $queries;        
    }
    
	protected function _load ($params, $countOnly)
	{	
		$this->_params = $params;			

        $queries = $this->getTrashQueries($countOnly);
        		
		foreach ($queries as $objName=>$query)
		{
            app()->flushCacheIfInvalid($query);
            
			$res = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
			if ($res !== false)	
			{																	
				if ($countOnly)
				{
					$this->_total = $res[0]['total'];					
				}
				else 
				{										
					// Now load the data into the object.
					foreach ($res as $row)
					{
						$ob = new $objName();
						$ob->setDBKeyValueArray ($row);
						$this->addItem ($ob);
					}
				}
			}
			else 
			{
				$this->addErrorObject ($res, ERROR_TYPE_SERIOUS);
			}
		}
		
		if (!$countOnly)
		{	
			if ($params->have(FILTER_SORTBY))
			{
				if ($params->find_value(FILTER_SORTBY) == 'name')
				{
					if ($params->have(FILTER_SORTDIR) && $params->find_value(FILTER_SORTDIR) == 'desc')
					{
						usort ($this->_objects, 'sortItemsByNameDesc');										
					}
					else
					{
						usort ($this->_objects, 'sortItemsByNameAsc');						
					}
				}
			}
														
			// Do any sorting or limiting
			$startIndex = 0;
			$perpage = count($this->items());
			if ($params->have(FILTER_START) && $params->have (FILTER_PERPAGE))
			{
				$startIndex = $params->find_value (FILTER_START);
				$perpage = $params->find_value(FILTER_PERPAGE);
				$this->_objects = array_slice ($this->_objects,$startIndex,$perpage);
			}						
		}
	}
	
	public function restoreTrash ()
	{
		$this->load (new iwParameterCollection());
		if ($this->_total > 0)
		{		
			foreach ($this->items() as $item)	
			{
				$item->restore();
			}			
			$this->addNotification(str('STR_TRASH_RESTORED'));
            
            $this->flushCache();
		}
		else
		{
			$this->addError(str('STR_NOITEMSTORESTORE'),ERROR_TYPE_WARNING);
		}
	}
	
	/**
	 * Retrieves the icon 
	 * @param bool $large True if the larger icon is to be retrieved.
	 * @return unknown
	 */
	function getItemIcon ($large=true)
	{
		return app()->getTemplateFile('images/trash_icon_large.jpg');
	}
		
	public function load (iwParameterCollection $params, $countOnly=false)
	{
		
		// Stores the number of trash items
		$this->_load($params, true);
				
		// Stores the items to display
		$this->_load($params, false);

		$this->setIcon (shTrashCollection::getItemIcon(true));
		$this->setTitle (str('STR_TRASH'));
		$this->setSubtitle (sprintf(str('STR_TRASH_TOTAL'),$this->count()));
	}
	
	public function processFilters ()
	{
		
		$params = $this->getStoredParameters();
		$this->storeParameters ($params);

		return $params;
	}

	function getFilterHtml ()
	{
		$html = '';
		return $html;
	}

	function getAvailableActions ($item)
	{
		$imageroot = app()->getTemplateFile('images/');
		
		$type_encoded = urlencode (base64_encode(get_class($item)));
		if (app()->isAdminLoggedIn())
		{
			$actions[] = 
			shPage::makeActionArray (str('STR_ACTION_RESTORE'), 
							 $imageroot.'restore_small.jpg',
							'index.php?a='.URLACTION_RESTORE.'&id='.$item->get(PROP_ID).'&t='.$type_encoded,
							'', 
							str('STR_ACTION_RESTORE'));
						
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_DELETE'), 
							 $imageroot.'delete_small.jpg',
							 'javascript:areYouSure(\'delete\', \'index.php?a='.URLACTION_DESTROY.'&'.URLARG_ID.'='.$item->get(PROP_ID).'&t='.$type_encoded.'\');',
							 '', 
							 str('STR_ACTION_DELETE'));
		}				
		
		return $actions;
	}
			
	/**
	 * Returns "shTrashCollection" since any type of object can be in this
	 * collection	 
	 */
	public function getClassName()
	{
		return 'shTrashCollection';
	}	
	
	function renderAsListItem ($item)
	{
		$img = $item->getItemIcon(false);						
		
		$description = iwUtilities::trucateWithEllipsis (strip_tags ($item->get(PROP_DESCRIPTION)), 125);
		if ($description == '')
		{
			$description = str('STR_NO_DESCRIPTION');
		}
				
		$smarty =  app()->getSmartyObject();
		$smarty->assign ('collection_item_id', $item->get(PROP_ID).','.get_class($item));
		$smarty->assign ('collection_item_name', $item->get(PROP_NAME));
		$smarty->assign ('collection_item_show_details', true);
		$smarty->assign ('collection_item_image', $img);
		$smarty->assign ('collection_item_selectable', $this->itemsAreSelectable());
		$smarty->assign ('collection_array_name', get_class ($this));
		$smarty->assign ('collection_item_specialinfo', ucwords($item->getTypeName(false)));
		$smarty->assign ('collection_item_description', $description);
		$smarty->assign ('collection_item_actions', $this->getAvailableActions($item));
		$smarty->assign ('collection_item_droptype', 'trash');
		$smarty->assign ('collection_item_viewlink', '');		
		$smarty->assign ('collection_item_draggable', false);
		$collection_item_html = smarty_templ_fetch('collections/collection.item.tpl.html',$smarty);
								
		return $collection_item_html;		
	}
	
 	function getFormButtons ()
 	{
 		if (app()->isAdminLoggedIn())
		{
			$smarty = app()->getSmartyObject();
			return smarty_templ_fetch('collections/collection.actions.trash.tpl.html',$smarty);
		}

		return $buttons;
 	}

	function queryChildItemForDisplay()
	{
		return false;
	}
}
?>
