<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shUserCollection extends shCollection
{
	function buildQuery (iwParameterCollection $params, $countOnly)
	{
		
		$select = '';
		$query = '';
		
		if ($countOnly)
		{
			$select = 'select COUNT(*) AS total ';
		}
		else
		{
			$select = 'select iw_users.*,sh_groups.group_id AS group_id,sh_groups.name AS group_name,' .
					  'sh_companies.company_id AS company_id, sh_companies.name AS company_name ';
		}
		
		// No user filtering
		$query = $select.
					  'FROM sh_companies,sh_groups,iw_users ' .
					  'INNER JOIN sh_user_access_map AS suam ON (suam.username = iw_users.username) '.
					  'WHERE iw_users.status <> '.USER_STATUS_DELETED.' AND sh_groups.group_id=suam.group_id AND sh_companies.company_id=suam.company_id ';
		
		
		// Filter by company
		if ($params->have(FILTER_COMPANY))
		{
			$requestedCompanies = $params->find_value(FILTER_COMPANY);
			$companies = array ();
			foreach ($requestedCompanies as $company)
			{
				if ($company!=0)
				{
					$companies[] = '(suam.company_id = '.$company.') ';
				}
			}
			
			if (count ($companies))
			{
				$query .= 'AND ('.implode (' OR ',$companies).')'; 
			}
		}
		
		if ($params->have(FILTER_GROUP))
		{
			$groups = array ();
			$requestedGroups = $params->find_value(FILTER_GROUP);
			foreach ($requestedGroups as $group)
			{
				if ($group!=0)
				{
					$groups[] = '(suam.group_id = '.$group.') ';
				}
			}
			
			if (count ($groups))
			{
				$query .= 'AND ('.implode (' OR ',$groups).')'; 
			}
		}
		
		if (!$countOnly)
		{
			$query .= $this->getOrderBy($params);
			$query .= ' '.$this->getOrderDirection($params);
		}		
		
        app()->flushCacheIfInvalid($query);
		return $query;
	}
	
	function load (iwParameterCollection $params,$countOnly=false)
	{
		$this->_params = $params;		
		$limit = $this->getLimit ($params);				
		$queryCount = $this->buildQuery ($params, true);
		$count = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$queryCount);
		if ($count == false)	
		{																	
			iwUtilities::logDatabaseError();
			return;			
		}
		$this->_total = $count[0]['total'];			
		
		$query = $this->buildQuery ($params, false);
		$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);		
		if ($rows !== false)	
		{			
			// Now load the data into the object.
			foreach ($rows as $row)
			{
				$user = new shUser;
				$user->setDBKeyValueArray ($row);
				
				// Get the company and group
				$group = new shGroup;
				$company = new shCompany;				
				$group->set(PROP_ID, $row['group_id']);
				$company->set(PROP_ID, $row['company_id']);
				$group->set(PROP_NAME, $row['group_name']);
				$company->set(PROP_NAME, $row['company_name']);
				
				$user->setDomain($company,$group);
				$this->addItem ($user);
			}
				
			$title = 'Users';			
			if ($params->have (FILTER_COMPANY))
			{
				$title .= '<span style="font-size:11px;font-weight:normal">&nbsp;'.str('STR_USER_FILTER_BYORG').'</span>';				
			}
			
			if ($params->have (FILTER_GROUP))
			{
				$title .= '<span style="font-size:11px;font-weight:normal">&nbsp;'.str('STR_USER_FILTER_BYGROUP').'</span>';
			}
				
			$this->setIcon (shUser::getItemIcon(true));
			$this->setTitle ($title);
			$this->setSubtitle (sprintf(str('STR_USER_TOTAL'),$this->count()));								
		}
		else {
			$this->addErrorObject ($rows, ERROR_TYPE_SERIOUS);
		}
	}    		
	
	function getAdminActions ()
	{
		return array ();
	}

	function getActions ()
	{
		return array ();
	}	
	
	
	/**
	 * Each line in the bulk registration input must look like this
	 * 	[username],[password],[first_name],[last_name],[email],[company],[group]
	 */	
	function bulkRegistration ($lines)
	{
				
		$companies = array ();
		$group = array ();
		$users = array ();
		
		$i = 0;
		foreach ($lines as $line)
		{
			$i++;			
			list ($username,$password,$firstname,$lastname,$email,$company,$group) = explode (',',$line);
			$username = trim ($username);
			$password = trim ($password);
			$firstname = trim ($firstname);
			$lastname = trim ($lastname);
			$email = trim ($email);
			$company = trim ($company);
			$group = trim ($group);
			
			if ($username == '') {
				$this->addError (sprintf(str('STR_USERBATCH_INVALID_USERNAME'),$i));
				continue;				
			}
			
			if ($password == '') {
				$password = 'verint';
			}
			
			$groupObj = null;
			if ($group == '') 
			{
				$this->addError (sprintf(str('STR_USERBATCH_INVALID_NOGROUP'),$i));
				continue;
			}
			else
			{
				// find the group with that name
				$query = 'select * from sh_groups where name LIKE "'.$group.'"';
                app()->flushCacheIfInvalid($query);
				$result = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
				if ($result !== false)	
				{																					
					if (count ($result) > 1)
					{
						// Don't do anything if there's more than one group with that name.
						$this->addError (sprintf(str('STR_USERBATCH_MULTIGROUP'),$group,$i), ERROR_TYPE_SERIOUS);
						continue;
					}	
					else if (count ($result) == 0)
					{						
						// Create a new group with the given name
						$groupObj = new shGroup;
						$groupObj->set(PROP_NAME, $group);
						$groupObj->set(PROP_DESCRIPTION, '['.str('STR_AUTOGEN_GROUP').']');
						
						if (!$groupObj->add ())
						{
							$this->addError (sprintf(str('STR_USERBATCH_DBERR_CREATEGROUP'),$group,$i), ERROR_TYPE_SERIOUS);
							continue;
						}
						
						$this->addNotification (sprintf(str('STR_USERBATCH_GROUPCREATED'),$group,$i));
					}
					else
					{
						$groupObj = new shGroup;
						if (!$groupObj->setDBKeyValueArray ($result[0]))
						{
							$this->addError (sprintf(str('STR_USERBATCH_GROUPNOTPOP'),$group,$i), ERROR_TYPE_SERIOUS);
							continue;
						}
					}
				}
				else
				{
					$this->addError (sprintf(str('STR_USERBATCH_DBERR_FINDGROUP'),$group,$i), ERROR_TYPE_SERIOUS);
					continue;
				}
			}
			
			$companyObj = null;
			if ($company == '') 
			{
				$this->addError (sprintf(str('STR_USERBATCH_NOCOMPANY'), $i), ERROR_TYPE_SERIOUS);
				continue;
			}
			else
			{
				// find the company with that name
				$query = 'select * from sh_companies where name LIKE "'.$company.'"';
                app()->flushCacheIfInvalid($query);
				$result = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
				if ($result !== false)
				{
					if (count ($result) > 1)
					{
						// Don't do anything if there's more than one company with that name.
						$this->addError (str('STR_ERR_MORE_THAN_ONE_COMPANY',$company,$i), ERROR_TYPE_SERIOUS);
						continue;
					}	
					else if (count ($result) == 0)
					{
						// Create a new company with the given name
						$companyObj = new shCompany;
						$companyObj->set(PROP_NAME, $company);
						$companyObj->set(PROP_DESCRIPTION, '['.str('STR_AUTOGEN_COMPANY').']');
						// Add the group
						$companyObj->addGroup ($groupObj);
						if (!$companyObj->add ())
						{
							$this->addError (sprintf(str('STR_USERBATCH_DBERR_CREATEORG'),$company, $i),ERROR_TYPE_SERIOUS);
							continue; 
						}
						
						$this->addNotification (sprintf(str('STR_USERBATCH_ORGCREATED'),$company, $i));
					}
					else 
					{
						// See if the group already belongs to this company
						$companyObj = new shCompany;
						if ($companyObj->setDBKeyValueArray ($result[0]))
						{
							if (!$companyObj->findGroup ($groupObj->get(PROP_ID)))
							{
								// Add the group
								$companyObj->addGroup ($groupObj);
								if (!$companyObj->update ())
								{
									$this->addError (sprintf(str('STR_USERBATCH_DBERR_UPDATEORG'),$company,$i,$group), ERROR_TYPE_SERIOUS);
									continue; 
								}
							}
						}						
					}
				}
				else
				{
					$this->addError (sprintf(str('STR_USERBATCH_DBERR_FINDORG'),$company,$i), ERROR_TYPE_SERIOUS);
					continue;
				}
			}

			// USER
			
			$userObj = new shUser;
			$userObj->set(PROP_USERNAME, $username);
			$userObj->set(PROP_PASSWORD, md5($password));
			$userObj->set(PROP_FIRST_NAME, $firstname);
			$userObj->set(PROP_LAST_NAME, $lastname);
			$userObj->set(PROP_EMAIL, $email);
			$userObj->setDomain($companyObj,$groupObj);

			if (!$userObj->newRegistration ())
			{
				$this->addError (sprintf(str('STR_USERBATCH_REGUSERFAILED'),$username,$i), ERROR_TYPE_SERIOUS);
				continue;
			}
								
			$this->addNotification (sprintf(str('STR_USERBATCH_ADDEDUSER'),$username,$i));
		}
		
		return true;
	}
	
	public function processFilters ()
	{
				
		if (!isset ($_GET[URL_FILTERED]))
		{
			// Clear the filters and all remembered parameters
			//	if the 'filtered' param is not given.
			$this->storeParameters(array());
		}
		
		$params = $this->getStoredParameters();				
				
		if (!($params instanceof iwParameterCollection))
		{
			$params = new iwParameterCollection();
		}
		
		$company = @$_POST[FORM_FILTER_COMPANY];				
		if ($company != '')
		{
			$p = $params->add(new iwParameterArray(FILTER_COMPANY));
			$p->addItem_nokey($company);
		}
		
		$group = @$_POST[FORM_FILTER_GROUP];				
		if ($group != '')
		{
			$p = $params->add(new iwParameterArray(FILTER_GROUP));
			$p->addItem_nokey($group);
		}
					
		$this->storeParameters ($params);		
		
		return $params;
	}
	
	function getFilterHtml ()
	{
		$html = '';
			
 		if ($this->_showFilter)
		{
			$html .= $this->getCompanyFilterControl();
			$html .= $this->getGroupFilterControl();
		}
		
		return $html;
	}	
	
 	function getFormButtons ()
 	{
 		if (app()->isAdminLoggedIn())
		{
 			$buttons = '<table cellpadding="2" cellspacing="0"><tr><td colspan="2"><div style="font-weight:bold;">'.str('STR_SELECTED_ITEMS').':</div></td></tr>';
			$buttons .= '<tr><td><input type="submit" name="delUser" value="'.str('STR_USER_SUBMIT_DELETE').'"/></td></tr>';
			$buttons .= '</table>';
 			return $buttons;
		}
 	}	

	function getClassName()
	{
		return 'shUser';
	}

	
	function itemsAreSelectable () 
	{
		return true;
	}
	
	function getSortableFields ()
	{
		
		return array (
				str('STR_USER_SORT_NAME')=>$GLOBALS['TABLE_USER']['fields']['USERNAME'],
				);
	}

	function toArray()
	{
		$final = array();
		foreach ($this->items() as $user)
		{
			$arr = $user->toArray();
			$final[] = $arr;
		}
		return $final;
	}	
}
?>
