<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/** The application object is a singleton that controls all the
	various contained objects, their interactions and their
	interfaces to the user.
*/
class iwApplication extends iwObject
{
	protected $_database = NULL;
	protected $_trail = array ();
	protected $_activePage = NULL;
    protected $_activePlugins = array();

	/**  Constructor */
	public function __construct () 
    {
		$this->addProperties ();
	}

	/**
	 * Called when unserialized
	 */
	public function __wakeup ()
	{
		// if the database is non-null, we have to try and reconnect;
		if ($this->_database)
		{
			$this->_database = NULL;
			$this->getDatabase();            
		}                
	}

	protected function addProperties ()
	{
		$this->_addProperty ('admin_email','');
		$this->_addProperty ('require_new_user_confirm',false);
		$this->_addProperty ('allow_duplicate_emails',false);
		$this->_addProperty (PROP_TEMPLATE_DIR,'');
		$this->_addProperty ('page_dir','');
        $this->_addProperty ('plugin_dir',IW_BASE_DIR.'plugins');
		$this->_addProperty ('trail',array());
		$this->_addProperty (PROP_LIBRARY_DIR,IW_BASE_DIR.'files/library');
		$this->_addProperty (PROP_FILEBOX_DIR,IW_BASE_DIR.'files/filebox');
		$this->_addProperty ('archive_dir_relative','cache/archives/');
		$this->_addProperty (PROP_TEMP_DIR, 'cache/temp');
		$this->_addProperty ('data_dir_relative', 'cache/data');
		$this->_addProperty ('user_obj',null);
		$this->_addProperty ('section', 0);
		$this->_addProperty ('location', '');
		$this->_addProperty ('debug', '');
		$this->_addProperty ('db_type', '');
		$this->_addProperty ('db_name', '');
		$this->_addProperty ('db_host', '');
		$this->_addProperty ('db_user', '');
		$this->_addProperty ('db_pass', '');
        $this->_addProperty ('use_smtp', SMTP_ENABLED);
        $this->_addProperty ('smtp_host', SMTP_HOST);
        $this->_addProperty ('smtp_port', SMTP_PORT);
        $this->_addProperty ('smtp_user', SMTP_USERNAME);
        $this->_addProperty ('smtp_pass', SMTP_PASSWORD);
		$this->_addProperty (PROP_TEMPORARY_UPLOAD, '');
		$this->_addProperty (PROP_TEMPORARY_UPLOAD_TYPE, '');

	}

	public function &getActivePage ()
	{
		return $this->_activePage;
	}

	/**
	 * Retrieves a references to the database object that is currently active.
	 *	If none is active, then a connection is established.
	 *	@return ADOConnection Returns reference to the database object or false if
	 *				if a connection could not be established.
	 */
	public function &getDatabase ()
	{
		if ($this->_database == NULL)
		{
            if (!defined('DB_TYPE'))
            {
                require_once ('config/dbconfig.php');
                $this->set ('db_type', DB_TYPE);
                $this->set ('db_name', DB_NAME);
                $this->set ('db_host', DB_HOST);
                $this->set ('db_user', DB_USER);
                $this->set ('db_pass', DB_PASS);                        
            }
                        
			$dbType = $this->get ('db_type');
			$this->_database = NewADOConnection($dbType);
            
            if (LOG_SQL)
            {
                $this->_database->fnExecute = 'onDatabaseActivity';
            }

			$dbName = $this->get ('db_name');
			$dbHost = $this->get ('db_host');
			$dbUser = $this->get ('db_user');
			$dbPass = $this->get ('db_pass');
            
			$connection_string = "$dbType://$dbUser:$dbPass@$dbHost/$dbName";

			$connected = $this->_database->Connect ($dbHost,$dbUser, $dbPass,$dbName);
			if (!$connected)
			{
				$this->addErrorObject ($connected,ERROR_TYPE_FATAL);
				return false;
			}

		}

		$GLOBALS['ADODB_CACHE_DIR'] = IW_BASE_DIR.'/cache/data/db';
		
		return $this->_database;
	}

	/**
	 * Redirects to the given location
	 * @param $url The URL to redirect to
	 */
	public function redirect ($url)
	{
		if ($url[0] == '?')
		{
			// Prepend full url.  Some servers will simply prepend the
			//	current uri if the redirect uri does not appear absolute.
			$url = iwUtilities::appendPathSlash(IW_BASE_URL).$url;
		}
		
		header ("Location: $url");
		exit();
	}

	/**
	 * Loads the last stored URL for reference.
	 */
	public function loadLocation ()
	{
		return $this->get('location');
	}

	/**
	 * Stores the current browser URl for future reference
	 */
	public function saveLocation ()
	{
		$this->set('location',$this->_getUrlWithParameters());
	}

	/**  Adds a global error that will be made visible to the user automatically
     *	@param PEARErr $errorObj The error object to add to the global list
     *	@param int $type The type of error this is
	*/
	public function addGlobalError ($errorObj, $type) {

		$this->addErrorObject ($errorObj, $type, false);
	}

	/**  Adds a global notification that will be made visible to the user automatically
     *	@param string $message The message to add
	*/
	public function addGlobalNotification ($message) {

		$this->addNotification ($message, false);
	}

	/**  This will return an array of error strings
     *	@return array An array of all the error strings queued up.
	*/
	public function getGlobalErrors () {

		$errorObjects = $this->getErrors ();
		$errors = array ();
		if (count ($errorObjects) > 0) {
			foreach ($errorObjects as $err) {

				$msg = $err->getMessage();
				array_push ($errors, $msg);
			}
		}
		return $errors;
	}

	/**  Called at the beginning of any root script to establish a logged in user
     *	This will check to see if this session recognizes any user as logged in
     *	and sets up the environment with that user as the logged in user.  It
     *	also stores that user's object in this application object.
	*/
	public function setupUserEnvironment () {

		$user = new iwUser;

		// Finds the user in the database that is currently active under this session.
		if ($user->setUserFromSessionID (session_id()))	{

			// Now compare the user's hash with the session hash (for double confirmation)
			if ($user->getUserHash () == @$_SESSION['USER_HASH']) {

				// Now set the user object to this user.
				$this->set ('user_obj', $user);
			}
		}
		else {

			// Make sure that there is no logged in user variables set.
			$this->logoutUser ();
		}
	}

	/**  Get the logged in user object
     *	@return SSUser Returns the logged in user object or NULL if there is no logged in user
	*/
	public function& getLoggedInUserObject () {

		$ret = $this->get('user_obj');
		return $ret;
	}

	/**  Logs in a new user with the given password
     *	@param $username The user's name as entered by the user attempting to login
     *	@param $password The password entered by the user trying to login
     * 	@param $userClass The user class to use for instantiation
     * 						(useful if you have derived from iwUser and want that to
     * 						be used to store information about the user)
     *	@return bool True, if logged in successfully
	*/
	public function loginUser ($username, $password, $userClass='iwUser') {

		$user = new $userClass;
		if ($user->setUser ($username))
		{
			// Store the logged in user in the session.
			$_SESSION['USER_HASH'] = $user->getUserHash ();

			if ($user->login ($password)) {

				$this->set ('user_obj', $user);
				$this->notify (EVENT_LOGGED_IN, $this->getLogInfo());
				return true;
			}
		}
		else {
			$this->addError ($this->loadString('STR_USERNAMENOTFOUND'), ERROR_TYPE_SERIOUS, false);
		}

		// Clear the user.
		$this->set ('user_obj', false);

		return false;
	}

	public function getLoggedInUsername ()
	{

		$user = $this->getLoggedInUserObject();
		if ($user)
		{
			return $user->get(PROP_USERNAME);
		}
		else
		{
			return '';
		}
	}

	public function getLogInfo ()
	{

		$user = $this->getLoggedInUserObject();
		if ($user)
		{
			$username = $user->get(PROP_USERNAME);
		}
		else
		{
			$username = 'Guest';
		}

		return array('type'=>iwUser::getType(),'id'=>$username,'username'=>$username);
	}

	/**  Logs out the logged in user
	*/
	public function logoutUser () {

		if ($this->isUserLoggedIn ()) 
		{
			$user = $this->get ('user_obj');
			
			// Tell everyone we're logging out.
			$this->notify (EVENT_LOGGED_OUT, $this->getLogInfo());

			// Update the db.
			$user->logout ();

			// Clear the user object just in case.
			$this->set ('user_obj', false);

			// Clears ALL session data.
			$_SESSION = array ();

			if (isset($_COOKIE[session_name()])) 
			{
    			setcookie(session_name(), '', time()-42000, '/');
			}

			session_destroy();
		}
	}

	/**  Determines if there's a user logged in.
     *	@return bool True, if logged in successfully
	*/
	public function isUserLoggedIn () {

		$user = $this->get ('user_obj');

		if ($this->get ('user_obj') && 
			(@$_SESSION['USER_HASH'] != '')) 
		{
			return true;
		}

		return false;
	}

	public function refreshLoggedInUser ()
	{
		// Reload the user object from the database
		$user = new shUser;
		$user->set(PROP_USERNAME, $this->getLoggedInUsername());
		if ($user->load())
		{
			$this->set('user_obj',$user);
		}
	}

	/**
	 * Returns true if the logged in user is a super administrator
	 *
	 * @return bool True if user is super admin
	 */
	public function isSuperAdminLoggedIn ()
	{
		$user = $this->getLoggedInUserObject();
		if ($user)
		{						
			if ($user->getPermissions()->getPermissionGroup() == USER_TYPE_ADMIN)
			{
				return true;
			}
		}

		return false;
	}
	
	/**
	 * This function was written originally when 
	 * there was only one type of administrator.  Now there
	 * are domain and group administrators as well as 
	 * super administrators.  This function now returns
	 * true only if the logged in user is a *SUPER* administrator.
	 * @return bool Returns true if the logged in user is a 
	 * 			super administrator.
	 */
	public function isAdminLoggedIn ()
	{
		return $this->isSuperAdminLoggedIn();
	}

	/**  
	 * Checks the user type of the logged in user against the given user type
     *	@param int $type The type of user to check against.
     *	@return bool True is returned if the user types are equal
	 */
	public function verifyUserType ($type) {

		if ($this->isUserLoggedIn ()) 
		{
			return $user->getPermissions()->getPermissionGroup() == $type;
		}

		return false;
	}

	/**  Use this to remember certain settings within a session
     *	This is useful for remembering form information WITHIN A SESSION.
     *	All this information will be lost when the user's session
     *	is terminated (when the browser is closed, for example).
     *	@param string $key This is the key you will use to retrieve the object later
     *	@param mixed $object A variable you need to store (a copy will be made)
	*/
	public function rememberValue ($key, $var) {

		if (!isset ($_SESSION['MEMORY'])) {
			$_SESSION['MEMORY'] = array ();
		}

		// Store a copy of this object in memory
		$_SESSION['MEMORY'][$key] = $var;
	}

	/**  Use this to retrieve a stored session variable
     *	@param string $key The key used to store the variable
     *	@return mixed The value of the stored variable.
	*/
	public function retrieveValue ($key) {

		if (isset ($_SESSION['MEMORY']) &&
		isset ($_SESSION['MEMORY'][$key])) {

			return $_SESSION['MEMORY'][$key];
		}

		return '';
	}

	/**
	* Constructs the current URL with parameters
	* @param exclusions array An array of parameter names that should be excluded from the list of attached URL parameters
	* @return string The current URL
	*/
	public function _getURLWithParameters ($exclusions=false)
	{
		$params = '';

		if (!$exclusions || !is_array ($exclusions))
		$exclusions = array();

		while (list ($key, $val) = each ($_GET))
		{
			$include = true;

			// Only add to the list if this is not one of the exclusions
			if ($exclusions)
			{
				foreach ($exclusions as $exclusion)
				{
					if ($exclusion[0] == '*')
					{
						// Search as substring
						if (is_numeric (strpos ($key,substr($exclusion,1))))
						{
							$include = false;
							break;
						}
					}
					else
					{
						// Search as literal.
						if (strcasecmp ($exclusion,$key)==0)
						{
							$include = false;
							break;
						}
					}
				}
			}

			if ($include)
			$params .= "$key=$val&";
		}

		if ($params != '')
		{
			$params = '?'.$params;
		}

		reset ($_GET);

		// Get rid of the trailing ampersand if there is one.
		if ($params != '')
		{
			if ($params[strlen ($params)-1] == '&')
			$params = substr ($params, 0, strlen ($params)-1);
		}

		$url = basename ($_SERVER['PHP_SELF']).$params;
		return $url;
	}

	/**  Redirects the user's browser to the last stored page.
     *	If there is no page stored then the user is redirected
     *	to the front page.
	*/
	public function redirectToLastLocation () {

		$url = $this->loadLocation ();
		if (!$url) {
			$url = 'index.php';
		}
		header ('Location:'.$url);        
		exit();
	}

	/**  This will add a mod record to the database indicating that the given object was modified by a user
     *	The function will take care of adding the more generic data
     *	like the user who modified it, the time and date, and other info
     *	@param SSTableObject $objectModified This must be a table class derived object
     *	@return mixed A copy of the mod record (SSMod) that was added or false if there was a problem
	*/
	public function addModifyRecord ($objectModified, $action) {

		if (!is_object ($objectModified)) {
			return false;
		}

		$type = $objectModified->getType();
		if ($type != 0) {

			$mod = new iwModRecord;
			$id = $objectModified->getUniqueID ();

			if ($id != 0) {

				$user = $this->getLoggedInUserObject ();
				$userName = '';

				// Modification ALWAYS requires a logged in user.
				if ($user) {
					$userName = $user->get ('username');

					$mod->set ('target_id', $id);
					$mod->set ('target_type', $type);
					$mod->set ('mod_date', time ());
					$mod->set ('mod_ip', $_SERVER['REMOTE_ADDR']);
					$mod->set ('client_info', isset ($_SERVER['HTTP_USER_AGENT'])?$_SERVER['HTTP_USER_AGENT']:'[none found]');
					$mod->set ('action', $action);
					$mod->set ('mod_data', serialize ($objectModified));

					if ($mod->add ()) {
						return $mod;
					}
				}
				else {
					$this->addCriticalError (sprintf (STR_13, $objectModified->getTypeName(false)));
				}
			}
			else {
				$this->addError (sprintf($this->loadString('STR_INVALIDOBJECTID'), $objectModified->getTypeName(false)),ERROR_TYPE_SERIOUS);
			}
		}
		return false;
	}

	/** Adds a message to the debug window when DEBUG flag is on.
	 *	@param string $text The text of the message to display
	 *  @param string $file The name/path of the file requesting the message
	 *  @param string $function The line number requesting the message
	 *  @param string $line The name of the function requesting the message
	 *  @param string $class The name of the class requesting the message
	 *  @access public
	 */
	public function addDebugMessage ($text, $file='', $function='', $line='', $class='')
	{
		if (IW_DEBUG)
		{
			$debug = $this->get ('debug');
			$debug .= '<table width="100%"><tr><td bgcolor="#CC0000"><font color="#FFFFFF"><b>';
			$debug .= 'File: '.basename ($file).' | Function: '.$function.' | Class: '.$class.' | Line: '.$line.'</b></td></tr>';
			$debug .= '<tr><td>'.$text.'</td></tr></table>';

			$this->set ('debug', $debug);
		}
	}

	/** Wraps up the necessary calls to send an email through the StoryStream engine
	 * 	@param string $from The person/email address from whom this email will appear to come
	 * 	@param string $to The email address of the person who will receive this message.
	 * 	@param string $subject The subject of the email
	 *	@param string $body The body of the email message
	 * 	@param array $attachments Paths to files that should be attached.
	 *	@return bool Returns true if the email was sent, false otherwise.
	 */
	public function sendMail ($from, $to, $subject, $body, $attachments=array()) {

		$mail = new PHPMailer();
		$mail->From = $from;
		$mail->FromName = "";

        $langPath = 'include/components/phpmailer/language/';
		$mail->SetLanguage ($this->getCurrentLanguage(), $langPath);

		if ($this->get('use_smtp'))
		{
			$mail->Mailer = "smtp";
			$mail->Host = $this->get('smtp_host');
			$mail->Port = $this->get('smtp_port');
			$mail->SMTPAuth = $this->get('smtp_user') != '';
			$mail->Username = $this->get('smtp_user');
			$mail->Password = $this->get('smtp_pass');
		}
		else
		{
			$mail->Mailer = "mail";
		}

		$mail->Body    = $body;
		$mail->Subject = $subject;
		$mail->AddAddress($to);

		foreach ($attachments as $filepath)
		{
			$mail->AddAttachment($filepath);
		}

		if ($mail->Send())
		{
			return true;
		}

		$this->addError (sprintf($this->loadString('STR_MAILERROR'),$mail->ErrorInfo), ERROR_TYPE_SERIOUS, false);

		return false;
	}

	/**  Sends a registration email to the email address stored
     *	This should only be called by newRegistration and newEmailAddress
     *	since both of these require that the email address be confirmed.
     *	@return bool True if the mail was sent successfully.
	*/
	public function sendRegistrationEmail ($user)
	{
		$from = $this->get ('admin_email');
		$to = $user->get ('email');
		$subject = 'Account Registration';
		$hash = $user->get ('hash');

		$crlf = "\r\n";
		$link = IW_BASE_URL.'/?a=confirm&hash='.$hash.'&email='.urlencode ($to);
		$msg = $this->loadTranslatableFile('confirm_registration.txt');
		$text = sprintf ($msg, settings()->company_name, $link);

		return $this->sendMail ($from, $to, $subject, $text, array());
	}

	/**
	 * Determines if the string table was already cached by looking for
	 * the cached file in the cache/data directory
	 * @return bool Returns true if cached.
	 */
	protected function isStringTableCached($stringTable='engine_translations')
	{
		return file_exists($this->getCurrentLanguageCachePath().$stringTable.'.php');
	}

    protected function getCurrentLanguageCachePath()
    {
        return $this->get('data_dir_relative').'/lang/'.$this->getCurrentLanguage().'/';
    }
    
    protected function prepareLanguageCacheDirectory()
    {
        $cacheRoot = $this->get('data_dir_relative');
        $langRoot = $cacheRoot.'/lang/';
        if (!file_exists($langRoot))
        {
            if (!@mkdir($langRoot))
            {
                return false;
            }
        }
        $langPath = $langRoot.$this->getCurrentLanguage().'/';
        if (!file_exists($langPath))
        {
            if (!@mkdir($langPath))
            {
                return false;
            }
        }
        
        return true;
    }
    
    /**
    * @desc Retrieves the currently active language code
    * @return string The language code (e.g. 'es' or 'en')
    */
    public function getCurrentLanguage()
    {
        $lang = DEFAULT_LANGUAGE;
        nx_fire_event('EVENT_FILE_LANGUAGE_NEEDED',array('lang'=>&$lang));
        return $lang;
    }
    
    /**
    * @desc Used when non-english string resource is not available.  This is
    * slow and the translation file should be updated to contain this missing
    * resource as soon as possible
    * @param string $stringTable The name of the string resource
    * @param string $stringCode The string resource to translate
    * @return Returns the string.
    */
    public function getEnglishTranslation($stringTableName,$stringCode)
    {
        if ($this->getCurrentLanguage() != 'en')
        {
            $templateFile = 'text/lang/en/'.$stringTableName.'.txt';
            $path = $this->getTemplateFile($templateFile);
            
            if (file_exists($path))
            {
                $strings = parse_ini_file($path,false);
                if (isset($strings[$stringCode]))
                {
                    return $strings[$stringCode];
                }
                else
                {
                    return '[StringTable '.$stringTableName.': String '.$stringCode.' not found]';
                }
            }
        }
        else
        {
            return '[StringTable '.$stringTableName.': String '.$stringCode.' not found]';
        }
        
        return '[StringTable '.$stringTableName.' not found]';
    }
    
	/**
	 * Generates a new cache file in the cache/data directory that 
	 * contains all the string in the given string table file fore the 
	 * current language (default is 'en')
	 */
	public function cacheStringTable($stringTableName='engine_translations',$lang='')
	{
        if ($lang=='')
        {
            $lang = $this->getCurrentLanguage();
        }
        
        $templateFile = 'text/lang/'.$lang.'/'.$stringTableName.'.txt';
		$path = $this->getTemplateFile($templateFile);
        
        if (!file_exists($path) && ($lang != 'en'))
        {
            $templateFile = 'text/lang/en/'.$stringTableName.'.txt';
            $path = $this->getTemplateFile($templateFile);
        }
        
		if (file_exists($path))
		{
			$values = parse_ini_file($path);

            if (!$this->prepareLanguageCacheDirectory())
            {
                $this->addError("Unable to create string cache table. Translations will not be available");
                return false;
            }
            
            $cacheFile = $this->getCurrentLanguageCachePath().$stringTableName.'.php';
            
			$fp = fopen($cacheFile, 'w+');
			fwrite($fp,"<?php\r\n");
			fwrite($fp,"// Generated on ".strftime("%c")."\r\n");
			fwrite($fp,'$GLOBALS["'.$stringTableName.'"] = array ('."\r\n");
			foreach ($values as $key=>$value)
			{
				// Do replacement of special characters/strings
				$value = str_replace('{q}','\\"',$value);

				// Now write this string to the file.
				fwrite ($fp,"'$key'=>\"$value\",\r\n");
			}
			fwrite($fp,"'filler'=>'filler'\r\n");
			fwrite($fp,");\r\n");
			fwrite($fp,"?>\r\n");
			fclose($fp);
		}
	}

	/**
	 * Loads the given string from the string table. 
     * @param string $stringName The string code to load
     * @param string $stringTableName The name of the string table (should
     *              match the name of the translation source file).
	 * @return string The string translation
	 */
	public function loadString($stringName,$stringTableName='engine_translations')
	{
		if (!$this->isStringTableCached($stringTableName))
		{
			$this->cacheStringTable($stringTableName);
		}

		$cacheFile = $this->getCurrentLanguageCachePath().$stringTableName.'.php';
		include($cacheFile);

		if (isset($GLOBALS[$stringTableName][$stringName]))
		{
			return $GLOBALS[$stringTableName][$stringName];
		}
		else
		{
			// Maybe the code has been updated so let's clear
			//	the cache and try again.
			$this->cacheStringTable($stringTableName);

			// Now reexecute the chached file to see if it's there.
			include($cacheFile);

			if (isset($GLOBALS[$stringTableName][$stringName]))
			{
				return $GLOBALS[$stringTableName][$stringName];
			}
			else
			{
                // If english not current language, this will check the english string table
                //  to see if it can be found there so that at least there's some text 
                //  available.
                return $this->getEnglishTranslation($stringTableName,$stringName);
			}
		}
	}

	/**
	 * This will load the given file from the appropriate language 
	 * source and return its contents as a string.
	 * @param string $filename The name of the file (no path)
	 * @return string The contents of the file or a static english error
	 * string upon failure. 
	 */
	public function loadTranslatableFile($filename)
	{
		$path = $this->getTemplateFile('text/lang/'.$this->getCurrentLanguage().'/'.$filename);
		if (file_exists($path))
		{
			return file_get_contents($path);
		}

		return '[Could not find translation file]';
	}

	/**
	 * This determines if the deployment is secure - meaning
	 * that the install directory has been removed/renamed
	 * and that directories have been protected.
	 * @param array $errors On output contains an array of error strings
	 * @return bool Returns true if the security check passed, false otherwise. 
	 */
	public function securityCheck(&$errors)
	{
		// Check to see if the install directory is there.
		if (file_exists('include/install') && !IW_DEBUG)
		{                    
			$errors[] = str('STR_SECURITY_DEL_INSTALL_DIR');
		}        
        // Check to see that the config files are not set to a permission of 777
        $permission = iwUtilities::getFilePermissions('config/dbconfig.php');
        if ($permission == '0777')
        {
            $errors[] = str('STR_SECURITY_DEL_CONFIG_FILE_UNPROTECTED');
        }
        $permission =  iwUtilities::getFilePermissions('config/share.config.php');
        if ($permission == '0777')
        {
            $errors[] = str('STR_SECURITY_DEL_CONFIG_FILE_UNPROTECTED');
        }
		
		return (count($errors) == 0);
	}
	
	public function showPage ($page_name)
	{
        if (defined('LOG_SQL') && LOG_SQL)
        {
            db()->LogSQL(true);
        }
        else
        {
            db()->LogSQL(false);
        }
        
        if (defined('IW_DEBUG') && IW_DEBUG)
        {
            define('ADODB_OUTP','outputQuery');
            db()->debug = 1;
        }
        
		$pageFile = $page_name.'.page.php';
		include ($this->get('page_dir').'/'.$pageFile);
		
		if (class_exists ($page_name))
		{            
			$this->_activePage = new $page_name;
			$this->_activePage->setArguments ($_GET);
			$this->_activePage->setFormArguments ($_POST);

			$template = $this->_activePage->getTemplate ();
            
            $templateFile = '';
            if ($template)
            {
			    $templateFile = $template.'.tpl.html';
            }

            $smarty = $this->getSmartyObject ();
            $errors = array();        
            $this->clearTrail();
                        
            if (!$this->securityCheck($errors))
            {                
                $smartyContent = $this->getSmartyObject ();
                $smartyContent->assign('errors',$errors);
                $smartyContent->assign('title',str('STR_SECURITY_ERROR'));
                $smartyContent->assign('subtitle',str('STR_SECURITY_ERROR_DESC'));
                $this->pushBreadcrumb ('Security Problems','');
                                
                $content = smarty_templ_fetch('fatal.tpl.html',$smartyContent);
                $smarty->assign('content',$content);
                $smarty->assign(SMARTY_PAGETRAIL, $this->getTrailHtml());
            }
            else
            {            
			    $this->pushBreadcrumb ('Home','index.php');                
			    $this->_activePage->display ();                
            }
            
            foreach ($this->_activePage->getSmartyVariables() as $dataName=>$dataValue)
            {
                $smarty->assign ($dataName, $dataValue);
            }
            
            $smarty->assign('template_path', $this->get (PROP_TEMPLATE_DIR));
            
            if ($templateFile)
            {                
			    smarty_templ_display($templateFile,$smarty);
            }
		}
		else
		{
			$this->addError (sprintf($this->loadString('STR_PAGECOULDNOTBEFOUND'),$page_name), ERROR_TYPE_SERIOUS);
		}
		
		$execTime = stopTimer();
		
		if (defined('EXEC_TIME') && EXEC_TIME)
		{
			e('Execution time: '.$execTime);
		}
	}

	public function getSmartyObject ()
	{
		$smarty = new shSmarty;
		$smarty->initializeSmarty ();
		$smarty->template_dir = $this->get(PROP_TEMPLATE_DIR);		
		$smarty->register_function('includealt','smarty_function_includealt');
		$smarty->register_function('templ_path','smarty_function_templ_path'); 		
		$smarty->register_function('getform','smarty_function_getform');
        $smarty->register_function('byteformat','smarty_function_byte_format');
        $smarty->register_function('str','smarty_function_str');
		$smarty->register_function('public_documents','smarty_function_public_documents');
        $smarty->register_function('navbar','smarty_function_navbar');        
		
		$smarty->assign('root',$smarty->template_dir);
		$smarty->assign (SMARTY_COMPANY_NAME, settings()->company_name);
		$smarty->assign (SMARTY_SITE_TAGLINE, settings()->site_tagline);
		        
		$smarty->assignPermissions();
        
		return $smarty;
	}
    
	public function notify ($eventName, $eventProperties)
	{
		$mod = new iwModRecord;
		$mod->set('target_type',$eventProperties['type']);
		$mod->set('target_id',$eventProperties['id']);
		$mod->set(PROP_USERNAME,$eventProperties['username']);
		$mod->set('target_action',$eventName);
		$mod->set('mod_date', iwUtilities::databaseDateFormat(time ()));
		$mod->set('mod_ip', $_SERVER['REMOTE_ADDR']);
		$mod->set('client_info', @$_SERVER['HTTP_USER_AGENT'] == '' ? 'none given' : $_SERVER['HTTP_USER_AGENT']);
		$mod->set('action',$eventName);

		switch ($eventName)
		{
			case EVENT_OBJECT_ADDED:
				if ($eventProperties['type'] != OBJECT_TYPE_LOG)
                {
				    $mod->add ();
                }
				break;
			case EVENT_OBJECT_EDITED:
				if ($eventProperties['type'] != OBJECT_TYPE_LOG)
                {
				    $mod->add ();
                }
				break;
			case EVENT_OBJECT_DELETED:
				if ($eventProperties['type'] != OBJECT_TYPE_LOG)
                {
				    $mod->add ();
                }
				break;
			case EVENT_OBJECT_TRASHED:
				if ($eventProperties['type'] != OBJECT_TYPE_LOG)
                {
				    $mod->add ();
                }
				break;
			case EVENT_OBJECT_VIEWED:
				$mod->add ();
				break;
			case EVENT_OBJECT_DOWNLOADED:
				$mod->add ();
				break;
			case EVENT_LOGGED_OUT:
				$mod->add ();
				break;
			case EVENT_LOGGED_IN:
				$mod->add ();
				break;
			default:
				$this->addError (sprintf($this->loadString('STR_UNKNOWNEVENT'), $eventName), ERROR_TYPE_SERIOUS);
				break;
		}
	}

	/**
	 * This retrieves the full path to the template
	 * file specifed. The given file should have the relative path
	 * from the root of a theme directory - e.g. (images/myfile.jpg)
	 * @param string $file The relative path to the file to get the theme path
	 * 				to
	 * @return string Returns the path from the site root to the theme file
	 * 			(e.g. themes/default/images/myfile.jpg if images/myfile.jpg is
	 * 				is specified in $file)
	 */
	public function getTemplateFile($file)
	{
		return smarty_templ_file_path($file);	
	}
	
	public function getTrailHtml ()
	{
		$html = '<span class="trail">';
		$index = 0;
		foreach ($this->_trail as $breadcrumb)
		{
			$html .= '<span class="breadcrumb">';

			if ($index > 0)
			{
				$html .= '<span class="trailSeparator">&nbsp;»&nbsp;</span>';
			}

			if ($breadcrumb['link'] && ($index != (count($this->_trail)-1)))
			{
				$html .= "<a href=\"".$breadcrumb['link']."\">".$breadcrumb['name'].'</a>';
			}
			else
			{
				$html .= $breadcrumb['name'];
			}

			$html .= '</span>';
			$index++;
		}
		$html .= '</span>';

		return $html;
	}

	public function clearTrail ()
	{
		$this->_trail = array ();
	}

	public function pushBreadcrumb ($name,$link='')
	{
		array_push($this->_trail,array ('name'=>$name,'link'=>$link));
	}

	public function getTemporaryUploadType()
	{
		return $this->get(PROP_TEMPORARY_UPLOAD_TYPE);
	}

	public function storeTemporaryUploadType ($mimeType)
	{
		$this->set(PROP_TEMPORARY_UPLOAD_TYPE, $mimeType);
	}

	public function getTemporaryUploadPath()
	{
		return $this->get(PROP_TEMPORARY_UPLOAD);
	}

	public function storeTemporaryUploadPath ($path)
	{
		$this->set(PROP_TEMPORARY_UPLOAD, $path);
	}
	
	public function loadNamedValue($name)
	{
		if (isset($_SESSION[$name]))
		{
			return $_SESSION[$name];
		}
		
		return '<not found>';
	}
	
	public function saveNamedValue($name,$value)
	{
		$_SESSION[$name] = $value;		
	}
    
    /**
    * @desc Retrieves all the plugins that are installed on the system
    *       regardless of their activation status.
    * @return array An array of associative arrays in this format:
    *           name => The name of the plugin
    *           path => The full path to the plugin
    */
    public function getPlugins()
    {
        $dbPlugins = $this->getPluginsInDatabase();
        
        $pluginArray = array();
        $pluginDir = $this->plugin_dir;        
        $dh = opendir($pluginDir);

        while (($file = readdir($dh)) !== false) 
        {            
            $fullPath = $pluginDir . '/' . $file;
            if (is_dir($fullPath) && $file != '.' && $file != '..')
            {
                // plugin folder?  look for plugin main file.
                $dh2 = opendir($fullPath);
                while (($file2 = readdir($dh2)) !== false)
                {
                    $fullPath2 = $fullPath .'/'.$file2;
                    if (!is_dir($fullPath2))
                    {
                        if (file_exists($fullPath2))
                        {
                            $matches = array();
                            preg_match('/plugin\.(.*)\.php/',$file2,$matches);
                            if (count($matches) == 2)
                            {
                                // See if there's an xml file for this plugin
                                $xmlFile = $matches[1].'.xml';
                                $xmlFilePath = $fullPath.'/'.$xmlFile;
                                
                                $version = '[Unknown]';
                                $name = $matches[1];
                                $author = '[Unknown]';
                                $description = '[Uknown]';
                                $id = $matches[1];
                                
                                if (file_exists($xmlFilePath))
                                {
                                    // Get plugin info
                                    $xml = new SimpleXMLElement(file_get_contents($xmlFilePath));
                                    $version = $xml['version'];
                                    $name = $xml['name'];
                                    $author = $xml['author'];    
                                    $description = $xml->description;                                                                         
                                    $id = $xml['id'] ? $xml['id'] : $matches[1];
                                }    
                                
                                $active = $this->isPluginActive($id,$dbPlugins);
                                if ($active == -1)
                                {
                                    $this->updatePluginInDatabase($id, false);                            
                                    $active = 0;
                                }
                                														
                                $pluginArray[] = array(
                                    'name'=>(string)$name,
									'version'=>(string)$version,
									'author'=>(string)$author,
									'description'=>(string)$description,
                                    'path'=>(string)$fullPath2,
                                    'id'=>(string)$id,
                                    'active'=>$active);
                            }
                        }
                    }                    
                }
            }
        }
        
        return $pluginArray;
    }
    
    /**
    * @desc Adds/Updates a plugin in the database
    * @param string $plugin The name of the plugin
    * @param bool $isActive 1 for active, 0 for inactive.
    */
    public function updatePluginInDatabase($plugin, $isActive)
    {
		$plugins = $this->getPluginsInDatabase();
        $active = $this->isPluginActive($plugin,$plugins);
        if ($active == -1)
        {
            $query = "insert into sh_plugins (name,active) VALUES ('$plugin','$isActive');";
            $this->getDatabase()->Execute($query);
        }
        else if ($active != $isActive)
        {
            $query = "update sh_plugins set active=$isActive where name='$plugin'";
            $this->getDatabase()->Execute($query);
        }
    }
    
    /**
    * @desc Determines if the given plugin is marked as active
    *          in the given plugin array.  The plugin array given is
    *           what is returned by getPluginsInDatabase
    * @param string $plugin The name of the plugin
    * @param array $pluginArray The array of plugins returned by getPluginsInDatabase
    * @return int Returns 1 if plugin is active, 0 if not active and -1 if not found.
    */
    public function isPluginActive($plugin,$pluginArray)
    {
        if (is_array($pluginArray))
        {
            foreach ($pluginArray as $pluginIt)
            {
                if ($pluginIt['name'] == $plugin)
                {
                    return $pluginIt['active']; 
                }
            }
        }
        
        return -1;
    }
    
    /**
    * @desc Gets an array of details about a plugin as stored in
    *       the database.  Not dependant on the existance of the 
    *       plugin on disk.
    * @return array An array of associative arrays that look like this:
    *          name => The name of hte plugin
    *          active => 1 if active, 0 otherwise.
    */
    public function getPluginsInDatabase()
    {
        $plugins = array();
        $query = 'select name,active from sh_plugins';
        $rs = $this->getDatabase()->Execute($query);

        if ($rs !== false)
        {
		    while ($array = $rs->FetchRow()) 
		    {
			    $plugins[] = $array;
		    }
            
            return $plugins;
        }
        else
        {
            $this->addError('The plugins database could not be found.',ERROR_TYPE_SERIOUS);
        }
    }
    
    /**
    * @desc Looks for plugins in the plugins directory
    */
    public function registerPlugins()
    {
        $pluginDir = $this->plugin_dir;
        $dh = opendir($pluginDir);
        
        $plugins = $this->getPlugins();
        
        foreach ($plugins as $plugin)
        {                        
            if ($plugin['active'] && file_exists($plugin['path']))
            {            	             
			    include_once($plugin['path']);                 
            }
        }
        
        $results = nx_fire_event('EVENT_PLUGIN_INIT',array());
    }
    
    /**
     * Marks cached values for a table as invalid so that the next time
     * data is retrieved, the cache will first be flushed for that particular
     * query.
     * @param string $tablename The name of the table that is being marked as invalid.  Any
     *          queries that include this table name will be flushed in the cache.
     */
    public function invalidateTable($tablename)
    {
        $invalidationTable = $this->getInvalidationTable();
        if (is_array($invalidationTable))
        {
            $invalidationTable[strtolower($tablename)] = '1';
            $this->writeInvalidationTable($invalidationTable);
        }        
    }
    
    /**
     * Writes out the invalidation table as given in the associated
     * array. 
     * @param $invalidationTable array The associatve array of table=>invalidate(0|1)
     */
    protected function writeInvalidationTable($invalidationTable)
    {
        $fp = fopen($this->data_dir_relative.'/invalidation.ini','w+');
        
        foreach($invalidationTable as $table=>$value)
        {
            fwrite($fp,$table.'='.($value?'1':'0')."\n");
        }
        
        fclose($fp);
    }
    
    /**
    * @desc If the given query has a table name that is marked as "to be invalidated"
    * in the stored invalidation table, it will flushed in the cache and the table will
    * be marked as "invalidated" or (0).  
    * @param string $query The query to check
    */
    public function flushCacheIfInvalid($query)
    {
        $inv = $this->getInvalidationTable();
        foreach($inv as $key=>$value)
        {
            if (stripos($query,$key) !== false)
            {
                $this->getDatabase()->CacheFlush($query);
                $this->markTableFlushed($key);
                break;
            }
        }
    }
    
    /**
    * @desc Returns an associative array of table > inval (0 for no flush necessary and
    *   1 for must flush.
    * @return array Returns the associative array described above.
    */
    public function getInvalidationTable()
    {
        $file = $this->data_dir_relative.'/invalidation.ini';
        if (file_exists($file))
        {
            return parse_ini_file($file,false);        
        }
        else
        {
            return array();
        }
    }
    
    /**
     * Mark a table as flushed (0) in the invalidation table.
     * @param string $tablename The name of the table to mark
     */
    public function markTableFlushed($tablename)
    {
        $invalidationTable = $this->getInvalidationTable();
        if (is_array($invalidationTable))
        {
            $invalidationTable[strtolower($tablename)] = '0';
            $this->writeInvalidationTable($invalidationTable);
        }        
    }
}

/**
 * Called by ADODB engine everytime a SQL query is executed
 */
function onDatabaseActivity ($db, $sql, $en)
{
    if (!isset($GLOBALS['db_debug']))
    {
        $GLOBALS['db_debug'] = array('sql'=>array());
    }
    
    $GLOBALS['db_debug']['sql'][] = array('query'=>$sql, 'trace'=>iwUtilities::getTrace());
}

?>