<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**  Represents a single modification to the database
	Every time a scene, story or fork is edited, a new 
	modification record is added to the database.  It helps
	trace a story's history.
*/
class iwModRecord extends iwDbTable
{
	/** The user object associated with this record (not necessarily a valid object) */
	public $user = null;
	
	/** Constructor: Adds required properties*/
	function __construct () 
	{
		parent::__construct();
	}	
	
	/** Adds all associated properties
     *	This need only be called once per instantiation of this class
     *	and is handled automatically by the base class as long
     *	as its constructor is called.		
	*/
	function addProperties () {
		$this->_addProperty (PROP_ID, 0);
		$this->_addProperty ('target_type', '');
		$this->_addProperty ('target_id', '');
		$this->_addProperty (PROP_USERNAME,'');
		$this->_addProperty ('mod_date', 0);
		$this->_addProperty ('mod_ip', 0);
		$this->_addProperty ('action', 0);
		$this->_addProperty ('mod_data', 0);
		$this->_addProperty ('client_info', '');
	}
	
	/**  Gets all the database field names with associated values
     *	This will return an associative array where the keys
     *	are the table field names and the values are the values
     *	stored in this class.
     *	@param bool $includeDBKey If true, then the key field for the table is returned in the array as well.
     *	@return array An associative array of dbase fields => class values
	*/
	function getDBKeyValueArray ($includeDBKey) {
	
		$tableConstant = $this->getTableConstant();
				
		$fields = array
				(
					$GLOBALS[$tableConstant]['fields']['TARGET_TYPE'] => $this->get ('target_type'),
					$GLOBALS[$tableConstant]['fields']['TARGET_ID'] => $this->get ('target_id'),
					$GLOBALS[$tableConstant]['fields']['USER_ID'] => $this->get (PROP_USERNAME),
					$GLOBALS[$tableConstant]['fields']['MOD_DATE'] => $this->get ('mod_date'),
					$GLOBALS[$tableConstant]['fields']['MOD_IP'] => $this->get ('mod_ip'),
					$GLOBALS[$tableConstant]['fields']['ACTION'] => $this->get ('action'),
					$GLOBALS[$tableConstant]['fields']['MOD_DATA'] => $this->get ('mod_data'),
					$GLOBALS[$tableConstant]['fields']['CLIENT_INFO'] => $this->get ('client_info'),
				);
				
		if ($includeDBKey) {
			$fields [$GLOBALS[$tableConstant]['fields']['ID']] = $this->get (PROP_ID);
		}
		
		return $fields;
	}

	/**  Sets all the object properties based on the given database field values
     *	Given is an associative array where the keys are the database field names
     *	and the values are the values of those fields.
     *	@param array $dbFieldsAndValues The associative array of database field names and values.
     *	@return bool True if all the required fields were found and copied over.
	*/
	function setDBKeyValueArray ($dbFieldsAndValues) {
			
		$tableConstant = $this->getTableConstant();
		$this->set ('target_type', @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['TARGET_TYPE']]);
		$this->set ('target_id', @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['TARGET_ID']]);
		$this->set (PROP_USERNAME, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['USER_ID']]);
		$this->set ('mod_date', @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['MOD_DATE']]);
		$this->set ('mod_ip', @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['MOD_IP']]);
		$this->set ('action', @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ACTION']]);
		$this->set ('mod_data', @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['MOD_DATA']]);
		$this->set ('client_info', @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['CLIENT_INFO']]);
		
		// If we have the user information, store that too.
		if (isset ($dbFieldsAndValues['first_name']))
		{
			$this->user = new shUser;
			if (!$this->user->setDBKeyValueArray($dbFieldsAndValues))
			{
				$this->user = null;
			}			
		}
		
		return true;
	}
		
	/**  Gets the associative array required to mark a record as deleted
     *	@param bool $checkKey If this is true, then the unique key field for the object is checked for validity as well
     *	@return array An associative (key>value) array populated with the keys/values required to mark the object as deleted
	*/
	protected function getDBKeyValueForDelete ($isRestoring=false) 
	{
			return array ();
	}

	
	/** Retrieves the classes friendly type name
	 * @param bool $asPlural True to return the plural version, false otherwise.
	 * @return string The name as a string
	 *  @access public
	 */
	function getTypeName ($asPlural) 
	{		
		return $asPlural ? 'logs' : 'log';
	}
	
	/** Returns an object type code of OBJECT_TYPE_UNKNOWN if unspecified
	 * @return int The object type value
	 * @access public
	 */
	function getType () 
	{
		return OBJECT_TYPE_LOG;
	}
	
	/**  Gets the key string for the associative array that contains field information for the table associated with this object.
     *	@see tables.inc.php
     *	@return string The key field string for the assocative array that contains field information for the table associated with this object.
	*/
	function getTableConstant () {
		return 'TABLE_MOD';
	}
	
	/**  Checks if the data stored in this object is valid.
     *	All this does is verify that there is valid data in the *required*
     *	fields.
     *	@param bool $checkKey If this is true, then the unique key field for the object is checked for validity as well
     *	@return bool True, if all required fields are valid.
	*/
	function requiredFieldsValid ($checkKey = false) 
	{
	
		$invalidField = false;
		
		if ($this->get('target_type') == '') {
			$this->addError (str('STR_MOD_INVALIDTARGET'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}
		if ($this->get('target_id') == '') {
			$this->addError (str('STR_MOD_INVALIDTARGET_ID'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}
		if ($this->get (PROP_USERNAME) == '') {
			$this->addError (str('STR_MOD_INVALID_USER'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}
		if ($this->get('mod_date') == '') {
			$this->addError (str('STR_MOD_INVALID_DATE'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}
		if ($this->get('action') == '') {
			$this->addError (str('STR_MOD_INVALID_ACTION'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}		
		
		if ($checkKey) {
			if ($this->get (PROP_ID) == '') {
				$this->addError (str('STR_MOD_NOKEY'), ERROR_TYPE_SERIOUS);
				$invalidField = true;
			}
		}
		
		return !$invalidField;
	}
	
	/**  Returns the unique ID for this object as stored in memory (NOT THE DATABASE)
     *	@param bool $fieldName If true, then this function will return the table's field name for the unique ID instead of the actual unique ID
     *	@return mixed The unique ID field for this object.  If an array is returned, then more than one field makes up a key.
	*/
	function getUniqueID ($fieldName=false) 
	{
	
		if ($fieldName) 
		{
			return $GLOBALS[$this->getTableConstant()]['fields']['ID'];
		}
		else 
		{
			return $this->get (PROP_ID);
		}
	}
	
	/**
	 * The basic mod object doesn't use any icon.  Users of this class can
	 * 	override to return a custom icon.
	 */
	public function getItemIcon ($large=true)
	{
		return '';
	}	
	
	public function renderAsListItem ($selectable, $details, $actions=true){return '';}
	public function render ($params=false){return '';}
}
?>