<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


define ('SORT_DATE', 132);

class iwObjectArraySort {

	public $sortType = SORT_REGULAR;
	
    public $property;
    public $objArr;
    public $direction;
    public $orderBy;

    /**
     *
     * This function sorts an array of objects in ascending or descending order based
     * on a string or numerical property, maintaining array index associations.
     * Final array is stored in the $objArr property.
     *
     * @param array $objArr: an array of objects to sort
     * @param string $orderBy: the string name of the property to group or sort by
     * @param int $d1: The direction to group or sort by. 0 == ascending, 1 == descending (optional)
     * @param string $sortBy: a string name of another property to sort on (optional)
     * @param int $d2: The direction to sort $sortBy. 0 == ascending, 1 == descending (optional)
     *
     **/
    function __construct($objArr, $orderBy, $d1 = 0, $sortBy = NULL, $d2 = 0, $sortType=SORT_REGULAR) 
    {
        if (!is_array($objArr)) {
            $this->error('first argument is not an array');
        }
        $this->property = $orderBy;
        $this->direction = $d1;
        $this->orderBy = $orderBy;
        $this->sortType = $sortType;
        
        uasort($objArr, array($this, "cmp"));
        	        
        $this->objArr = $objArr;
        
        if ($sortBy!==NULL) 
        {
            $holding = array();
            $this->property = $sortBy;
            $this->direction = $d2;
            foreach ($objArr as $key=>$val) {
                $holding[$val->{$orderBy}][$key] = $val;
            }
            $this->objArr = array();
            foreach ($holding as $key=>$val) {
                uasort($val, array($this, "cmp"));
                foreach($val as $k=>$v) {
                    $this->objArr[$k] = $v;
                }
            }
        }
    }

    /**
     * callback function required by sortObjArray() (private)
     **/
    function cmp ($a, $b) 
    {
        if (!is_object($a) || !is_object($b)) 
        {
            $this->error("array contains non-objects, must contain only consistant objects");
        }
        
        if ($this->direction == 1) 
        {
            $tmp = $a;
            $a = $b;
            $b = $tmp;
        }
        
        if ($this->sortType == SORT_REGULAR)
        {
	        if (is_numeric($a->get($this->property) && 
	        	is_numeric($b->get($this->property))))
	        {
	            if ($a->get($this->property) == $b->get($this->property)) return 0;
	            return ($a->get($this->property) < $b->get($this->property)) ? -1 : 1;
	        } 
	        else 
	        {
	            return strcasecmp($a->get($this->property), $b->get($this->property));
	        }
        }
        else 
        {
       	
			$aDate = strtotime ($a->get($this->property));
			$bDate = strtotime ($b->get($this->property));
			
			if (!$aDate || !$bDate)
			{
				return 0;
			}
			else 
			{
		        if ($aDate == $bDate) return 0;
		        return ($aDate < $bDate) ? -1 : 1;
			}        			        	
        }
    }

    /**
     * error handler (private)
     **/
    function error($msg) {
        $trace = debug_backtrace();
        $trace = $trace[count($trace)-1];
        echo "<pre>\n\n[FATAL] ".get_class($this)."::".$trace['function']."[".$trace['line']."] - ".$msg."\n\n";
        var_dump($trace);
        echo "</pre>";
        die();
    }
}

?>