<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class iwUtilities
{
	/**
	 * Print_r convenience static public function, which prints out <PRE> tags around
	 * the output of given array. 
	 * @param array	$var
	 */
	static public function pr($var)
	{
		echo "<div style=\"text-align:left\"><pre>";
		print_r($var);
		echo "</pre></div>";
	}

	static public function byteFormat($input, $dec=0)
	{
		$prefix_arr = array(" bytes", " KB", " MB", " GB", " TB");
		$value = round($input, $dec);
		$i=0;
		while ($value>1024)
		{
			$value /= 1024;
			$i++;
		}
		$return_str = round($value, $dec).$prefix_arr[$i];
		return $return_str;
	}

	static public function databaseDateFormat ($input)
	{
		$time = $input;
		if (is_string ($time))
		{
			$time = strtotime ($input);
		}

		return date ('Y-m-d H:i:s', $time);
	}

	static public function dateTimeFormat ($input, $longDate)
	{
		$time = $input;
		if (!is_numeric ($time))
		{
			if ($time != '')
			{
				$time = strtotime ($input);
			}
			else
			{
				return '';
			}
		}

		if ($longDate)
		    return date (settings()->time_date_format_long, $time);
		else
		    return date (settings()->time_date_format_short, $time);

	}

	static public function dateFormat ($input, $longDate)
	{
		$time = $input;
		if (!is_numeric($time))
		{
			if ($time != '')
			{
				$time = strtotime ($time);
			}
			else
			{
				return '';
			}
		}

		if ($longDate)
        {
		    return date (settings()->date_format_long, $time);
        }
		else
        {
		    return date (settings()->date_format_short, $time);
        }

	}

	static public function getValue ($array, $key)
	{
		if (isset ($array[$key]))
		{
			return $array[$key];
		}

		return '';
	}

	static public function dateFormatRss ($date)
	{
		if (is_string($date))
		{
			// Convert to time stamp
			$date = strtotime($date);
		}

		return date(DATE_ATOM,$date);
	}

	static public function getTag ($tagName, $value, $emptyIfNoValue)
	{
		if ($value != '' || !$emptyIfNoValue)
		{
			return sprintf ("<%s>%s</%s>\r\n", $tagName,$value,$tagName);
		}
	}

	static public function basenameNoExtension ($f)
	{
		$f = basename ($f);

		$ext = strrchr($f, '.');

		if($ext !== false)
		{
			$f = substr($f, 0, -strlen($ext));
		}

		return $f;
	}

	static public function getExtension ($f)
	{
		return substr($f, strrpos($f, '.')+1);
	}


	static public function logToFile ($msg,$file='log.txt')
	{
		$fp = fopen (IW_BASE_DIR.'/cache/logs/'.$file, 'a+');
		if ($fp)
		{
			fwrite ($fp,"<div class=\"logentry\">$msg</div>\r\n");
			fclose($fp);
		}
	}

	static public function logDatabaseError ()
	{
		$error = app()->getDatabase()->ErrorMsg();
		if ($error != '')
		{
			$message = sprintf('DBERR: %s',$error);
			iwUtilities::logToFile($message);
		}
	}

	static public function trucateWithEllipsis($s, $n)
	{
		for ( $x = 0; $x < strlen($s); $x++ ) {
			$o = ($n+$x >= strlen($s) ? $s : ($s{$n+$x} == " " ? substr($s,0,$n+$x) . "..." : ""));
			if ( $o != "" ) { return $o; }
		}
	}

	static public function escapeStringIfMagicQuotesOff ($string)
	{
		if (get_magic_quotes_gpc() == 0)
		{
			return addslashes ($string);
		}
		else
		{
			return $string;
		}
	}

	static public function unescapeString ($string)
	{
		return stripslashes ($string);
	}

	static public function makeStringJavascriptSafe($string)
	{
		$newstring = str_replace(array("\r","\n"), "", $string);
		return trim(strip_tags($newstring));
	}


	static public function getTrace ($html=false) 
    {
		if (PHPVERSION() >= 4.3)
		{
			$lineSkip = 2;
			$dbg = debug_backtrace ();
			$s = '';
			foreach ($dbg as $pt) {
				if ($lineSkip > 0)
				{
					// Skip the first couple of the trace because they reflect
					//	debugging lines.
					$lineSkip--;
					continue;
				}
					
				$file = isset ($pt['file'])?basename($pt['file']):'unknown';
				$line = (isset ($pt['line'])?$pt['line']:'unknown');
				@$s .= 'static public function: '.$pt['static public function'].' in '.($file).' (Line '.$line.')'.($html?'<br>':"\n");
			}

			return $s;
		}

		return '[Trace Not Supported]';
	}

	/** Displays a string that lists all the static public functions leading to the caller static public function 
	 */
	static public function traceback () {
		$s = iwUtilities::getTrace();
		echo '<pre>'.$s.'</pre>';
	}

	static public function phpIniSettingToBytes($setting)
	{
		$mul = substr($setting, -1);
		$mul = ($mul == 'M' ? 1048576 : ($mul == 'K' ? 1024 : ($mul == 'G' ? 1073741824 : 1)));
		return $mul*(int)$setting;
	}	
    
    /**
    * @desc Retrieves file permissions as an octal string with a leading zero
    * @param string $file The full path to the file.
    */
    static public function getFilePermissions($file)
    {
        if(!file_exists($file)) return false;
        $perms = fileperms($file);
        return substr(decoct($perms), 2);
    }
    
    /**
     * Appends a slash to the end of the path only if one 
     * is not already there.
     * @param string $path The original path
     * @return string The path with the slash appended
     */
    static public function appendPathSlash($path)
    {
    	if ($path[strlen($path)-1] != '/' && 
    		$path[strlen($path)-1] != '\\')
    	{
    		return $path.'/';			
    	}
    	
    	return $path;
    }
}

function l($message)
{
	iwUtilities::logToFile($message);	
}

function p($var)
{
	iwUtilities::pr($var);
}

function e($msg)
{
	echo '<div class="echo">'.$msg.'</div>';
}

function g($var)
{
	if (isset($_GET[$var]))
		return $_GET[$var];
	else
		return '';
}

function gorp($var)
{
    if (!isset($_GET[$var]))
    {
        return @$_POST[$var];
    }
    else
    {
        return $_GET[$var];
    }    
}

function ps($var)
{
	if (isset($_POST[$var]))
	{
		$value = $_POST[$var];
		
		$cleaner = new HTML_Safe();
		if (is_array($value))
		{
			foreach($value as $key=>$val)
			{
				$value[$key] = $cleaner->parse($val);
				$cleaner->clear();	
			}
			
			return $value;
		}
		else
		{
			return $cleaner->parse($value);
		}
	}
	else
	{
		return '';
	}
}

?>
