<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

abstract class shFileType
{
	/**
	 * Gets a thumbnail image for use in displaying files of this type
	 * @param string $documentPath The path to the document
	 * @param string $sizeWidth The width of the thumbnail to retrieve
	 * @param string $sizeHeight The height of the thumbnail to retrieve
	 * @return string The path to the thumbnail image.
	 */
	abstract public function getThumbnailPath(shItemDocument $doc, $sizeWidth, $sizeHeight);
	
	/**
	 * Retrieves the html used to display this file to the user.  Return an empty string
	 * to not display a preview.
	 * @param string $documentPath The path to the document on the disk
	 */
	abstract public function getPreview(shItemDocument $doc, $localPath);
	
	/**
	 * Retrieves the mime type for the given document.  If the mime type does not apply for
	 * this filetype then an empty string is returned.
	 * @param string $documentPath The path to the document
	 * @param array The array of possible mime types (e.g. application/pdf)
	 */
	abstract public function getMimeType($documentPath);
	
	/**
	 * Gets a description of the file type 
	 * @param string $documentPath The path to the document
	 * @return string A description of the document file type 
	 */
	abstract public function getDescription($documentPath='');
	
	/**
	 * Returns a url that gives users more information about the file type (for example, a 
	 * where a reader can be downloaded)
	 * @return string The url
	 */
	abstract public function getRelatedUrl();
	
	/**
	 *  Retrieves the filename extensions that files of this type can have as an 
	 *	array of strings (e.g. 'doc','docx',etc)
	 *	@return array An array of possible extensions.
	 */
	abstract public function getFilenameExtensions();
	
	/**
	 * Gets the associated Nexus file type.  This file type string
	 * must be unique and should be descriptive.
	 * @return string The nexus file type (one of the DOCTYPE_xxx defines)
	 */
	abstract public function getNexusFileType();
}

?>