<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFileTypeImage extends shFileType
{
	public function getThumbnailPath(shItemDocument $doc, $maxWidth, $maxHeight)
	{
        try
        {
            $path = storage()->retrieveDocument($doc);            
            $thumb = new iwImage($path);            
            $icon = $thumb->getThumbnail('',$maxWidth, $maxHeight,true);
            $thumb->destroy();            
            storage()->removeTempDocument($doc,$path);
            
            return $icon;        
        }
        catch(Exception $e)
        {
            app()->addError($e->getMessage());
        }        
        
        return '';
	}
		
	public function getPreview(shItemDocument $doc, $localPath)
	{
        $dim = unserialize ($doc->get(PROP_DATA));
        if (is_array ($dim) && $dim['width'] > 0 && $dim['height'] > 0)
        {
            // we have dimensions so use them.
            $additionalInfo = '';
            if ($dim['width'] > 500)
            {
                $original = $dim['width']; 
                $dim['width'] = 500;
                $dim['height'] = ($dim['width']/$original) * $dim['height'];
                $additionalInfo = str('STR_DOC_IMAGE_SHRUNK_TO_FIT');
            }
            
            $content = "<img width='".$dim['width']."' height='".$dim['height']."' src=\"download.php?type=".$doc->getType()."&id=".$doc->get(PROP_ID)."&sl\"\>";
            if ($additionalInfo)
            {
                $content .= '<div class="subtext">'.$additionalInfo.'</div>';
            }                    
        }
        else
        {
            // No dimensions stored so let the browser figure it out.
            $content = "<img src=\"download.php?type=".OBJECT_TYPE_ITEM."&id=".$doc->get(PROP_ID)."&sl\"\>";
        }
        
        return $content;        
    }
	
	public function getMimeType($documentPath)
	{
		$type = '';
		@list($unused,$unused,$extension) = pathinfo($documentPath);
		
		if ($extension == 'jpg' || $extension == 'jpeg')
		{
			$type = 'image/jpeg';
		}
		else if ($extension == 'gif')
		{
			$type = 'image/gif';
		}
		else if ($extension = 'png')
		{
			$type = 'image/png';		
		}
		
		return array($type);
	}
	
	public function getDescription($documentPath='')
	{
        $description = '';
        if ($documentPath)
        {		
		    $info = getimagesize ($documentPath);
		    if (is_array($info))
		    {
			    $width = $info[0];
			    $height = $info[1];			
			    $type = $info[2];			
			    $mime = image_type_to_mime_type ($type);
			    
			    $description = "Type: $mime, Width: $width, Height: $height";			
		    }
		    else
		    {
			    $description = 'Image File';
		    }		
        }
        else
        {
            $description = 'Image File';
        }
		
		return $description;
	}
	
	public function getRelatedUrl()
	{		
		return '';
	}
	
	public function getFilenameExtensions()
	{
		return array ('jpg','jpeg','gif','png','bmp');
	}

	public function getNexusFileType()
	{
		return DOCTYPE_IMAGE;
	}
	
}

?>