<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFormCompany extends iwForm
{
	public function getFormAction()
	{
		return URLACTION_MODCOMPANY;	
	}
			
	function getFormType()
	{
		return FORMTYPE_COMPANY;	
	}
		

	public function getTitle()
	{
		$id = $this->getUrlIdField();
		if ($id == '')
		{
			return str('STR_COMPANY_ADD_FORM_TITLE');
		} 
		else
		{
			return str('STR_COMPANY_EDIT_FORM_TITLE');
		}
	}
		
	/**
	 * If the ID parameter is in the url then that group
	 * is loaded and stored in the data field of the class
	 * otherwise a new object is created and set.
	 * @return bool Returns true unless an invalid id is given
	 * 				in the url id parameter. 
	 */
	protected function loadData()
	{
		// Check the cache first.	
		if ($this->recallFormData())
		{
			return true;
		}
		
		$id = $this->getUrlIdField();
		if ($id != '')
		{
			$company = new shCompany();
			if ($company->loadId($id))
			{
				$this->setData($company);
				return true;
			}
			else
			{
				$this->setData(null);
				return false;
			}
		}
		
		// adding data
		$this->setData(new shCompany);
		return true;
	}
		
	public function getRedirectLink()
	{
		if (($this->getData() instanceof shCompany) && 
			($this->getData()->id != ''))
		{
			return '?'.URLARG_ACTION.'='.URLACTION_VIEWCOMPANY.
					'&'.URLARG_ID.'='.$this->getData()->id;
		}
		
		return '';
	}
	
	/**
	 * This will return an associative array of data that should 
	 * be applied to whatever template system is being used.
	 * @return array An associative array of data
	 */
	function getFields()
	{				
		$fields = array();
		
		$fields[] = new shFormField(FIELD_TYPE_HIDDEN,$this->getData()->id,PROP_ID);
		$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->name,PROP_NAME,str('STR_COMPANY_NAME'));
		$fields[] = new shFormField(FIELD_TYPE_TEXTAREA,$this->getData()->description,PROP_DESCRIPTION,str('STR_COMPANY_DESCRIPTION'));
		$fields[] = new shFormField(FIELD_TYPE_HTML,$this->getData()->intro_page,PROP_INTRO_PAGE,str('STR_COMPANY_WEB'));		
		$fields[] = new shFormField(FIELD_TYPE_EDITABLEIMAGE,$this->getData()->getLogoLink(),PROP_LOGO, str('STR_COMPANY_LOGO'));
				
		$groupSel = new shFormField(FIELD_TYPE_GROUPSELECTOR,'','group_list');		
		$groupSel->fieldOptions['form'] = $this->getFormName();
		$groupsInCompany = $this->getData()->getGroups();		
		$ids = array();
		foreach ($groupsInCompany as $group) 
		{
			$ids[] = $group->id;						
		}
		$groupSel->fieldValue = $ids;
		$fields[] = $groupSel;
		
		return $fields;	
	}

	/**
	 * Renders the group creation dialog.
	 *
	 * @return unknown
	 */
	protected function getFormSuffixContent()
	{
		$smarty = app()->getSmartyObject();
		return smarty_templ_fetch('controls/control.ajax.addgroup.tpl.html',$smarty);				
	}
		
	/**
	 * This just reads the contents of the file that was uploaded 
	 * and returns the contents.  Returns an empty string if no logo
	 * was uploaded.
	 * @return string The contents of the uploaded file
	 */
	function getCompanyLogoFromForm ()
	{
		if (isset($_FILES['logo']))
		{
			if ($_FILES['logo']['type'] != '')
			{
				if ($_FILES['logo']['type'] == 'image/jpeg' ||
					$_FILES['logo']['type'] == 'image/pjpeg')
				{
					return file_get_contents ($_FILES['logo']['tmp_name']);
				}
				else
				{
					$this->addError (sprintf(str('STR_COMPANY_INVALID_THUMB'),$_FILES['logo']['type']), ERROR_TYPE_SERIOUS);
				}
			}
		}
		
		return '';
	}
	
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit ()
	{
		$company = new shCompany;
		
		if ($this->isEditingObject())
		{
			// Load the current properties
			$company->set(PROP_ID, ps(PROP_ID));
			if (!$company->load ())
			{
				$this->addError (sprintf(str('STR_COMPANY_NOTFOUND'),$company->id));
				return false;
			}
		}		
		
		if (ps('clear_logo') != '')
		{
			$company->set(PROP_LOGO, '');
		}
		else
		{
			$logo = $this->getCompanyLogoFromForm();
			l('Logo: '.$logo);
			if ($logo != '')
			{
				// Only set the logo if something new was given.
				$company->set(PROP_LOGO, $logo);
			}
		}
		
		$company->set(PROP_NAME,ps(PROP_NAME));
		$company->set(PROP_DESCRIPTION,ps(PROP_DESCRIPTION));						
		$company->set(PROP_INTRO_PAGE, $_POST[PROP_INTRO_PAGE]);		
		$company->set(PROP_STATUS, COMPANY_STATUS_ACTIVE);
		
		// Get the selected Companies/Groups that have access to this object.
		$selectedGroups = isset($_POST['group_list']) ? $_POST['group_list'] : array();
		if (!is_array($selectedGroups))
		{
			if ($selectedGroups != '')
			{
				$selectedGroups = array ($selectedGroups);
			}
			else
			{
				$selectedGroups = array ();
			}
		}
		
		// Replace the current set with what we have here.
		$company->clearGroups();
		foreach ($selectedGroups as $cg)
		{
			if (is_numeric($cg))
			{
				$company->addGroup($cg);			
			}
		}
				
		$this->setData($company);
		$this->rememberFormData();
		
		if ($company->requiredFieldsValid())
		{
			if ($this->isEditingObject())
			{
                $continue = true;
                nx_fire_event('EVENT_OBJECT_EDITING',array('object'=>$company,'continue'=>&$continue));
                if ($continue)
                {                                                                       
				    if ($company->update ())
				    {
					    $this->forgetFormData();
					    $this->addNotification (str('STR_COMPANY_UPDATED'));
                        
                        nx_fire_event('EVENT_OBJECT_EDITED',array('object'=>$company));
                        
					    return true;
				    }
                }
			}
			else
			{
                
                $continue = true;
                nx_fire_event('EVENT_OBJECT_ADDING',array('object'=>$company,'continue'=>&$continue));
                if ($continue)
                {                                                                       
				    if ($company->add ())
				    {
					    $this->forgetFormData();
					    $this->addNotification (str('STR_COMPANY_ADDED'));
                        
                        nx_fire_event('EVENT_OBJECT_ADDED',array('object'=>$company));
                        
					    return true;
				    }
                }
			}
		}
		
		return false;		
	}

	function authorize ()
	{
		$id = g(URLARG_ID);
		if (!$id) 
		{
			$id = ps(PROP_ID);
		}
		
		$user = app()->getLoggedInUserObject();
		return $user->getPermissions()->isAllowed(URLACTION_MODCOMPANY,$id);
	}		
	
}
?>
