<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * ‰
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFormDocument extends iwForm
{
	public $_categoryReturnId = '';
		
	public function getFormAction()
	{
		return URLACTION_MODDOC;	
	}

	/**
	 * Return an empty string so that the base knows to display
	 * the content area as is (no surrounding form template)
	 *
	 * @return string Empty string
	 */
	protected function getFormTemplate()
	{
		return '';	
	}
		
	function geTitle()
	{
		$id = $this->getFormOrUrlId();
		if ($id == '' || $id == 0)
		{
			return str('STR_DOC_ADD_FORM_TITLE');
		} 
		else
		{
			return str('STR_DOC_EDIT_FORM_TITLE');
		}	
	}
	
	function getFormType()
	{
		return FORMTYPE_DOCUMENT;	
	}

	public function getRedirectLink()
	{
		if (($this->getData() instanceof shItemDocument) && 
			($this->getData()->id != ''))
		{
			return '?'.URLARG_ACTION.'='.URLACTION_VIEWDOC.
					'&'.URLARG_ID.'='.$this->getData()->id;
		}
		
		return '';		
	}
	
	function getTitle()
	{
		$id = $this->getUrlIdField();
		if ($id == '')
		{
			return str('STR_DOC_ADD_FORM_TITLE');
		} 
		else
		{
			return str('STR_DOC_EDIT_FORM_TITLE');
		}		
	}
	
	/**
	 * This will return an associative array of data that should 
	 * be applied to whatever template system is being used.
	 * @return array An associative array of data
	 */
	protected function getContentArea (&$smarty)
	{		
		if (g(URL_MULTIUPLOAD) != '')
		{
			return $this->handleMultiUpload();
		}
				
		$data = array ();
		$item = new shItemDocument;
						
		$id = $this->getUrlIdField();
				
		// First check if there's a cached object
		$obj = $this->getData();
		if (!$obj || ($obj->get(PROP_ID) != $id))
		{		
			// Is there an id in the url?  If so, then
			//	we are editing.
			$id = $this->getUrlIdField();
			if ($id != '')
			{
				$item->set(PROP_ID,$id);
				if (!$item->load())
				{
					$this->addError (sprintf(str('STR_DOC_NOTFOUND'), $id), ERROR_TYPE_SERIOUS);
				}
			}
		}		
		else
		{			
			$item = $obj;			
		}
		
		$cgpairs = array ();
        $uploader = new shUploaderAjax();
        $uploader->setUploadLocation(DEST_UPLOAD_PURGATORY);
		if ($item)
		{			
			$data['doc_name'] = $item->get(PROP_NAME);
			$data['doc_description'] = $item->get(PROP_DESCRIPTION);		
			$data['document_id'] = $item->get(PROP_ID);
			
			$data['doc_filename'] = '';
			$data['doc_size'] = '';
			if ($item->get (PROP_ORIGINAL_FILENAME) == '')
			{
				$files = uploads()->getPurgatoryFiles();
				if (count($files) > 0)
				{
					$docProps = $files[0];
					$data['doc_filename'] = $docProps->originalFilename;
					$data['doc_size'] = iwUtilities::byteFormat($docProps->fileSize);							
				}
			}
			else
			{
				$data['doc_filename'] = $item->get (PROP_ORIGINAL_FILENAME);
				$data['doc_dimensions'] = $item->getImageDimensions(IMAGE_BOTH);
				$data['doc_size'] = iwUtilities::byteFormat(storage()->getFileSize($item));							
			}
			$data['doc_binary'] = true;
			$data['doc_type'] = $item->get(PROP_SUBTYPE_ID);		
            
            $file = new shUploadDocumentProperties();
            $file->originalFilename = $data['doc_filename'];
            $file->fileSize = $data['doc_size'];
            $uploader->setStartingFile($file);
		}

        $data['upload_control'] = $uploader->getMarkup();
        
		$data['is_editing'] = $this->isEditingObject();
		
		// Get related items
		$relatedItems = $item->getRelatedItems();		
		sort($relatedItems);
		$list = '';
		foreach ($relatedItems as $ritemid)
		{
			if ($list != '') $list .= ',';
			
			$list .= $ritemid;	
		}
		$data['related_item_ids'] = $list;
		
		// Get attributes
		$allAttributes = new shAttributeCollection();
		$allAttributes->load (new iwParameterCollection());
		$attributes = $item->getAttributes();		
		$data['attributes'] = array();
		foreach ($allAttributes->items() as $attrib)
		{			
			$selected = false;
			foreach ($attributes as $itemAttrib)
			{				
				if ($itemAttrib->get(PROP_ID) == $attrib->get(PROP_ID))
				{
					$selected = true;
					break;
				}
			}
			
			$data['attributes'][] = 
					array (
						'value'=>$attrib->get(PROP_ID),
						'name'=>$attrib->get(PROP_NAME),
						'selected'=>$selected);
		}
		
		
		// Collect a list of all the categories	
		$allCategories = app()->getLoggedInUserObject()->getAuthorizedCategories();
		$categories = $item->getCategories();
		$data['categories'] = array();
		$urlparentid = g(URL_PARENTID);	
		$this->_categoryReturnId = $urlparentid;
					
		if (count($allCategories) == 0)
		{
			$this->addError(str('STR_NO_AVAILALBLE_CATEGORIES'));	
			return '';		
		}
		
        $parentCategory = null;
		foreach ($allCategories as $cat)
		{
			$selected = false;
            
            // If a parent category is in the url, then let's set 
            //  parent to that category (as a convenience)
			if ($urlparentid && ($cat->get(PROP_ID) == $urlparentid))
			{                
                $parentCategory = $cat;
				$selected = true;
			}
			else
			{
				foreach ($categories as $itemCat)
				{								
					if ($cat->get(PROP_ID) == $itemCat->get(PROP_ID))
					{
						$selected = true;
						break;
					}
				}
			}
						
            if (count($allCategories) == 1)
            {
                // There is only one category so automatically add it to the list
                $selected = true;
            }
            
			$data['categories'][] = array (
						'value'=>$cat->get(PROP_ID), 
						'name'=>$cat->get(PROP_NAME), 
						'selected'=>($selected));
		}

		$data['max_document_size'] = iwUtilities::byteFormat(DOCUMENT_MAX_FILESIZE);
		
		$permission = app()->getLoggedInUserObject()->getPermissions();
		
		$domainRestriction = DOMAIN_NONE;
		$allowEveryone = false;
		if ($permission->allowGlobalAddDocument)
		{
			$domainRestriction = DOMAIN_GLOBAL;
			$allowEveryone = true;
		}
		else if ($permission->allowDomainAddDocument)
		{			
			$domainRestriction = DOMAIN_COMPANY;
		}
		else if ($permission->allowDomainGroupAddDocument)
		{
			$domainRestriction = DOMAIN_GROUP;
		}			
        
		$data['domains'] = shCompanyGroupPair::getHierarchicalDomainList($item,$allowEveryone);
        // If a parent category is in the url, then let's try and match the 
        //  default domains to those of the category for convenience.
        if ($parentCategory)
        {
            $pairs = $parentCategory->getDomains();
			for ($i=0;$i<count($data['domains']);$i++)
			{
				$company = $data['domains'][$i];
				for($k=0;$k < count($company['groups']);$k++)
				{
					$group = $company['groups'][$k];
					
					$cid = $company['id'];
					$gid = $group['id'];

	                foreach($pairs as $pair)
	                {
	                    $gidCat = $pair->getGroup()->get(PROP_ID);
	                    $cidCat = $pair->getCompany()->get(PROP_ID);
	                    if ($gidCat == $gid && $cid == $cidCat)
	                    {	                    	
	                        $data['domains'][$i]['groups'][$k]['selected'] = true;
	                    }
	                }                
					
				}
			}			
        }
        		        
		foreach ($data as $prop=>$value)
		{
			$smarty->assign($prop, $value);
		}
		
		$content = smarty_templ_fetch('forms/form.mod.document.tpl.html',$smarty);

        				
		return $content;	
	}
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit ()
	{				
		$item = new shItemDocument;				
		if ($this->isEditingObject())
		{
			$item->set(PROP_ID,ps(PROP_ID));
			if (!$item->load())
			{
				$this->addError (str('STR_DOC_ORIGINALNOTFOUND'), ERROR_TYPE_SERIOUS);
				return false;
			}			
		}
				
        // STANDARD PROPERTIES
        //--------------------------------------------------        
		$item->set(PROP_NAME,ps(PROP_NAME));
		$item->set(PROP_DESCRIPTION,ps(PROP_DESCRIPTION));
		$item->set(PROP_STATUS,ITEM_STATUS_ACTIVE);

        // CATEGORIES PROPERTY
        //--------------------------------------------------        
		// Get the selected categories .        
		$selectedCats = ps('category_list');
		
		if (!is_array($selectedCats))
		{
			if ($selectedCats != '')
			{
				$selectedCats = array ($selectedCats);
			}
			else
			{
				$singleCat = ps('category');
				if ($singleCat != '')
				{
					$selectedCats = array ($singleCat);					
				}
                else
				{
	                // if no category is selected do NOT 
	                //  default to adding root since the user
	                //  may not have access to this category
					$selectedCats = array();
				}				
			}
		}
		
		// Replace the current set with what we have here.
		$item->clearCategories();
		$user = app()->getLoggedInUserObject();
		
		if ($user)
		{
			if (count($selectedCats) > 0)
			{
				foreach ($selectedCats as $cat)
				{
					$item->addCategory($cat);
				}
			}
		}		

        // RELATED ITEMS
        //--------------------------------------------------
		// Replace the current set of related items with the ones 
		//	submitted with the form (if any)
		$item->clearRelatedItems();
		$ritems = ps('related_ids');		
		$ritems = split (',',$ritems);
		foreach ($ritems as $ri)
		{
			$ri = trim ($ri);
			if ($ri != '' && is_numeric ($ri))
			{
				$item->addRelatedItem ($ri);
			}
		}
		
        // DOMAIN PROPERTY
        //--------------------------------------------------
		// Get the selected Companies/Groups that have access to this object.
		$selectedCG = ps('domains');
		if (!is_array($selectedCG))
		{
			if ($selectedCG != '')
			{
				$selectedCG = array ($selectedCG);
			}
			else
			{   
				$selectedCG = array (COMPANY_ALL.','.GROUP_ALL);
			}
		}
		
		// Replace the current set with what we have here.
		$item->clearCompanyGroupPairs();
		foreach ($selectedCG as $cg)
		{
			list($company,$group) = split(',',$cg);
			$item->addCompanyGroup ($company,$group);			
		}
							 
        // Store in the cache in case form submission fails 
        //  and we need to redisplay entered data.
		$this->setData($item);
		$this->rememberFormData();
			
        $contentUpdated = false;
		if ($item->requiredFieldsValid())
		{
            $originalFileName = '';
            $currentFilePath = '';
            $fileType = '';
            
			$this->getUploadedFileInfo($originalFileName, $currentFilePath, $fileType);            

            if ($currentFilePath) 
            {                
                // Only set if there's something new to upload
                $name = $originalFileName;
                
                $name = str_replace(array('"',' '),'_',$name);
                
                $type = typeman()->determineNexusType($name, $fileType);
                
                // Check for room on disk for this document
                $size = filesize($currentFilePath);                
                if (!shFileAnalysis::isLibrarySpaceAvailable($size, MAX_STORAGE_IN_MB))
                {
                    $this->addError (str('STR_DOC_NOSPACE'),ERROR_TYPE_SERIOUS);
                            
                    return false;
                }                
                
                if ($type == DOCTYPE_SHARE)
                {
                    return $this->processZipMultiAdd($name,$currentFilePath, $item);
                }
                
                if ($type != DOCTYPE_UNKNOWN)
                {
                    $this->processStandardFile($name,$originalFileName,$currentFilePath,$item,$type);                                       
                    $contentUpdated = true;
                }
                else
                {
                    $this->addError (str('STR_DOC_UNSUP_TYPE'), ERROR_TYPE_SERIOUS);
                    return false;
                }
            }
            else
            {            
                // If we're editing the object an no file is specified, that's okay, we'll
                //    just use the existing one.  Otherwise, flag the error and exit.
                if (!$this->isEditingObject())
                {
                    $this->addError (str('STR_DOC_NOFILE'), ERROR_TYPE_SERIOUS);
                    return false;
                }
            }
            
            
			if ($this->isEditingObject())
			{			
                $continue = true;	
                nx_fire_event('EVENT_OBJECT_EDITING',array('object'=>$item,'continue'=>&$continue));
                if ($continue)
                {                                                                       
				    if ($item->update ())
				    {					
					    if ($contentUpdated)
					    {
						    // This will rename the file and delete the original
						    $item->storeFileToFinalLocation ();
					    }

					    app()->storeTemporaryUploadPath('');
					    $this->forgetFormData();			
					    $this->addNotification (str('STR_UPDATEDDOC'));
                        
                        nx_fire_event('EVENT_OBJECT_EDITED',array('object'=>$item));
					    
					    return true;
				    }
                }
			}
			else
			{
                $continue = true;
                nx_fire_event('EVENT_OBJECT_EDITING',array('object'=>$item,'continue'=>&$continue));
                if ($continue)
                {                                                       
                    
				    if ($item->add ())
				    {
					    // This will rename the file and delete the original
					    $item->storeFileToFinalLocation();
					    
					    app()->storeTemporaryUploadPath('');
					    $this->forgetFormData();
					    $this->addNotification (str('STR_ADDEDDOC'));
                        
                        nx_fire_event('EVENT_OBJECT_ADDED',array('object'=>$item));
                        
					    return true;
				    }
                }
			}
		}
		
		return false;		
	}	

    /**
    * @desc Prepares an uploaded document for storage in the library.
    * @param string $name The name of the file (after spaces and special characters have been removed)
    * @param string $originalFilename The name of the file as uploaded by the user.
    * @param string $currentFilePath The path to the file on the server
    * @param shItemDocument $item The document object
    */
    protected function processStandardFile($name,$originalFileName, $currentFilePath,$item, $fileType)
    {
        $uploadsDir = app()->get('upload_dir');
        $tempPath = $uploadsDir.'/'.$name;
        
        $moved = false;
        if (is_uploaded_file($currentFilePath))
        {
            if (!move_uploaded_file($currentFilePath,$tempPath))
            {
                iwUtilities::logToFile($currentFilePath.'|||'.$tempPath);
                $this->addError (str('STR_DOC_FILEMOVE_FAILED'), ERROR_TYPE_SERIOUS);
                return false;
            }
        }
        else
        {
            if (file_exists($tempPath))
            {
                // Sometimes during form submission, a temp file could be left behind. This assures
                //  that it won't affect current submissions.
                @unlink($tempPath);
            }
                
            if (!@rename($currentFilePath,$tempPath))
            {                                
                l('Unable to rename '.$currentFilePath.' to '.$tempPath);
                $this->addError (sprintf(str('STR_DOC_RENAME_FAILED'),$originalFileName), ERROR_TYPE_SERIOUS);
                return false;                        
            }
        }
            
        $item->loadDocument($fileType,$originalFileName,$tempPath);
        
        uploads()->clearUserPurgatory();
    }
    
    /**
    * @desc Handles when a file with the extension .share is uploaded. When
    * this happens it means that the user has zipped a file with the intention of
    * adding all the files in the zip as separate documents.
    * @param string $name The name of the file.
    * @param string $currentFilePath The path to the file on the server
    * @param shItemDocument $item The document object
    */
	protected function processZipMultiAdd($name,$currentFilePath, $item)
    {               
        // This will take care of adding items (one for each file in zip file).
        $uploadsDir = app()->get('upload_dir');
        $tempPath = $uploadsDir.'/'.basename($name,'.share').'.zip';

        $moved = false;
        if ($posted)
        {
            $moved = move_uploaded_file($currentFilePath,$tempPath);
        }
        else
        {
            $moved = rename($currentFilePath,$tempPath);
        }
            
        if ($moved)
        {
            $filesAdded = array();
            $filesNotAdded = array ();
            
            $archiveDir = app()->get('archive_dir_relative');
            
            $zip = new Archive_Zip ($file);
            $files = $zip->extract (array('remove_all_path'=>true,'add_path'=>$archiveDir));
            
            foreach ($files as $unzippedFile)
            {
                $filename = $unzippedFile['filename'];
                $type = typeman()->determineNexusType($unzippedFile['filename']);
                if ($type != DOCTYPE_UNKNOWN)
                {
                    // Prepare a new item object to be inserted into the database
                    $newItem = clone($item);
                    
                    // Set the name of the item as whatever the user entered for the name prepended
                    //    to the archived file's name.
                    $newItem->set(PROP_NAME,$item->get(PROP_NAME).'-'.iwUtilities::basenameNoExtension($filename));
                    
                    // Now move the file into the uploads directory (as expected by the
                    //    load document function
                    $tempPath = app()->get('upload_dir').'/'.basename($filename);
                    if (!copy ($filename,$tempPath))
                    {
                        // Failed to copy into good location
                        $filesNotAdded[] = $unzippedFile['filename'];
                        continue;
                    }
                    
                    // Now prepare the document for adding.
                    $newItem->loadDocument ($type,basename($filename),$tempPath);                
                    
                    // Add the item to the database
                    if ($newItem->add ())
                    {
                        // This will rename the file and delete the original                    
                        $newItem->storeFileToFinalLocation ();

                        // Store filename for future message
                        $filesAdded[] = $unzippedFile['filename'];
                        
                        // Delete the extracted file.
                        unlink($filename);
                        
                    }                                
                    else
                    {
                        $filesNotAdded[] = $unzippedFile['filename'];
                    }
                }
                else 
                {
                    $filesNotAdded[] = $unzippedFile['filename'];
                }
            }
            
            if (count($filesAdded) > 0)
            {
                $this->addNotification (sprintf(str('STR_MULTIODC_ADD'),count($filesNotAdded)));    
            }        
            
            if (count($filesNotAdded) > 0)
            {
                $this->addError(sprintf(str('STR_MULTIODC_ADDFAILED'),count($filesNotAdded)), ERROR_TYPE_SERIOUS);
            }
            
            if (count($filesNotAdded) == 0)
            {
                // Forget form data only if all the items were successfully added.
                $this->forgetFormData();
            }
            
            // Delete the zip file.
            unlink ($file);
            
            return count($filesAdded) > 0;
            
        }
        else
        {
            return false;
        }     
    }
	
	/**
	 * Gets uploaded file information no matter how it was uploaded. The 
	 * possibilities are:
	 * 	1) Upload In Form - Standard way of uploading a document along with 
	 * 		all the other information that is associated with a document
	 *  2) Upload Separately - Allows for the document to be uploaded 
	 * 		prior to the actual document being submitted.  Gives the user
	 * 		a chance to see if the upload is successful before filling in 
	 * 		all the other info. 
	 */
	function getUploadedFileInfo(&$originalFileName, 
									&$currentFilePath, &$fileType)
	{
        $files = uploads()->getPurgatoryFiles();
        
        if (is_array($files) && (count($files) > 0))
        {
            $fileType = $files[0]->mimeType;
            $currentFilePath = $files[0]->currentFilePath;
            $originalFileName = $files[0]->originalFilename;                       
        }
	}
		
	function authorize ()
	{
		$id = $this->getUrlIdField(URLARG_ID);
		if (!$id) 
		{
			$id = ps(PROP_ID);
		}
		
		$user = app()->getLoggedInUserObject();		
		return $user->getPermissions()->isAllowed(URLACTION_MODDOC,$id);
	}
	
	/**
	 * Handles a request to upload multiple uploads.  
	 * @return string The html for multi upload
	 */
	function handleMultiUpload()
	{
		$smarty = app()->getSmartyObject();
		
		$catId = g(URL_PARENTID);
		
		$field = new shFormField(FIELD_TYPE_CATEGORYSELECTOR,$catId,'parentid',str('STR_CAT_PARENT'));
		$smarty->assign('multiupload_categoryfield',$field->getHtml());
		
		$field = new shFormField(FIELD_TYPE_SINGLE_DOMAINSELECTOR,array(),'domains',str('STR_CAT_DOMAINSWITHACCESS'));
		$smarty->assign('multiupload_domainfield',$field->getHtml());
			
		$uploader = new shUploaderJUpload();
        $uploader->setCategory($catId);
        $uploader->setUploadLocation(DEST_UPLOAD_LIBRARY);
		$smarty->assign('multiupload_code', $uploader->getMarkup());		
		return smarty_templ_fetch('forms/form.add.document.multiple.tpl.html',$smarty);				
	}	
}
?>
