<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * The form factory is used to generate forms based on type or
 * action information supplied by the caller.
 */
class shFormFactory extends iwObject
{
	var $_forms;
	
	/**
	 * Maintains a list of all the possible form objects
	 * During one of the creation calls, these string are used
	 * to generate an instance of the form class.
	 */
	public function shFormFactory()
	{
		$this->_forms = array 
			(
				'shFormLogin',
				'shFormGroup',
				'shFormAttribute',
				'shFormCompany',
				'shFormDocument',
				'shFormRegisterUser',
				'shFormSendUserMail',
				'shFormSendGroupEmail',
				'shFormPost',
				'shFormSettings',
				'shFormLostPassword',
				'shFormUsageReport',
				'shFormFind',
				'shFormCategory',
				'shFormUser',
				'shFormPoolItem',
				'shMultiSubmitForm',
				'shFormMultiDocument',
                'shFormPlugins'
			);
	}

	/**
	 * Tries to make a form based on get/post variables.
	 * It looks at the URLARG_ACTION parameter in the url first
	 * then it looks at the FORM_TYPE parameter in the pos 
	 * (if any).  Returns null if no form needs to be made.
	 * @return iwForm A form if necessary or null if none specified.
	 */
	public function makeForm()
	{
		$action = g(URLARG_ACTION);
		if ($action != '')
		{
			$obj = $this->createFormFromAction($action);
			if ($obj != null)
			{
				return $obj;		
			}
		}
		
		$type = ps(FORM_TYPE);
		if ($type != '')
		{
			$obj = $this->createFormFromType($type);
			if ($obj != null)
			{
				return $obj;
			}
		}
		
		return null;
	}
	
	/**
	 * Takes a type constant (of the form FORMTYPE_X) and 
	 * returns a form object that matches that type
	 * @param string $type The type of the form (FORMTYPE_X)
	 * @return iwForm Returns a form object or null if not found.
	 */
	public function createFormFromType($type)
	{		
		foreach ($this->_forms as $form)
		{			
			$formOb = new $form();
			if ($formOb->getFormType() == $type)
			{
				return $formOb;				
			}	
			else
			{
				unset($formOb);
			}	
		}
		
		return null;
	}
		
	/**
	 * Takes an action constant (of the form URLACTION_X) and 
	 * returns a form object that matches that action
	 * @param string $action The action associated with the form (URLACTION_X)
	 * @return iwForm Returns a form object or null if not found.
	 */
	public function createFormFromAction($action)
	{
        $forms = $this->_forms;
        nx_fire_event('EVENT_FORMS_NEEDED',array('forms'=>&$forms));
        
		foreach ($forms as $form)
		{
			$formOb = new $form();
			if ($formOb->getFormAction() == $action)
			{
				return $formOb;				
			}		
			else
			{
				unset($formOb);
			}
		}
		
		return null;
	}
}

?>
