<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFormGroup extends iwForm
{
	public function getFormAction(){return URLACTION_MODGROUP;}	
	public function getFormType(){return FORMTYPE_GROUP;}
	
	public function getTitle()
	{
        $id = '';
        if (g(URLARG_ACTION) == URLACTION_MODGROUP)
        {
        		$id = $this->getUrlIdField();
        }
        
		if ($id == '')
		{
			return str('STR_GROUP_ADD_FORM_TITLE');
		} 
		else
		{
			return str('STR_GROUP_EDIT_FORM_TITLE');
		}
	}
				
	/**
	 * If the ID parameter is in the url then that group
	 * is loaded and stored in the data field of the class
	 * otherwise a new object is created and set.
	 * @return bool Returns true unless an invalid id is given
	 * 				in the url id parameter. 
	 */
	protected function loadData()
	{
        // We only get the id from the url field if we're on a page
        //  that is specifically trying to modify/add a group.
        $id = '';
        if (g(URLARG_ACTION) == URLACTION_MODGROUP)
        {
		    $id = $this->getUrlIdField();
        }
        
		if ($id != '')
		{
			$group = new shGroup();
			if ($group->loadId($id))
			{
				$this->setData($group);
				return true;
			}
			else
			{
				$this->setData(null);
				return false;
			}
		}
		
		// adding data
		$this->setData(new shGroup);
		return true;
	}
	
	/**
	 * Returns an array of field objects containing:
	 *  - Name
	 *  - Description
	 *  @return array An array of shFormField objects
	 */
	function getFields()
	{
		$fields = array();

		$group = $this->getData();
		
		$field = new shFormField();
		$field->fieldName = PROP_ID;
		$field->fieldType = FIELD_TYPE_HIDDEN;
		$field->fieldValueType = FLD_VALUE_TYPE_NUMBER;
		$field->fieldId = PROP_ID;
		$field->fieldValue = $group->id;
		$field->required = true;
		$fields[] = $field;
		
		$field = new shFormField();
		$field->label = str('STR_GROUP_NAME');
		$field->fieldName = PROP_NAME;
		$field->fieldType = FIELD_TYPE_TEXT;
		$field->fieldValueType = FLD_VALUE_TYPE_STRING;
		$field->fieldId = $fieldName = PROP_NAME;
		$field->fieldValue = $group->name;
		$field->fieldHelp = '';
		$field->required = true;
		$fields[] = $field;
		
		$field = new shFormField();
		$field->label = str('STR_GROUP_DESCRIPTION');
		$field->fieldName = PROP_DESCRIPTION;
		$field->fieldType = FIELD_TYPE_TEXTAREA;
		$field->fieldValueType = FLD_VALUE_TYPE_STRING;
		$field->fieldId = $fieldName = PROP_DESCRIPTION;
		$field->fieldValue = $group->description;
		$field->fieldHelp = '';
		$field->required = false;
		$fields[] = $field;
		
		return $fields;
	}
		
	public function getRedirectLink()
	{
		$grp = $this->getData();
		if ($grp instanceof shGroup)
		{
			return sprintf('?%s=%s&%s=%d', URLARG_ACTION,
				URLACTION_VIEWGROUP,URLARG_ID,$grp->id);	
		}
				
		return parent::getViewLink();
	}
		
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit ()
	{
		$group = new shGroup;
		$group->set(PROP_NAME,ps(PROP_NAME));
		$group->set(PROP_DESCRIPTION,ps(PROP_DESCRIPTION));
		$group->set(PROP_STATUS,GROUP_STATUS_ACTIVE);		
		
		$user = app()->getLoggedInUserObject();
		
		$this->setData($group); 
		$this->rememberFormData();
				
		if ($group->requiredFieldsValid())
		{			
			if ($this->isEditingObject())
			{
                $continue = true;
                nx_fire_event('EVENT_OBJECT_EDITING',array('object'=>$group,'continue'=>&$continue));
                if ($continue)
                {                                                                           
				    $group->set(PROP_ID,ps(PROP_ID));
				    if ($group->update ())
				    {
					    $this->_results = $group;
					    
					    $this->forgetFormData ();
					    
					    $this->addNotification (str('STR_GROUP_UPDATED'));
                        
                        nx_fire_event('EVENT_OBJECT_EDITED',array('object'=>$group));
                        
					    return true;
				    }
                }
			}
			else
			{
                $continue = true;
                nx_fire_event('EVENT_OBJECT_ADDING',array('object'=>$group,'continue'=>&$continue));
                if ($continue)
                {                    
				    if ($group->add ())
				    {                    
					    // If the current is a domain admin, then we need to 
					    //	automatically add the group to the user's domain.					
					    if ($user->getPermissions()->getPermissionGroup() == USER_TYPE_DOMAIN_ADMIN)
					    {
						    $org = $user->getCompany();
						    if ($org)
						    {
							    $org->addGroup($group);
							    $org->update();	
						    }
					    }				
						    
					    $this->_results = $group;
					    
					    $this->forgetFormData ();
					    
					    $this->addNotification (str('STR_GROUP_ADDED'));
                        
                        nx_fire_event('EVENT_OBJECT_ADDED',array('object'=>$group));
                        
					    return true;
				    }
                }
			}
		}
		
		return false;		
	}

	/**
	 * Returns the results of the form submission.  The results value is an
	 * an empty array if the form submit failed.  Otherwise, it contains an
	 * associative array:
	 * 
	 * 'name' => group name
	 * 'id' => group id
	 *
	 * @return array The array described above. 
	 */
	public function getResults()
	{
		if ($this->_results)
		{
			return array('name'=>$this->_results->get(PROP_NAME), 
					 'id'=>$this->_results->get(PROP_ID));
		}
		
		return array();
	}
	
	function authorize ()
	{
		$id = $this->getUrlIdField();
		if (!$id) 
		{
			// See if there's a group id in the post data
			$id = ps(PROP_ID);
		}
		
		$user = app()->getLoggedInUserObject();
		return $user->getPermissions()->isAllowed(URLACTION_MODGROUP,$id);
	}		
}
?>
