<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


class shFormLogin extends iwForm
{
	public function getFormAction()
	{
		return URLACTION_LOGIN;	
	}
	
	function getFormType()
	{
		return FORMTYPE_LOGIN;	
	}
	
	/**
	 * The login form displays other information so we have a special template for it.
	 */
	protected function getFormTemplate()
	{
		return 'forms/form.login.front.tpl.html';
	}	
	
	/**
	 * Fields for this form are:
	 * 	username
	 *  password
	 *  (link)forgot password
	 *  (link)register (optional)
	 *  @return array An array of shFormField
	 */
	function getFields()
	{
		$fields = array();

		$field = new shFormField(FIELD_TYPE_TEXT,'',PROP_USERNAME,str('STR_USER_FORM_USERNAME'));
		$field->fieldValueType = FLD_VALUE_TYPE_STRING;
		$field->fieldId = PROP_USERNAME;
		$field->required = true;
		$fields[] = $field;
		
		$field = new shFormField(FIELD_TYPE_PASSWORD,'',PROP_PASSWORD,str('STR_USER_FORM_PASSWORD'));
		$field->fieldValueType = FLD_VALUE_TYPE_STRING;
		$field->fieldId = PROP_PASSWORD;
		$field->required = true;
		$fields[] = $field;
		
		$field = new shFormField(FIELD_TYPE_LINK,str('STR_USER_FORGOTPASS'));
		$field->fieldOptions = '?'.URLARG_ACTION.'='.URLACTION_LOSTPASS;
		$field->required = false;
		$fields[] = $field;

		if (settings()->allow_self_reg)
		{
			$field = new shFormField(FIELD_TYPE_LINK,str('STR_USER_REGISTERACCOUNT'));
			$field->fieldOptions = '?'.URLARG_ACTION.'='.URLACTION_REGISTER;
			$field->required = false;
			$fields[] = $field;
		}
				
		return $fields;	
	}
	
	/**
	 * Always go to the frontpage after login
	 */
	public function getRedirectLink()
	{        
		return 'index.php';	
	}
	
	/**
	 * The login form adds template data for public documents
	 */
	function addTemplateData(&$smarty)
	{
	}
	
	/**
	 * Sets focus to the username field when the page loads
	 */
	function getDefaultField ()
	{
		return PROP_USERNAME;
	}
	
	/**
	 * The login form should be available to anyone.
	 */
	function authorize ()
	{
		return true;
	}
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit ()
	{			
		$user = new shUser;
		$user->set(PROP_USERNAME,ps(PROP_USERNAME));
		$user->set(PROP_PASSWORD,ps(PROP_PASSWORD));
						
		if (app()->loginUser ($user->get(PROP_USERNAME), $user->get(PROP_PASSWORD), 'shUser'))
		{						
			if (app()->getLoggedInUserObject()->isFirstTimeLogin())
			{
				$link = 'index.php?a=mod_user&id='.$user->get(PROP_USERNAME).'&pc=1';
				$this->addNotification (sprintf(str('STR_LOGIN_FIRSTTIME'),$link));
			}
			else
			{			
				$this->addNotification (str('STR_LOGIN_SUCCESS'));
			}

			return true;
		}
		// NOTE: Purposely not putting in any way of rememembering invalid
		//	data to protect from XSS attacks on login.
									
		return false;	
	}	
		
	function getRedirectLocation ()
	{
		return 'index.php';
	}

	/**
	 * The load data sees if there is data stored in 
	 * session cache (from previous login attempts)
	 */	
	protected function loadData()
	{
		$user = $this->recallFormData();
		
		if ($user instanceof shUser)
		{			
			$this->setData($user);
		}
		else
		{
			$this->setData(new shUser); 					
		}		
		
		return true;
	}	
}
?>
