<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFormPlugins extends iwForm
{
	public function getFormAction(){return URLACTION_PLUGINS;}	
	public function getFormType(){return FORMTYPE_PLUGINS;}
	
	public function getTitle()
	{
		return str('STR_PLUGIN_FORM_TITLE');
	}
				
	/**
	 * If the ID parameter is in the url then that group
	 * is loaded and stored in the data field of the class
	 * otherwise a new object is created and set.
	 * @return bool Returns true unless an invalid id is given
	 * 				in the url id parameter. 
	 */
	protected function loadData()
	{
        $this->setData(app()->getPlugins());
		return true;
	}
	
	/**
	 * Returns an array of field objects containing:
	 *  - Name
	 *  - Description
	 *  @return array An array of shFormField objects
	 */
	function getFields()
	{
		$fields = array();        
        foreach ($this->getData() as $plugin)
        {                 
            $fieldContainer = new iwFormFieldContainer();
            
            $fieldContainer->label = $plugin['name'];
			$fieldContainer->class = 'pluginHeader';
			
			$fieldContainer->addField(new shFormField(FIELD_TYPE_STATIC, $plugin['description'], '', str('STR_PLUGINS_DESCRIPTION')));
			$fieldContainer->addField(new shFormField(FIELD_TYPE_STATIC, $plugin['author'], '', str('STR_PLUGINS_AUTHOR')));
			$fieldContainer->addField(new shFormField(FIELD_TYPE_STATIC, $plugin['version'], '', str('STR_PLUGINS_VERSION')));
            $fieldContainer->addField(new shFormField(FIELD_TYPE_HIDDEN, $plugin['id'],'id',''));
						
            $field = new shFormField();
            $field->label = str('STR_PLUGINS_ACTIVE');            
            $field->fieldName = $plugin['id'];
            $field->fieldType = FIELD_TYPE_CHECKBOX;
            $field->fieldId = $plugin['id'];
            $field->fieldValue = $plugin['active']?true:false;
            $field->required = false;
            $fieldContainer->addField($field);
            
            $fields[] = $fieldContainer;
        }
        
		return $fields;
	}
		
	public function getRedirectLink()
	{
		return '?'.URLARG_ACTION.'='.URLACTION_PLUGINS;
	}
		
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit ()
	{        
        $plugins = app()->getPlugins();
        foreach($plugins as $plugin)
        {        
            $id = $plugin['id'];              
            $enabled = ps($id)==''?0:1;            
            app()->updatePluginInDatabase($plugin['id'],$enabled);
        }        
        
        $this->addNotification(str('STR_PLUGINS_UPDATED'));
        
		return true;		
	}

	/**
	 * Returns the results of the form submission.  The results value is an
	 * an empty array if the form submit failed.  Otherwise, it contains an
	 * associative array:
	 * 
	 * 'name' => group name
	 * 'id' => group id
	 *
	 * @return array The array described above. 
	 */
	public function getResults()
	{
		return array();
	}
	
	function authorize ()
	{
		$user = app()->getLoggedInUserObject();
		return $user->getPermissions()->isAllowed(URLACTION_PLUGINS,0);
	}		
}
?>
