<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFormPost extends iwForm
{
	public function getFormAction()
	{
		return URLACTION_POST;	
	}
		
	function getFormType()
	{
		return FORMTYPE_CONVERSATION;	
	}

	public function getTitle()
	{
		return str('STR_POST_ADD_POST');	
	}
	
	/**
	 * If the ID parameter is in the url then that post
	 * is loaded and stored in the data field of the class
	 * otherwise a new object is created and set.
	 * @return bool Returns true unless an invalid id is given
	 * 				in the url id parameter. 
	 */
	protected function loadData()
	{	
		$post = new shPost;
		$post->item_id = g(URL_ITEMID);
		$post->parent_post_id = g(URL_PARENTID);
		$post->username = app()->getLoggedInUsername();
		$post->datePosted = iwUtilities::databaseDateFormat(time());
		$this->setData($post);
		return true;
	}
	
	/**
	 * This will return an associative array of data that should 
	 * be applied to whatever template system is being used.
	 * @return array An associative array of data
	 */
	function getFields()
	{
		
		$post = $this->getData();
		if ($post->parent_post_id == '')
		{
			$post->parent_post_id = 0;
		}
		
		$fields = array();
		
		$fields[] = new shFormField(FIELD_TYPE_HIDDEN,$this->getData()->id,'id','');
		$fields[] = new shFormField(FIELD_TYPE_HIDDEN,$this->getData()->parent_post_id,'parent_post_id','');
		$fields[] = new shFormField(FIELD_TYPE_HIDDEN,$this->getData()->datePosted,'post_date','');		
		$fields[] = new shFormField(FIELD_TYPE_HIDDEN,$this->getData()->item_id,'item_id','');		
		
		$fields[] = new shFormField(FIELD_TYPE_TEXT,stripslashes($this->getData()->title),'title',str('STR_POST_TITLE'));
		$fields[] = new shFormField(FIELD_TYPE_TEXTAREA,stripslashes($this->getData()->body),'body',str('STR_POST_MESSAGE'));
						
		return $fields;	
	}
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit ()
	{
		$success = true;
		
		$post = new shPost;
		$post->title = addslashes(ps('title'));
		$post->body = addslashes(ps('body'));
		$post->username = ps('username');
		$post->parent_post_id = ps('parent_post_id');		
		$post->datePosted = iwUtilities::databaseDateFormat(time());
		$post->item_id = ps('item_id');
		
		$username = app()->getLoggedInUsername();

		if ($post->body == '')
		{
			$this->addError(str('STR_POST_INVALID_BODY'));
			$success = false;			
		}
		$this->setData($post);
		$this->rememberFormData();
		
		if (!$success)
		{
			return false;	
		}
		
		if ($this->isEditingObject())
		{
            $continue = true;
            nx_fire_event('EVENT_OBJECT_EDITING',array('object'=>$post,'continue'=>&$continue));
            if ($continue)
            {                                                                   
			    $post->id = $this->_page->getFormArgument('post_id');
			    if ($post->update ())
			    {
				    if ($username != $post->username)
				    {
					    $this->addError (str('STR_POST_NOEDIT'), ERROR_TYPE_SERIOUS);
				    }
				    else
				    {			
					    $this->forgetFormData ();
					    $this->addNotification (str('STR_POST_UPDATED'));
                        
                        nx_fire_event('EVENT_OBJECT_EDITED',array('object'=>$post));
                        
					    return true;
				    }
			    }
            }
		}
		else
		{
            $continue = true;
            nx_fire_event('EVENT_OBJECT_ADDING',array('object'=>$post,'continue'=>&$continue));
            if ($continue)
            {                                                                   
			    if ($post->add ())
			    {
				    $this->forgetFormData ();
				    $this->addNotification (str('STR_POST_ADDED'));
                    
                    nx_fire_event('EVENT_OBJECT_ADDED',array('object'=>$post));
                    
				    return true;
			    }
			    else
			    {
				    $this->addError(str('STR_POST_DBADD_FAILED'));
			    }
            }
		}
		
		return false;		
	}
	
	function authorize ()
	{
		return true;
	}		
	
}
?>
