<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


class shFormSettings extends iwForm
{
    public function getFormAction()
    {
        return URLACTION_SETTINGS;    
    }        
    
    function getFormType()
    {
        return FORMTYPE_SETTINGS;    
    }

    public function getTitle()
    {
        return str('STR_SETTINGS_FORM_TITLE');    
    }
    
    public function loadData()
    {
        if ($this->recallFormData())
        {
            return true;    
        }
        else
        {
            $this->setData(settings());
            return true;
        }
    }
    
    /**
     * This will return an associative array of data that should 
     * be applied to whatever template system is being used.
     * @return array An associative array of data
     */
    function getFields()
    {                    
        $settings = $this->getData();
        
        $link = 'http://php.net/date';
        $link1 = 'http://us3.php.net/manual/en/timezones.php';
        $f = new shFormField(FIELD_TYPE_STATIC,sprintf(str('STR_SETTINGS_HINT_PHPDATE'),$link),'','');                                                                                                                               
        $f->class = 'hint';
        $f1 = new shFormField(FIELD_TYPE_STATIC,sprintf(str('STR_SETTINGS_HINT_PHPTZ'),$link1),'','');
        $f1->class = 'hint';
        
        $fields[] = new shFormField(FIELD_TYPE_TEXT,$settings->company_name,SETTING_COMPANY_NAME,str('STR_SETTINGS_COMPANY'));
        $fields[] = new shFormField(FIELD_TYPE_TEXT,$settings->site_tagline,SETTING_SITE_TAGLINE,str('STR_SETTINGS_TAGLINE'));
        $fields[] = new shFormField(FIELD_TYPE_TEXT,$settings->admin_email,SETTING_ADMIN_EMAIL,str('STR_SETTINGS_ADMINEMAIL'));
        $fields[] = new shFormField(FIELD_TYPE_TEXT,$settings->time_date_format_long,SETTING_DATE_TIME_LONG,str('STR_SETTINGS_DTFLONG'));
        $fields[] = $f;        
        $fields[] = new shFormField(FIELD_TYPE_TEXT,$settings->time_date_format_short,SETTING_DATE_TIME_SHORT,str('STR_SETTINGS_DTFSHORT'));
        $fields[] = $f;        
        $fields[] = new shFormField(FIELD_TYPE_TEXT,$settings->date_format_long,SETTING_DATE_LONG,str('STR_SETTINGS_DFLONG'));
        $fields[] = $f;        
        $fields[] = new shFormField(FIELD_TYPE_TEXT,$settings->date_format_short,SETTING_DATE_SHORT,str('STR_SETTINGS_DFSHORT'));
        $fields[] = $f;        
        $fields[] = new shFormField(FIELD_TYPE_TEXT,$settings->time_format,SETTING_TIME,str('STR_SETTINGS_TFORMAT'));
        $fields[] = $f;        
        $fields[] = new shFormField(FIELD_TYPE_CHECKBOX,$settings->allow_self_reg,SETTING_ALLOW_SELF_REG,str('STR_SETTINGS_SELFREF'));
        $fields[] = new shFormField(FIELD_TYPE_TEXT,$settings->timezone,SETTING_TIMEZONE,str('STR_SETTINGS_TIMEZONE'));       
        $fields[] = $f1;
        
        $selDomain = $settings->self_reg_company.','.$settings->self_reg_group;
        $field = new shFormField(FIELD_TYPE_SINGLE_DOMAINSELECTOR,$selDomain,'com_groups',str('STR_SETTINGS_DEFAULTDOMAIN'));
        $field->fieldOptions['noaggregates'] = true;
        $data['settings'] = settings()->getAssocArray();
        $fields[] = $field;
                
        nx_fire_event('EVENT_SETTINGS_FORM_ADD_FIELDS',array('fields'=>&$fields));
        
        return $fields;    
    }
    
    /**
     * Returns to the main page.
     *
     */
    function getRedirectLink()
    {
        return 'index.php';    
    }
        
    /**
     * Depends on global edit settings permission value. Usually on 
     * super administrators have access to this setting
     *
     * @return unknown
     */
    function authorize ()
    {
        $user = app()->getLoggedInUserObject();        
		if ($user)
		{
        	return $user->getPermissions()->isAllowed(URLACTION_SETTINGS,'');
		}
		else
		{
			return false;
		}
    }
        
    /**
     * Override to implement form submit for this object
     * @return bool Returns true if the form submission was successful 
     */
    function _submit ()
    {
        $settings = &settings();
        
        $settings->time_date_format_long = ps(SETTING_DATE_TIME_LONG);
        $settings->time_date_format_short = ps(SETTING_DATE_TIME_SHORT);
        $settings->date_format_long = ps(SETTING_DATE_LONG);
        $settings->date_format_short = ps(SETTING_DATE_SHORT);
        $settings->time_format = ps(SETTING_TIME);
        $settings->company_name = ps(SETTING_COMPANY_NAME);
        $settings->site_tagline = ps(SETTING_SITE_TAGLINE);
        $settings->admin_email = ps(SETTING_ADMIN_EMAIL);
        $settings->allow_self_reg = ps(SETTING_ALLOW_SELF_REG)=='1'?'1':'0';
        $settings->timezone = ps(SETTING_TIMEZONE)==''?'America/New_York':ps(SETTING_TIMEZONE);        
        
        $company = 1;
        $group = 1;
        $selectedComGroup = ps('comp_groups');
        if ($selectedComGroup)
        {
            list($company,$group) = split(',',$selectedComGroup);
        }
        
        $settings->self_reg_company = $company;
        $settings->self_reg_group = $group; 
        
		app()->invalidateTable('sh_settings');
		
        //$settings->list_view_default = $this->_page->getFormArgument('list_view_default')==''?0:1;
                 
        nx_fire_event('EVENT_SETTINGS_FORM_APPLY_FIELDS',array());
        
        $this->addNotification (str('STR_SETTINGS_UPDATED'));        
        return true;        
    }    
}
?>
