<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
/**
 * Attributes (also called Tags) are alternate ways of marking an item of any
 * kind with a certain qualifier to make future searching easier.  There
 * are many similarities between attributes and categories though one 
 * distinguishing trait is that categories are nestable.
 */   
class shAttribute extends iwDbTable
{
	/**
	 * Adds the following properties
	 * 	@arg id
	 * 	@arg name
	 * 	@arg description
	 * 	@arg attribute_type
	 * 	@arg status
	 */	 	 	 	 	 	 	
	protected function addProperties () 
	{
		parent::addProperties ();
		$this->_addProperty (PROP_ID,0);
		$this->_addProperty (PROP_NAME,'');
		$this->_addProperty (PROP_DESCRIPTION,'');
		$this->_addProperty (PROP_ATTRIBUTE_TYPE,0);
		$this->_addProperty (PROP_ATTRIBUTE_STATUS,ATTRIBUTE_STATUS_ACTIVE);
	}

	/** 
	 *	
	 */
	public function getTypeName ($asPlural) 
	{		
		return $asPlural ? str('STR_TAG_NAME_PLURAL'): str('STR_TAG_NAME');
	}
	
	/** 
	 *	Returns @c OBJECT_TYPE_ATTRIBUTE
	 */
	public function getType () 
	{
		return OBJECT_TYPE_ATTRIBUTE;
	}

	/**  
	 *	Returns the following key fields:
	 *		@arg @c attribute_id
	 *		@arg @c attribute_name
	 *		@arg @c attribute_description
	 *		@arg @c attribute_type
	 *		@arg @c status	 	 	 	 	 	
	 */
	public function getDBKeyValueArray ($includeDBKey) {
	
		$tableConstant = $this->getTableConstant();
		
		$fields = array
				(
					$GLOBALS[$tableConstant]['fields']['ATTRIBUTE_ID'] => $this->get (PROP_ID),
					$GLOBALS[$tableConstant]['fields']['ATTRIBUTE_NAME'] => $this->get (PROP_NAME),
					$GLOBALS[$tableConstant]['fields']['ATTRIBUTE_DESCRIPTION'] => $this->get (PROP_DESCRIPTION),
					$GLOBALS[$tableConstant]['fields']['ATTRIBUTE_TYPE'] => $this->get (PROP_ATTRIBUTE_TYPE),
					$GLOBALS[$tableConstant]['fields']['ATTRIBUTE_STATUS'] => $this->get (PROP_ATTRIBUTE_STATUS)
		);
						
		return $fields;
	}
	
	/**
	 * @see The getDBKeyValueArray function describes the fields that this
	 * 		function expects in the given array.	 	  
	 */
	public function setDBKeyValueArray ($dbFieldsAndValues) {
	
		$tableConstant = $this->getTableConstant();
		$this->set (PROP_ID, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ATTRIBUTE_ID']]);
		$this->set (PROP_NAME, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ATTRIBUTE_NAME']]);
		$this->set (PROP_DESCRIPTION, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ATTRIBUTE_DESCRIPTION']]);
		$this->set (PROP_ATTRIBUTE_TYPE, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ATTRIBUTE_TYPE']]);
		$this->set (PROP_ATTRIBUTE_STATUS, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ATTRIBUTE_STATUS']]);
		return true;
	}
	
	/**  
	 *	Returns @c ATTRIBUTE_STATUS_ACTIVE if $isRestoring is true and 
	 *				returns @c ATTRIBUTE_STATUS_DELETED otherwise.	  
	 */
	protected function getDBKeyValueForDelete ($isRestoring=false) 
	{
		$fields = array ($GLOBALS[$this->getTableConstant()]['fields']['ATTRIBUTE_STATUS'] => $isRestoring?ATTRIBUTE_STATUS_ACTIVE:ATTRIBUTE_STATUS_DELETED);								
		return $fields;
	}
	
	/**  Gets the key string for the associative array that contains field information for the table associated with this object.
     *	@see tables.inc.php
     *	@return string The key field string for the assocative array that contains field information for the table associated with this object.
	*/
	function getTableConstant () {
		return 'TABLE_ATTRIBUTES';
	}
	
	/**  Checks if the data stored in this object is valid.
     *	All this does is verify that there is valid data in the *required*
     *	fields.
     *	@param bool $checkKey If this is true, then the unique key field for the object is checked for validity as well
     *	@return bool True, if all required fields are valid.
	*/
	function requiredFieldsValid ($checkKey = false) {
	
		$invalidField = false;
		
		if ($this->get(PROP_NAME) == '') {
			$this->addError (str('STR_ATTRIB_NONAME'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}
	
		return !$invalidField;
	}
	
	/**  
	 * If fieldname is true, then this will return @c 'attribute_id' string
	 * If fieldname is false, then this will return an integer	 	
	 */
	public function getUniqueID ($fieldName=false) 
	{
		if ($fieldName) {
			return $GLOBALS[$this->getTableConstant()]['fields']['ATTRIBUTE_ID'];
		}
		else {
			return $this->get (PROP_ID);
		}
	}
	
	/**
	 * Gets the number of documents that have this tag associated with them.
	 * @return int Returns the number of documents
	 */	 	 	
	public function getDocumentCount ()
	{				
		$itemsWithAttribute = new shItemDocumentCollection();
		$collectionParams = new iwParameterCollection();
		
		$p = new iwParameterArray(FILTER_ATTRIBUTES);
		$p->addItem_nokey ($this->get(PROP_NAME));
		$collectionParams->add($p);
		
		$p = new iwParameterArray(FILTER_STATUS);
		$p->addItem_nokey (ITEM_STATUS_ACTIVE);
		$p = $collectionParams->add($p);
				
		if (!app()->isAdminLoggedIn())
		{
			$p = new iwParameterArray(FILTER_USER);
			$p->addItem_nokey (app()->getLoggedInUserObject()->get(PROP_USERNAME));
			$p = $collectionParams->add($p);			
		}		
				
		// This is a costly operation and we may want to remove it. 
		$itemsWithAttribute->load($collectionParams, false);
		return $itemsWithAttribute->getTotal();
	}
		
	/**
	 * Displays all the details of this attribute in a full page format
	 */	 		
	public function render ($params=false)
	{						
		$username = app()->getLoggedInUserObject()->get(PROP_USERNAME);
		app()->notify (EVENT_OBJECT_VIEWED,array('type'=>$this->getType(),'id'=>$this->getUniqueID(),'username'=>$username));
		
		$html = '';
		$item = array ('id'=>$this->get(PROP_ID), 
							'name'=>$this->get(PROP_NAME), 
							'description'=>$this->get(PROP_DESCRIPTION),
							'type'=>$this->get(PROP_ATTRIBUTE_TYPE),
							'icon'=>$this->getItemIcon(true));

		$smarty = app()->getSmartyObject ();
		$smarty->assign ('attribute',$item);
									
		$username = app()->getLoggedInUserObject()->get(PROP_USERNAME);
		app()->notify (EVENT_OBJECT_VIEWED,array('type'=>$this->getType(),'id'=>$this->getUniqueID(),'username'=>$username));
                        
		$items = new shItemDocumentCollection();
			
        $query = 'select sh_items.* from sh_items, sh_item_attribute_map where sh_item_attribute_map.item_id=sh_items.item_id and 
                        sh_item_attribute_map.attribute_id='.$this->get(PROP_ID);
                        
        $results = db()->CacheGetAll(DB_CACHE_EXPIRE,$query);
        foreach($results as $r)
        {
            if (shAuthorize::canViewDocument($username,$r['item_id']))
            {
                $doc = new shItemDocument;
                if ($doc->setDBKeyValueArray($r))
                {
                    $items->addItem($doc);  
                }
            }
        }
        		
		$page = @$_GET['page'];
		if ($page == '')
		{
			$page = 1;
		}
		$perPage = 25;
		
		$documentList = $items->html($perPage,$page,false);
		$smarty->assign('document_list',$documentList);
		
		return smarty_templ_fetch('views/attribute.tpl.html',$smarty);												
	}		
	
	/**
	 * 
	 */
	public function getItemIcon ($large=true)
	{
				
		if ($large)
		{
			$icon = 'attrib_icon_large.jpg';
		}
		else
		{
			$icon = 'attrib_icon_small.jpg';
		}
		$icon = app()->getTemplateFile('images/'.$icon);
		return $icon;	
	}	
	
	/**
	 * Returns the following available actions for all users:
	 * 	@arg View
	 * Returns the following available actions for admin users only:	 
	 * 	@arg Edit
	 * 	@arg Send To Trash	 	 	  	
	 */	 
	public function getAvailableActions ($data='')
	{		
		$actions =  
			array (
				shPage::makeActionArray (str('STR_ACTION_VIEW'), 
								 app()->getTemplateFile('images/view_small.jpg'),
								'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWATTRIB.'&'.URLARG_ID.'='.$this->get(PROP_ID),
								'', 
								str('STR_ACTION_VIEW'))								
			);
				
			
		
		if (app()->isAdminLoggedIn())
		{
			$actions[] = 
			shPage::makeActionArray (str('STR_ACTION_EDIT'), 
							 app()->getTemplateFile('images/edit_small.jpg'),
							'index.php?'.URLARG_ACTION.'='.URLACTION_MODATTRIB.'&'.URLARG_ID.'='.$this->get(PROP_ID),
							'', 
							str('STR_ACTION_EDIT'));
						
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_TRASH'), 
							 app()->getTemplateFile('images/trash_small.jpg'),
							 'javascript:areYouSure(\''.str('STR_ASK_DELETE_ONE').'\', \'index.php?'.URLARG_ACTION.'='.URLACTION_DELATTRIB.'&'.URLARG_ID.'='.$this->get(PROP_ID).'\');',
							 '', 
							 str('STR_ACTION_TRASH'));
							
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_VIEWHISTORY'), 
							 app()->getTemplateFile('images/history_small.jpg'),
							 'index.php?a=view_mod&filter_object_type='.OBJECT_TYPE_ATTRIBUTE.'&filter_object_id='.$this->get(PROP_ID),
							 '', 
							 str('STR_ACTION_VIEWHISTORY'));		
		}				
		
		return $actions;
	}
		
	/**
	 *
	 */	 	
	public function renderAsListItem ($selectable, $details, $actions=true)
	{
		$img = $this->getItemIcon(false);						
		
		$description = iwUtilities::trucateWithEllipsis (strip_tags ($this->get(PROP_DESCRIPTION)), 125);
		if ($description == '')
		{
			$description = str('STR_NO_DESCRIPTION');
		}
		
		$link = 'index.php?a='.URLACTION_VIEWATTRIB.'&id='.$this->get(PROP_ID);
				
		$smarty =  app()->getSmartyObject();
		$smarty->assign ('collection_item_id', $this->get(PROP_ID));
		$smarty->assign ('collection_item_name', $this->get(PROP_NAME));
		$smarty->assign ('collection_item_show_details', $details);
		$smarty->assign ('collection_item_image', $img);
		$smarty->assign ('collection_item_elementid', TYPE_PREFIX_TAG.'_'.$this->get(PROP_ID).'_name');
		$smarty->assign ('collection_item_selectable', $selectable);
		$smarty->assign ('collection_array_name', get_class ($this));
		$smarty->assign ('collection_item_specialinfo',  sprintf(str('STR_TAG_DOCSWITHTAG'),$this->getDocumentCount()));
		$smarty->assign ('collection_item_description', $description);
		$smarty->assign ('collection_item_actions', $this->getAvailableActions());
		$smarty->assign ('collection_item_droptype', 'group');
		$smarty->assign ('collection_item_viewlink', $link);		
		$smarty->assign ('collection_item_draggable', true);
		$collection_item_html = smarty_templ_fetch('collections/collection.item.tpl.html',$smarty);
								
		return $collection_item_html;		
	}	
}

?>
