<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shModRecord extends iwModRecord
{
	function getAvailableActions ($data='')
	{
		$actions =  array();
		return $actions;
	}
	
	function getItemIcon ($large=true)
	{
		$action = $this->get('action');
		
		$fname = '';
		if (strcasecmp ($action, EVENT_OBJECT_ADDED)==0)
		{
			$fname = 'add_small.jpg';
		}
		else if (strcasecmp ($action, EVENT_OBJECT_EDITED)==0)
		{
			$fname = 'edit_small.jpg';
		}
		else if (strcasecmp ($action, EVENT_OBJECT_DELETED)==0)
		{
			$fname = 'delete_small.jpg';
		}
		else if (strcasecmp ($action, EVENT_OBJECT_TRASHED)==0)
		{
			$fname = 'trash_small.jpg';
		}
		else if (strcasecmp ($action, EVENT_OBJECT_VIEWED)==0)
		{
			$fname = 'view_small.jpg';
		}
		else if (strcasecmp ($action, EVENT_OBJECT_DOWNLOADED)==0)
		{
			$fname = 'download_small.jpg';
		}
		else if (strcasecmp ($action, EVENT_LOGGED_IN)==0)
		{
			$fname = 'login_small.jpg';
		}
		else if (strcasecmp ($action, EVENT_LOGGED_OUT)==0)
		{
			$fname = 'logout_small.jpg';
		}
		
		return app()->getTemplateFile('images/'.$fname);
	}	
	
	function getActionDescription()
	{
		$name = '';
		$type = '';
		switch ($this->get('target_type'))
		{
			case OBJECT_TYPE_COMPANY:
				$company = new shCompany;
				$company->set(PROP_ID, $this->get('target_id'));
				if ($company->load())
				{
					$name = $company->get(PROP_NAME);
				}

				$type = $company->getTypeName(false);
				break;

			case OBJECT_TYPE_GROUP:
				$group = new shGroup;
				$group->set(PROP_ID, $this->get('target_id'));
				if ($group->load())
				{
					$name = $group->get(PROP_NAME);
				}
				$type = $group->getTypeName(false);
				break;
			case OBJECT_TYPE_ITEM:
				$it = new shItemDocument;
				$it->set(PROP_ID, $this->get('target_id'));
				if ($it->load())
				{
					$name = $it->get(PROP_NAME);
				}
				$type = $it->getTypeName(false);
				break;
			case OBJECT_TYPE_CATEGORY:
				$cat = new shCategory;
				$cat->set(PROP_ID, $this->get('target_id'));
				if ($cat->load())
				{
					$name = $cat->get(PROP_NAME);
				}
				$type = $cat->getTypeName(false);
				break;
			case OBJECT_TYPE_SHARE_USER:
				$user = new shUser;
				$user->set(PROP_ID, $this->get('target_id'));
				if ($user->load())
				{
					$name = $user->get(PROP_USERNAME);
				}
				$type = $user->getTypeName(false);
				break;
		}

        $translatedAction = $this->translateEventName($this->get('action'));
        
		if ($type == '' && $name == '')
		{
			return $translatedAction;
		}
		else if ($type != '' && $name == '')
		{
			$link = '<a href="index.php?a=view_mod&filter_object_type='.$this->get('target_type').'&filter_object_id='.$this->get('target_id').'">%s</a>';
			return sprintf ($link,$translatedAction. ' '.$type.' (with ID '.$this->get('target_id').')');
		}
		else
		{
			$link = '<a href="index.php?a=view_mod&filter_object_type='.$this->get('target_type').'&filter_object_id='.$this->get('target_id').'">%s</a>';
			return sprintf ($link,$translatedAction.' '.$type.' "'.$name.'"');
		}		
	}
	
    public function translateEventName($event)
    {
        switch($event)
        {
            case EVENT_OBJECT_ADDED:
                return str('STR_EVENT_ADDED');
                
            case EVENT_OBJECT_EDITED:
                return str('STR_EVENT_EDITED');
                
            case EVENT_OBJECT_DELETED:
                return str('STR_EVENT_DELETED');
                
            case EVENT_OBJECT_TRASHED:
                return str('STR_EVENT_TRASHED');
                
            case EVENT_OBJECT_VIEWED:
                return str('STR_EVENT_VIEWED');
                
            case EVENT_OBJECT_DOWNLOADED:
                return str('STR_EVENT_DOWNLOADED');
                
            case EVENT_LOGGED_OUT:
                return str('STR_EVENT_LOGOUT');
                
            case EVENT_LOGGED_IN:
                return str('STR_EVENT_LOGIN');
                
            default:
                return $event;
        }
    }
    
	public function renderAsListItem ($selectable, $details, $actions=true)
	{
		$img = $this->getItemIcon(false);						
										
		$action = $this->getActionDescription();
		
		$bc = new iwBrowserCap ($this->get('client_info'));
		$description = $bc->property ('long_name').' '.
						$bc->property('version').
						' ('.$bc->property('platform').' '.$bc->property('os').')';
						
		$temp = str('STR_BY').' <a href="index.php?a=view_mod&filter_user='.$this->get(PROP_USERNAME).'">%s</a> '.str('STR_ON').' ';
		$temp .= iwUtilities::dateTimeFormat ($this->get('mod_date'), false);		
		$description = sprintf ($temp,$this->get(PROP_USERNAME));
	
		$smarty =  app()->getSmartyObject();
		$smarty->assign ('collection_item_id', $this->get(PROP_ID));
		$smarty->assign ('collection_item_name', $action);
		$smarty->assign ('collection_item_show_details', $details);
		$smarty->assign ('collection_item_image', $img);
		$smarty->assign ('collection_item_selectable', $selectable);
		$smarty->assign ('collection_array_name', get_class ($this));
		$smarty->assign ('collection_item_specialinfo', '');
		$smarty->assign ('collection_item_description', $description);
		$smarty->assign ('collection_item_actions', $this->getAvailableActions());
		$smarty->assign ('collection_item_droptype', 'mod');
		$smarty->assign ('collection_item_viewlink', '');		
		$smarty->assign ('collection_item_draggable', false);
		$collection_item_html = smarty_templ_fetch('collections/collection.item.tpl.html',$smarty);
								
		return $collection_item_html;		
	}	

	public function render ($params=false)
	{
		// Not currently supported
		return '';
	}
		
	/**
	 * Purges all mod records from the database that have the given date or earlier
	 * @param string $beforeDate The date (date, not time) before which records should be purged.
	 * @return bool Returns true if purge successful, false otherwise.
	 */
	static public function purgeHistory($beforeDate)
	{
		// Set the time to the end of the day
		$date = strtotime($beforeDate) + (60*60*23) + (60*59) + 59;				
		
		$query = 'delete from iw_modifications where date <= "'.iwUtilities::databaseDateFormat($date).'"';
		
		if (app()->getDatabase()->Execute($query))
		{
			return true;
		}
		
		return false;
	}
}

?>
