<?php

/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shUser extends iwUser {
	/*
	 * These values here are used for a very specific purpose.  When a 
	 * form is submitted with company/group data but there's another field
	 * that is invalid, the company/group info wasn't being refilled in
	 * to the correct form.  The reason, I think, is that PHP is having trouble
	 * with the fact that the company/group is being stored as objects and
	 * the IDs were being set on those.  It wasn't reinitializing contained
	 * objects with session data (just the top-level object).  So this allows 
	 * for data to be propogated throughout the life of this object in the 
	 * session.
	 */

	protected $_domain = null;

	protected $_permissions = null;

	/**
	* @desc Override to store information specific to nexus users
	*/
	function addProperties() {
		parent :: addProperties();

		$this->_domain = new shCompanyGroupPair();
		$this->_permissions = new shPermissions();

		$this->_addProperty(PROP_ITEMPOOL_SIZE, DEFAULT_ITEMPOOL_MAXSIZE_MB);
	}

	function get($key) 
    {
		if ($key == PROP_ID) 
        {
			// The ID for the user object is the username so 
			//	return that instead.
			return $this->get(PROP_USERNAME);
		} 
        else
        {
			if (($key == PROP_NAME) || ($key == PROP_DESCRIPTION)) 
            {
				$first = $this->get(PROP_FIRST_NAME);
				$last = $this->get(PROP_LAST_NAME);
				if ($first && $last) 
                {
					return $first . ' ' . $last;
				} 
                else 
                {
					return $this->get(PROP_USERNAME);
				}
			} 
            else 
            {
				return parent :: get($key);
			}
        }
	}

	public function isNumericId() {
		return false;
	}

	function set($key, $value) {
		if ($key == PROP_ID) {
			parent :: set(PROP_USERNAME, $value);
		} else {
			parent :: set($key, $value);
		}

	}

	/** Retrieves the classes friendly type name
	 * @param bool $asPlural True to return the plural version, false otherwise.
	 * @return string The name as a string
	 *  @access public
	 */
	function getTypeName($asPlural) {
		return $asPlural ? str('STR_USER_NAME_PLURAL') : str('STR_USER_NAME');
	}

	/** Returns an object type code of OBJECT_TYPE_UNKNOWN if unspecified
	 * @return int The object type value
	 * @access public
	 */
	function getType() {
		return OBJECT_TYPE_SHARE_USER;
	}

	function getDBKeyValueArray($includeDBKey) 
	{
		$tableConstant = $this->getTableConstant();

		// User table has additional fields - make sure and include them when retrieving data
		$fields = parent::getDBKeyValueArray($includeDBKey);
		$fields[$GLOBALS[$tableConstant]['fields']['USER_ITEMPOOL_MAX_SIZE_MB']] = $this->get(PROP_ITEMPOOL_SIZE);
		
		// Get the permissions data
		$this->_permissions->store($fields);
		
		return $fields;
	}

	function setDBKeyValueArray($dbFieldsAndValues) 
	{
		$tableConstant = $this->getTableConstant();
		$this->set(PROP_ITEMPOOL_SIZE, @ $dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['USER_ITEMPOOL_MAX_SIZE_MB']]);

		// Set the permissions data
		$this->_permissions->load($dbFieldsAndValues);

		return parent :: setDBKeyValueArray($dbFieldsAndValues);
	}

	function loadRelatedObjects() {

		// Load company and group from respective mapping tables
		$query = 'SELECT * FROM sh_user_access_map WHERE username="' . $this->get(PROP_USERNAME) . '"';
		$results = db()->GetRow($query);
		
		if ($results !== false) 
        {
			$group = new shGroup;
			$company = new shCompany;

			$group->set(PROP_ID, $results['group_id']);
			if ($results['group_id']) 
			{
				$group->load();
			}

			$company->set(PROP_ID, $results['company_id']);
			if ($results['company_id']) 
			{
				$company->load();
			}

			$this->setDomain($company, $group);
		}
		else 
		{
			$this->addErrorObject($results, ERROR_TYPE_SERIOUS);
		}

		return true;
	}

	function load() {
		if (parent::load()) 
        {
			$this->loadRelatedObjects();
			return true;
		}

		return false;
	}

	function _updateMappings() 
    {
		if ($this->_deleteMappings()) 
        {
			// Add copmany and group to respective mapping tables.
			$fields_values = array (
				'username' => $this->get(PROP_USERNAME
			), 'group_id' => $this->getGroup()->get(PROP_ID), 'company_id' => $this->getCompany()->get(PROP_ID));

			$ret = app()->getDatabase()->AutoExecute('sh_user_access_map', $fields_values, 'INSERT');
            app()->invalidateTable('sh_user_access_map');
            
			if ($ret) 
            {
				return true;
			} 
            else 
            {
				iwUtilities :: logDatabaseError();
			}
		}
	}

	function add() {
		if (parent :: add()) {
			$this->_updateMappings();
			return true;
		}

		return false;
	}

	function update() {
		if (parent :: update()) {
			$this->_updateMappings();
			return true;
		}
	}

    protected function deleteNotificationReferences()
    {
        $query = 'delete from sh_category_user_notification_map where username=\''.$this->get(PROP_USERNAME).'\'';
        $res = db()->Execute($query);
        if (!$res)
        {
            return false;
        } 
        
        return true;
    }
    
    public function onAfterPermanentDelete()
    {
        $this->_deleteMappings();  
        $this->deleteNotificationReferences();        
    }

	function _deleteMappings() {

		// Remove existing mappings first
		$res = app()->getDatabase()->query("DELETE FROM sh_user_access_map WHERE username='" . $this->get(PROP_USERNAME) . "'");
		if (($res)) {
			return true;
		} else {
			$this->addErrorObject($res, ERROR_TYPE_SERIOUS);
		}

		return false;

	}

	function render($params = false) {
		$html = '';
		$item = array (
			'username' => $this->get(PROP_USERNAME
		), 'first_name' => $this->get(PROP_FIRST_NAME), 'last_name' => $this->get(PROP_LAST_NAME), 'email' => $this->get(PROP_EMAIL), 'company' => $this->getCompany()->get(PROP_NAME), 'company_id' => $this->getCompany()->get(PROP_ID), 'group_id' => $this->getGroup()->get(PROP_ID), 'group' => $this->getGroup()->get(PROP_NAME), 'actions' => $this->getAvailableActions());

		$username = app()->getLoggedInUserObject()->get(PROP_USERNAME);
		app()->notify(EVENT_OBJECT_VIEWED, array (
		'type' => $this->getType(), 'id' => $this->getUniqueID(), 'username' => $username));

		$smarty = app()->getSmartyObject();
		$smarty->assign('user', $item);
		return smarty_templ_fetch('views/user.tpl.html', $smarty);
	}

	function getFullName() {
		$lastname = $this->get(PROP_LAST_NAME);
		$firstname = $this->get(PROP_FIRST_NAME);

		if ($lastname != '' && $firstname != '') {
			return $lastname . ', ' . $firstname;
		} else
			if ($lastname != '') {
				return $lastname;
			} else
				if ($firstname != '') {
					return $firstname;
				}

		return $this->get(PROP_USERNAME);
	}

	/**
	 * Retrieves the icon 
	 * @param bool $large True if the larger icon is to be retrieved.
	 * @return unknown
	 */
	function getItemIcon($large = true) {

		if ($large) {
			$icon = 'user_icon_large.jpg';
		} else {
			$icon = 'user_icon_small.jpg';
		}
		$icon = app()->getTemplateFile('images/' . $icon);
		return $icon;
	}

	function getAvailableActions($data = '') {
		$actions = array (
			shPage :: makeActionArray(str('STR_ACTION_VIEW'),
		app()->getTemplateFile('images/view_small.jpg'), 'index.php?' .
		URLARG_ACTION . '=' . URLACTION_VIEWUSER . '&' . URLARG_ID . '=' . $this->get(PROP_ID), '', str('STR_ACTION_VIEW')));

		if (app()->isAdminLoggedIn()) {
			$actions[] = shPage :: makeActionArray(str('STR_ACTION_EDIT'), app()->getTemplateFile('images/edit_small.jpg'), 'index.php?' .
			URLARG_ACTION . '=' . URLACTION_MODUSER . '&' . URLARG_ID . '=' . $this->get(PROP_ID), '', str('STR_ACTION_EDIT'));

			$actions[] = shPage :: makeActionArray(str('STR_ACTION_TRASH'), app()->getTemplateFile('images/trash_small.jpg'), 'javascript:areYouSure(\''.str('STR_ASK_DELETE_ONE').'\', \'index.php?' .
			URLARG_ACTION . '=' . URLACTION_DELUSER . '&' . URLARG_ID . '=' . $this->get(PROP_ID) . '\');', '', str('STR_ACTION_TRASH'));

			$actions[] = shPage :: makeActionArray(str('STR_ACTION_VIEWHISTORY'), app()->getTemplateFile('images/history_small.jpg'), 'index.php?a=view_mod&filter_object_type=' .
			OBJECT_TYPE_USER . '&filter_object_id=' . $this->get(PROP_ID), '', str('STR_ACTION_VIEWHISTORY'));

		}

		return $actions;
	}

	function renderAsListItem($selectable, $details, $actions=true) 
    {
		$img = $this->getItemIcon(false);

		$company = $this->getCompany();
		$group = $this->getGroup();

		$link = 'index.php?a=' . URLACTION_VIEWUSER . '&id=' . $this->get(PROP_ID);

		$groupLink = 'index.php?' . URLARG_ACTION . '=' . URLACTION_VIEWGROUP . '&' . URLARG_ID . '=' . $group->get(PROP_ID);
		$companyLink = 'index.php?' . URLARG_ACTION . '=' . URLACTION_VIEWGROUP . '&' . URLARG_ID . '=' . $company->get(PROP_ID);
		;

		$groupName = $group->get(PROP_NAME);
		$groupImage = $group->getItemIcon(false);
		$companyName = $company->get(PROP_NAME);
		$companyImage = $company->getItemIcon(false);

		$description = ucwords($group->getTypeName(false)).': <a href="' . $groupLink . '">' . $groupName . '</a>';
		$description .= '&nbsp;&nbsp;&nbsp;'.ucwords($company->getTypeName(false)).': <a href="' . $companyLink . '">' . $companyName . '</a>';

		$smarty = app()->getSmartyObject();
		$smarty->assign('collection_item_id', $this->get(PROP_ID));
		$smarty->assign('collection_item_name', $this->get(PROP_NAME));
		$smarty->assign('collection_item_show_details', $details);
		$smarty->assign('collection_item_image', $img);
		$smarty->assign('collection_item_selectable', $selectable);
		$smarty->assign('collection_array_name', get_class($this));
		$smarty->assign('collection_item_specialinfo', 'Username is <b>' . $this->get(PROP_USERNAME) . '</b>');
		$smarty->assign('collection_item_description', $description);
		$smarty->assign('collection_item_actions', $this->getAvailableActions());
		$smarty->assign('collection_item_droptype', 'user');
		$smarty->assign('collection_item_viewlink', $link);
		$smarty->assign('collection_item_draggable', true);
		$collection_item_html = smarty_templ_fetch('collections/collection.item.tpl.html', $smarty);

		return $collection_item_html;
	}

	/**
	* @desc Gets the group to which this user belongs. Will 
	*       always return a group object even if one has not
	*       been set yet (but it will have an id of 0)
	*/
	public function getGroup() {
		return $this->_domain->getGroup();
	}

	/**
	* @desc Gets the company to which this user belongs.  Will 
	*       always return a company object even if one has not
	*       been set yet (but it will have an id of 0)
	*/
	public function getCompany() {
		 return $this->_domain->getCompany();
	}

	/**
	* @desc Gets permission information for this user in the
	*       given domain.
	* @return shPermissions The permissions object associated with
	*           this user. If not set, defaults to NO PERMISSIONS
	*/
	public function getPermissions() {
		return $this->_permissions;
	}

	/**
	* @desc Gets permission information for this user in the
	*       given domain.
	* @param shPermissions The new permissions object to be 
	*           associated with this user. 
	*/
	public function setPermissions(shPermissions $permissions) {
		$this->_permissions = $permissions;
	}

	/**
	* @desc Sets the domain of this user.  Each user
	*       has a domain that includes a company and 
	*       associated group.
	*/
	public function setDomain($company, $group) {
        
		if (is_numeric($company)) {
			$companyOb = new shCompany;
			$companyOb->set(PROP_ID, $company);
			if (!$companyOb->load()) {
				$companyOb = null;
			}
		} else
			if ($company instanceof shCompany) {
				$companyOb = $company;
			}

		if (is_numeric($group)) {
			$groupOb = new shGroup;
			$groupOb->set(PROP_ID, $group);
			if (!$groupOb->load()) {
				$groupOb = null;
			}
		} else
			if ($group instanceof shGroup) {
				$groupOb = $group;
			}

		if ($groupOb && $companyOb) {
			$this->_domain->setGroup($groupOb);
			$this->_domain->setCompany($companyOb);
		}
	}
	
	/**
	 * Determines if the given user object has the same domain
	 * as this user object.
	 * @return bool Returns true if domains match, false otherwise.
	 * @param $user shUser The user object to compare with.
	 */
	public function userHasMatchingDomain(shUser $user)
	{
		if (($this->getCompany()->id == $user->getCompany()->id) &&
			($this->getGroup()->id == $user->getGroup()->id))	
		{
			return true;
		}
		
		return false;
	}
	
	/**
	 * Gets all the categories that this user has access to  
	 *
	 * @return array An array of shCategory records
	 */
	public function getAuthorizedCategories()
	{
		$permissionGroup = $this->getPermissions()->getPermissionGroup();
		
		$categories = new shCategoryCollection;
		$params = new iwParameterCollection();
		$params->add(new iwParameter(FILTER_USER,array($this)));		            
        $params->add(new iwParameter(FILTER_SORTBY,'name'));
        
		$categories->load($params);
		return $categories->items();
	}
}
?>
