<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

define ('kFormDisplayed',1);
define ('kFormNotDisplayed',2);
define ('kFormNoMatchingForm',3);
define ('kFormHandledContent',4);

class shPage extends iwPage
{
	public function showMessages ()
	{
		$errorData = array ();
		$errors = app()->getErrors();
		foreach ($errors as $error)
		{
			$msg = $error->message;
			$errorData[] = $msg;
		}
		
		$this->addSmartyVariable(SMARTY_ERRORS,$errorData);
		$this->addSmartyVariable(SMARTY_MESSAGES,app()->getNotifications());

		// Once we've prepared the errors for display, we have to assume
		//	that they should be removed from the queue
		app()->clearErrors();
		app()->clearNotifications();
	}	
	
	public function doFormAction ($canRedirect=true)
	{		
		// This will automatically figure out a form should
		//	be displayed or form data supplied by the user
		//	should be processed.  It will return the form
		//	object that should be used or null if no 
		//	form should be accessed.
		$form = $this->getForm();
		if (!$form) 
		{            
			return kFormNoMatchingForm;
		}
		
		// Verify that the current user has permission 
		//	to submit this form.
		if ($form->authorize())
		{						
			if ($form->isCancellingForm())
			{
				if ($canRedirect)
				{
					app()->redirectToLastLocation();
					
					// We should never get here.
					die ("Redirect failed.");
				}
				else
				{
					return kFormNotDisplayed;;
				}
			}
			else if ($form->isSubmittingForm())
			{
				if ($form->submitForm())
				{
					// We may not get here if the form did some redirection.
					
					if (!$form->redisplayOnSuccess())
					{
						// this indicates that the caller should
						//	not output anymore content - the form 
						//	handled it.
						return kFormHandledContent;
					}
				}
				else
				{
					// Form was not submitted successfully - redisplay
					//	the form by continuing to the next section.  The
					//	cached object will be retrieved and used to
					//	display previously entered data.

					// Make sure the form object wants to redisplay the form.
					if (!$form->redisplayOnError ())
					{
						return kFormNotDisplayed;;
					}
				}
			}

			$this->addContent($form->render());
			app()->pushBreadcrumb ($form->getTitle(),'');
			
			return kFormDisplayed;			
		}
		
		return kFormNotDisplayed;		
	}
		
	public function processStatusBar ()
	{
		
		$data = array ();

		if (app()->isUserLoggedIn())
		{
			$user = app()->getLoggedInUserObject();
			$this->addSmartyVariable (SMARTY_USER,array ('username'=> $user->get ('username')));
		}
		else
		{
			$this->addSmartyVariable (SMARTY_USER,'');
			$this->addSmartyVariable (SMARTY_LOGINURL,'index.php?a=login');
		}

		return $data;
	}

	public function login ()
	{
		
		app()->loginUser ($this->_formArguments[FORM_USERNAME], $this->_formArguments[FORM_PASSWORD],'shUser');
	}

	public function logout ()
	{
		
		app()->logoutUser ();
	}

	public function isLoggingOut()
	{
		return ($this->getUrlArgument(URLARG_ACTION) == URLACTION_LOGOUT);
	}

	public function isLoggingIn ()
	{
		return ($this->getUrlArgument(URLARG_ACTION) == URLACTION_LOGIN);
	}

	public function authorize ()
	{
		if (app()->isUserLoggedIn())
		{			
			$action = $this->getUrlArgument(URLARG_ACTION);
			$id = $this->getUrlArgument(URLARG_ID);
			
			$user = app()->getLoggedInUserObject();
			$permissions = $user->getPermissions();
			return $permissions->isAllowed($action, $id);
		}
		else
		{
			switch ($this->getUrlArgument(URLARG_ACTION))
			{
				case URLACTION_VIEWDOC:
					if ($this->getUrlArgument(URL_ID) != '')
					{
						// Trying to see a specific document.  Let's see if they have permission
						return shAuthorize::canViewDocument ('', $this->getUrlArgument(URL_ID));
					}
					else
					{
						// Displaying the list of docuements so allow - the lister will
						//	handle filtering out those documents that the user can't see.
						return true;
					}		
				case URLACTION_CONFIRMUSER:
					$user = new shUser;
					if ($user->confirmUser($this->getUrlArgument('hash'), $this->getUrlArgument('email')))
					{
						$this->addNotification(str('STR_USER_CONFIRMED'));
					}
					break;
				default:
					return false;
			}
		}
	}
	
	public function doUrlAction ()
	{
		$this->addSmartyVariable(SMARTY_ISFRONTPAGE, false);

		$id = $this->getUrlArgument(URL_ID);
		$isCollection = ($id == '');        
        
		$actions = new shActions();		
		$action = $this->getUrlArgument(URLARG_ACTION);
                
        $continue = true;
        $results = nx_fire_event('EVENT_EXECUTE_ACTION_PRE',array('action'=>$action,'id'=>$id,'page'=>&$this,'continue'=>&$continue));
        if (!$continue)
        {
            return true;
        }
                
        nx_fire_event('EVENT_EXECUTE_ACTION_PRE_CONTENT',array('action'=>$action,'id'=>$id,'page'=>&$this));
                
		switch ($action)
		{

        case URLACTION_SERVER_INFO:
            $this->displayServerInfo();
            break;
		case URLACTION_DELPOOLITEM:
			$item = new shPoolItem;
			$item->set(PROP_ID,$id);
			if ($item->load())
			{
				$item->enableTrueDeletion(true);
				$item->delete();
			}
			else
			{
				$this->addError (sprintf(str('STR_FILEBOX_CANNOTDELETE'), $id),ERROR_TYPE_SERIOUS);
			}
			app()->redirectToLastLocation();
			break;
						
		case URLACTION_DELDOC:
			if ($actions->doDeleteObject('shItemDocument',$id,false))
			{
				app()->redirectToLastLocation();
			}
			break;
		case URLACTION_DELATTRIB:
			if ($actions->doDeleteObject('shAttribute',$id,false))
			{
				app()->redirectToLastLocation();
			}
			break;
		case URLACTION_DELCOMPANY:
			if ($actions->doDeleteObject('shCompany',$id,false))
			{
				app()->redirectToLastLocation();
			}
			break;
		case URLACTION_DELGROUP:
			if ($actions->doDeleteObject('shGroup',$id,false))
			{
				//app()->redirectToLastLocation();
			}
			break;
		case URLACTION_DELCAT:
			if ($actions->doDeleteObject('shCategory',$id,false))
			{
				app()->redirectToLastLocation();
			}
			break;
		case URLACTION_DELUSER:
			if ($actions->doDeleteObject('shUser',$id,false))
			{
				app()->redirectToLastLocation();
			}
			break;
            
		case URLACTION_MULTIDOC_RESULTS:
			$this->displayMultiUploadResults();
			break;
			
		case URLACTION_EMPTYTRASH:
			
			$trash = new shTrashCollection;
			$trash->emptyTrash();
			break;
						
		case URLACTION_VIEWBIN:
			
			$actions->doListObjects(OBJECT_TYPE_BRIEFCASE);
			break;
			
		case URLACTION_VIEWPOOL:
			
			if ($isCollection)
			{
				$actions->doListObjects(OBJECT_TYPE_POOLITEM);
			}
			else
			{
				$actions->doViewObject('shPoolItem',$id);
			}
			
			break;
			
		case URLACTION_VIEWTRASH:
			
			$actions->doListObjects(OBJECT_TYPE_SPECIAL_TRASH);
			break;
			            
		case URLACTION_VIEWDOC:
			
			if ($isCollection)
			{
				$actions->doListObjects (OBJECT_TYPE_ITEM);
			}
			else
			{
				$actions->doViewObject('shItemDocument',$id);
			}
			break;
			
		case URLACTION_VIEWCAT:
			
			if ($isCollection)
			{
				$actions->doListObjects (OBJECT_TYPE_CATEGORY);
			}
			else
			{
				$actions->doViewObject('shCategory',$id);
			}
			break;
			
		case URLACTION_VIEWCOMPANY:
			
			if ($isCollection)
			{
				$actions->doListObjects (OBJECT_TYPE_COMPANY);
			}
			else
			{
				$actions->doViewObject('shCompany',$id);
			}
			break;
			
		case URLACTION_VIEWGROUP:

			if ($isCollection)
			{
				$actions->doListObjects(OBJECT_TYPE_GROUP);
			}
			else
			{
				$actions->doViewObject('shGroup',$id);
			}
			
			break;

		case URLACTION_VIEWUSER:
			
			if ($isCollection)
			{
				$actions->doListObjects(OBJECT_TYPE_USER);
			}
			else
			{
				$actions->doViewObject('shUser',$id);
			}
			
			break;
			
		case URLACTION_VIEWMOD:

			if ($isCollection)
			{
				$actions->doListObjects(OBJECT_TYPE_LOG);
			}
			else
			{
				$actions->doViewObject('shModRecord',$id);
			}
			
			break;
						
		case URLACTION_VIEWATTRIB:

			if ($isCollection)
			{
				$actions->doListObjects(OBJECT_TYPE_ATTRIBUTE);
			}
			else
			{
				$actions->doViewObject('shAttribute',$id);
			}
			
			break;
						
		case URLACTION_VIEWUSAGE:
			
//			if (isset ($data['report']))
//			{
//				app()->pushBreadcrumb ('Usage Report','index.php?'.URLARG_ACTION.'='.URLACTION_USAGEREPORT);
//				$this->addContent ($data['report']);
//			}
			break;
			
		case URLACTION_DESTROY:

			$type = $this->getUrlArgument(URL_TYPE);
			$type = base64_decode ($type);		
			$actions->doDeleteObject($type,$id,true);
			app()->redirectToLastLocation();
			
			break;
			
		case URLACTION_RESTORE:
			
			$type = $this->getUrlArgument(URL_TYPE);
			$type = base64_decode ($type);		
			$actions->doRestoreObject($type,$id,true);
			app()->redirectToLastLocation();
			
			break;
			
		case URLACTION_BROWSELIBRARY:
		
			$id = $this->getUrlArgument (URL_ID);
			$this->displayLibrary($id?$id:0);
			
			break;
			
		case URLACTION_BROWSEBYCAT:
			
			$this->displayBrowseByCategory();
			break;
			
		case URLACTION_BROWSEBYORG:
			
			$this->displayBrowseByOrg();
			break;
			
		case URLACTION_PURGEHISTORY:
		
			$actions->doPurgeHistory($this->getUrlArgument(URLARG_PURGEDATE));
			app()->redirectToLastLocation();									
			break;
        case URLACTION_USERMANAGER:
            
            $this->displayUserManager();            
            break;
            
        case URLACTION_DOCMANAGER:
            $this->displayDocumentManager();                    
            break;
        
		default:
			// No action to handle
			return false;
		}	        
        
		$this->addContent($actions->getOutput());
        
        nx_fire_event('EVENT_EXECUTE_ACTION_POST_CONTENT',array('action'=>$action,'id'=>$id));
		
		// An action was handled.
		return true;
	}	

    public function displayDocumentManager()
    {
        app()->pushBreadcrumb (str('STR_DOCMAN_CONSOLE'),app()->_getURLWithParameters(array()));
        $smarty = app()->getSmartyObject();

        // Prepare company list.
        $categories = new shCategoryCollection;
        $smarty->assign('category_selection',$categories->getCategoryFilterControl());
        
        // Prepare statistics
        $analysis = new shFileAnalysis;
        $docs = new shItemDocumentCollection;                
        $docs->load(new iwParameterCollection);
        $analysis->analyze ($docs);
        $stats = array();
        
        $avail = ((MAX_STORAGE_IN_MB*1024)*1024);
        $stats['total_size'] = iwUtilities::byteFormat($analysis->totalSizeInBytes);
        $stats['available_size'] = iwUtilities::byteFormat($avail);
        $stats['percentage_used'] = number_format(($analysis->totalSizeInBytes)/$avail, 2);
        $stats['total_docs'] = count($analysis->files);
        $stats['types'] = array();                
        foreach ($analysis->fileTypes as $fileTypeOb=>$count)
        {
            $fileType = new $fileTypeOb;
            $stats['types'][] = array('name'=>$fileType->getDescription(),'count'=>$count);
        }
        
        $smarty->assign('stats',$stats);   
        $this->addContent(smarty_templ_fetch('doc_management.tpl.html',$smarty));       
    }
    
    public function displayUserManager()
    {
        $smarty = app()->getSmartyObject();
        
        app()->pushBreadcrumb (str('STR_USERMAN_CONSOLE'),app()->_getURLWithParameters(array()));

        // Prepare company list.
        $companies = new shCompanyCollection;
        $companies->load(new iwParameterCollection());
        $companyInfo = array();
        foreach ($companies->items() as $item)
        {
            $companyInfo[] = array ('id'=>$item->get(PROP_ID), 'name'=>$item->get(PROP_NAME));
        }
        $smarty->assign(SMARTY_COMPANIES,$companyInfo);

        // Prepare Group list.
        $groups = new shGroupCollection;
        $groups->load (new iwParameterCollection());
        $groupInfo = array();
        foreach ($groups->items() as $item)
        {
            $groupInfo[] = array ('id'=>$item->get(PROP_ID), 'name'=>$item->get(PROP_NAME));
        }
        $smarty->assign(SMARTY_GROUPS,$groupInfo);

        // Prepare User list.
        $users = new shUserCollection;
        $users->load (new iwParameterCollection());
        foreach ($users->items() as $user)
        {
            $userInfo[] = array ('id'=>$user->get(PROP_USERNAME), 'name'=>$user->get(PROP_USERNAME));
        }
        
        $smarty->assign(SMARTY_USERS,$userInfo);        
        $this->addContent(smarty_templ_fetch('user_management.tpl.html',$smarty));
    }
    
   
	/**
	 * Displays the results of the multiple uploader.
	 */
	public function displayMultiUploadResults()
	{		
		$files = uploads()->getRecentFiles();
		
		$errors = array();
        $successes = array();
		foreach ($files as $file)
		{
            $ar = $file->toArray();
            if ($file->uploadResult == UPLOAD_RES_SUCCESS)
            {
                
                if ($file->databaseRecordId > 0)
                {
                    $action = ($file->uploadLocation==DEST_UPLOAD_LIBRARY)?URLACTION_VIEWDOC:URLACTION_VIEWPOOL;
                    $ar['link'] = 'index.php?'.URLARG_ACTION.'='.$action.'&'.URLARG_ID.'='.$file->databaseRecordId;
                }
                
                $successes[] = $ar;
            }
            else
            {
                $errors[] = $ar;
            }
		}
        
        $smarty = app()->getSmartyObject();
		$smarty->assign('success_files',$successes);
        $smarty->assign('error_files',$errors);
		$this->addContent(smarty_templ_fetch('multi_upload_results.tpl.html',$smarty));
	}
	
	public function displayBrowseByOrg()
	{
		app()->pushBreadcrumb(str('STR_BROWSEBYORG'));

		$step = $this->getUrlArgument('step');
		if ($step == '1' || $step == '')
		{
	        $field = new shFormField(FIELD_TYPE_SINGLE_DOMAINSELECTOR,'','com_groups','');
	        $field->fieldOptions['noaggregates'] = true;
			
			$smarty = app()->getSmartyObject();
			$smarty->assign('domain_selector',$field->getHtml());
			
			$this->addContent(smarty_templ_fetch('forms/form.browse.byorg.step1.html',$smarty));
		}
		else if ($this->getUrlArgument('step') == '2')		
		{	
			list($cid,$gid) = split(',',$this->getUrlArgument('com_groups'));
			$domain = new shCompanyGroupPair();
			if ($domain->load($cid,$gid))
			{
				$cname = $domain->getCompany()->get(PROP_NAME);
				$gname = $domain->getGroup()->get(PROP_NAME);
								
				$documents = new shItemDocumentCollection;				
				$params = new iwParameterCollection();
				
				$p = new iwParameterArray(FILTER_COMPANY);
				$p->addItem_nokey ($cid);				
				$params->add($p);
				
				$p = new iwParameterArray(FILTER_GROUP);
				$p->addItem_nokey ($gid);				
				$params->add($p);
                
				$documents->setTitle("Documents In $cname/$gname");
				
				$subtitle = str('STR_BROWSEBYORG_DESCRIPTION');
				$subtitle .= '<div><a href="index.php?a='.URLACTION_BROWSEBYORG.'">'.str('STR_CHOOSE_DIFFERENT_ORG').'</a></div>';
				$documents->setSubtitle($subtitle);
				$documents->setShowFilter(false);
				$documents->setIcon(shCompany::getItemIcon(true));
				
				$this->displayGenericCollection($documents,$params);								
			}						
		}
	}
	
	/**
	 * Displays a collection object in a standard form taking care of things 
	 * like pagination and sorting.
	 *
	 * @param iwObjectCollection $collection The collection to display
	 * @param iwParameterCollection $params Parameters such as start page, # per page, etc.
	 */
	protected function displayGenericCollection(iwObjectCollection $collection, 
												iwParameterCollection $params)
	{
		
		$page = $this->getUrlArgument(URL_PAGE,1);
		$perPage = 10;		
		$start = (($page-1) * $perPage);
		
		// Make sure that the collection has a chance to change the default
		//	parameters for the collection listing.
		$params->mergeParameters($collection->processFilters());							   
		
		$params->add (new iwParameter(URL_START,$start));
		$params->add (new iwParameter(URL_PERPAGE,$perPage));
				
		// Determine if the url indicates what the sort should be.
		$sortby = $this->getUrlArgument(URL_SORTBY);
		$sortdir = $this->getUrlArgument(URL_SORTDIR,'desc');
		if ($sortby)
		{
			$params->add (new iwParameter(FILTER_SORTBY,$sortby));
			$params->add (new iwParameter(FILTER_SORTDIR,$sortdir));
		}		

		$collection->load ($params);
		$this->addContent ($collection->html($perPage,$page));
	}

	/**
	 * Adds content to the page for displaying documents organized by category
	 */
	public function displayBrowseByCategory()
	{
		app()->pushBreadcrumb('Browse By Category');
		
		$finalArray = array();
		$categories = new shCategoryCollection;
        
        $p = new iwParameterCollection();
        $p->add(new iwParameter(FILTER_SORTBY,'name'));
        $p->add(new iwParameter(FILTER_SORTDIR,'asc'));
		$categories->load($p);
        
		$catObjects = $categories->items();
		foreach ($catObjects as $cat)
		{
			if (shAuthorize::canViewCategory(app()->getLoggedInUsername(), $cat->get(PROP_ID)))
			{			
				$array = $cat->toArray();
				$itemCollection = $this->getRecentDocuments(5,$cat->get(PROP_ID));
				$array['documents'] = $itemCollection->toArray();
				$finalArray[] = $array;
			}
		}

		$smarty = app()->getSmartyObject();
		$smarty->assign('categories',$finalArray);
		$this->addContent(smarty_templ_fetch('controls/control.view.documents.bycat.tpl.html',$smarty));
	}
	
	/**
	 * Gets the most recent documents (optionally) in a particular category
	 * @return shItemDocumentCollection The collection containing the info
	 * @param $max int The number of documents to return (max)
	 * @param $inCategoryId integer[optional] If > -1 then this is the category to return recent documents for
	 */
 	public function getRecentDocuments($max,$inCategoryId=-1)
	{
		$collection = new shItemDocumentCollection;
		
		$collectionParams = new iwParameterCollection();
		$collectionParams->add(new iwParameter(FILTER_SORTBY,'item_modified'));
		$collectionParams->add(new iwParameter(FILTER_START,0));
		$collectionParams->add(new iwParameter(FILTER_PERPAGE,$max));
		$collectionParams->add(new iwParameter(FILTER_SORTDIR,'desc'));
		
		$p = new iwParameterArray(FILTER_STATUS);
		$p->addItem_nokey (ITEM_STATUS_ACTIVE);
		$collectionParams->add($p);
		
		// Limit which documents are made visible.
		if (!app()->isAdminLoggedIn())
		{
			$p = new iwParameterArray(FILTER_USER,app()->getLoggedInUserObject());
			$collectionParams->add($p);			
		}
		
		if ($inCategoryId != -1)
		{
			$p = new iwParameterArray(FILTER_CATEGORY);
			$p->addItem_nokey ($inCategoryId);
			$collectionParams->add($p);
		}
		

		$collection->load ($collectionParams);
		return $collection;	
	}
	
	
	public function displayLibrary($categoryId)
	{
		$smarty = app()->getSmartyObject();
		$user = app()->getLoggedInUserObject();
		
		if (shAuthorize::canViewCategory($user->get(PROP_USERNAME), $categoryId))
		{
			$category = new shCategory;
			$category->set(PROP_ID, $categoryId);
			if ($category->load())
			{                                                         
				$data = $category->generateBrowserData();
				
				// Prepare the list of categories that are to be displayed in 
				// dialogs that have a field to select a category		
				$addCatDialogCategoryList = new shCategoryCollection;
				$addCatDialogCategoryList->load(new iwParameterCollection());
				
				$items = $addCatDialogCategoryList->toArray($categoryId); 
				for($i=0;$i<count($items);$i++)
				{
					$items[$i]['selected'] = ($categoryId == $items[$i]['id']);
				}
				$data['all_categories'] = $items;
				
				$data['can_add_categories'] = 
					shAuthorize::canAddCategoriesToCategory($user,$categoryId);				
				$data['can_add_documents'] = 
					shAuthorize::canAddDocumentsToCategory($user,$categoryId);
					
				$smarty->assign ('library', $data);

				$params = array();
				$params['viewtype'] = $this->getUrlArgument('viewtype');
				$smarty->assign('params',$params);
		
				$html = smarty_templ_fetch('library_browse.tpl.html',$smarty);
						
				$this->addContent ($html);
			}
		}
		else
		{
			$html = '<div class="importantMessage">'.str('STR_NO_PERMISSION').'</div>';
			$this->addContent ($html);
		}
	}
	
    /**
    * @desc Generates a standard 'action array' that can be used by templates
    * to display links to these actions. The action array has the following members
    *   - name: the name of the action (e.g. Edit)
    *   - icon: The relative path to the action icon (e.g. themes/default/images/dosomething.jpg)
    *   - link: The standard link to the action (e.g. index.php?a=do_something
    *   - special_link: A special link to the action (e.g. javascript:callFunction(1);))
    *   - desc: A description of the action
    *   - id: The id of the object being acted on.
    */
	static function makeActionArray ($actionName, $actionIcon, $actionLink, $specialLink, $actionDescription,$id='')
	{
		return array 
		(
			'name'=>$actionName,
			'icon'=>$actionIcon,
			'link'=>$actionLink,
			'special_link'=>$specialLink,
			'desc'=>$actionDescription,
			'id'=>$id
		);
	}
	
    /**
    * @desc Looks through an array of action arrays (see makeActionArray)
    *   to find an action of a particular type.
    *   @param int $actionId One of the ACTION_xxx defines
    *   @param array An array of action arrays (see makeActionArray)
    *   @return Returns an action array or false if not found.
    */
    static function findActionById($actionId,$actions)
    {
        foreach ($actions as $action)
        {
            if ($action['id'] == $actionId)
            {
                return $action;
            }
        }
        
        return false;
    }
    
	/**
	 * Determines which, if any, form object to display or process
	 * based on GET/POST variables in the current request.
	 * @return shForm Returns the form object or false if there is no
	 * 			form object requested.
	 */
	public function getForm()
	{
		// Now check to see if any of the standard forms
		//	have been submitted.
		$formFactory = new shFormFactory();		
		return $formFactory->makeForm();
	}
	
	/**
	 * This will add variable to the global template object
	 * that are almost always needed when displaying any page in  
	 * Nexus
	 */
	public function addPageTemplateVars()
	{
		$this->showMessages ();
		$this->addSmartyVariable (SMARTY_ITEMBININFO,array('count'=>shItemBin::getCount()));
				
		// Make sure we display the breadcrumb trail
		$this->addSmartyVariable (SMARTY_PAGETRAIL, app()->getTrailHtml());
		
		// Initialize the ajax system
		$this->addSmartyVariable (SMARTY_AJAXINIT, ajax()->getJavascript(IW_BASE_URL.'/include/components/xajax_0.5'));
		
		
		$this->addSmartyVariable(SMARTY_ISADMIN,app()->isAdminLoggedIn());
		$this->addSmartyVariable(SMARTY_LOGGEDIN, app()->isUserLoggedIn());		
	}
    
    /**
    * @desc Generates the html for displaying everything about the 
    *       server on which Nexus is installed.
    * @return string The html
    */
    public function displayServerInfo()
    {
        $smarty = app()->getSmartyObject();
        $smarty->assign('php',phpversion());
        $smarty->assign('php_memory_limit',iwUtilities::byteFormat(iwUtilities::phpIniSettingToBytes(ini_get('memory_limit'))));
        $smarty->assign('php_max_filesize',iwUtilities::byteFormat(iwUtilities::phpIniSettingToBytes(ini_get('upload_max_filesize'))));
        $smarty->assign('php_post_max_size',iwUtilities::byteFormat(iwUtilities::phpIniSettingToBytes(ini_get('post_max_size'))));
        
        $dbInfo = app()->getDatabase()->ServerInfo();
        $smarty->assign('db',$dbInfo['description']);
        
        $smarty->assign('base_url',IW_BASE_URL);
        $smarty->assign('base_path',IW_BASE_DIR);

        app()->pushBreadcrumb (str('STR_SERVER_INFO_TITLE'),'');        
        $this->addContent(smarty_templ_fetch('serverinfo.tpl.html',$smarty));
    }
}
?>
