<?php 
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shConversations 
{

	/**
	 * Edits the given information as a new post to the database.
	 * @param shItemDocument $item The item to associated with
	 * @param string $title The title of the post
	 * @param string $message The body of the post
	 * @param string $username The name of the user who is adding the post
	 * @param int $replyToId The ID of the post that this is a reply to.
	 * @return bool Returns true if added to database, false otherwise
	 */
	public function newPost ($item, $title, $message, $username, $replyToId=0)
	{
		$query = "insert into sh_item_conversations (title,body,username,creation_date, parent_post_id, item_id) VALUES ('$title', '$message', '$username', $replyToId, ".$item->get(PROP_ID).")";
		
		$result = app()->getDatabase()->query($query);
		if ( ($result))
		{
			return true;
		}
		
		return false;
	}
	
	/**
	 * Retrieves an array of posts associated with the given item
	 * 	@param mixed $item The item as an shItemDocument object or an ID
	 */
	public function getConversation($item, $asFlatList=true)
	{
		$id = 0;
		if (is_object($item))
		{
			$id = $this->get(PROP_ID);
		}
		else if (is_numeric ($item))		
		{
			$id = $item;
		}
		
		$postsAsFlatList = array();
		if ($id > 0)
		{
			$query = "select * from sh_item_conversations where item_id = $id order by creation_date desc";
            
            app()->flushCacheIfInvalid($query);
			$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
			if ($rows !== false)
			{
				foreach ($rows as $row)
				{
					$post = new shPost;
					if ($post->load($row))
					{
						$postsAsFlatList[] = $post;
					}					
				}
			}
		}
		
		if (!$asFlatList)
		{
			return $this->generateHierarchyFromFlatList($postsAsFlatList);
		}
		
		return $postsAsFlatList;
	}
		
	/**
	 * Takes the flat list of post objects and populates the 
	 * 'replies' field of each that has replies.  Then it returns
	 * 	hierarchical list of posts.
	 *	@param array $posts A flat list of posts.
	 *	@return array A hierarchical list of posts. 
	 */
	public function generateHierarchyFromFlatList (&$posts)
	{		
		$final = array();
		foreach ($posts as $post)
		{
			if ($post->parent_post_id == 0)
			{
				// This is a root post so get the replies recursively
				$this->populateReplies($post);
				
				// Add this post to the final arrya
				$final[] = $post;
			}
		}
		
		return $final;		
	}
	
	/**
	 * Takes a list of all posts and a target post and finds all the
	 * 	replies to that post.  And it does so recursively.  So if any
	 *	of the replies have replies, those posts will be populated with
	 *	replies as well.
	 *	@param array $allPosts The array of all shPost objects
	 *  @param shPost $parentPost The post for which replies should be found.
	 */
	public function populateReplies (&$allPosts, &$parentPost)
	{
		$replies = array();
		
		foreach ($allPosts as $post)
		{
			if ($post->parent_post_id == $parentPost->id)
			{
				$parentPost->replies[] = &$post;
				
				// now add the replies to the child post as well
				$this->populateReplies($allPosts, $post);
			}			
		}		
		
		return $replies;
	}
}

?>