<?php

class shFileTypeManager 
{
	public $_fileTypes = false;
	
	public function __construct()
	{		
		$this->_fileTypes = array(
				new shFileTypeArchive,				
				new shFileTypeExcel,
				new shFileTypeHtml,
				new shFileTypeImage,
				new shFileTypePdf,
				new shFileTypeRtf,
                new shFileTypeText,
				new shFileTypeWord,
                new shFileTypeBinary);      /** NOTE: BINARY SHOULD ALWAYS BE LAST SINCE IT CONTAINS A WILDCARD TYPE */
				
		nx_fire_event('EVENT_FILE_TYPES_NEEDED',array('file_types'=>&$this->_fileTypes));		
	}
	
	/**
	 * Uses the name of the file and the mime type to get the 
	 * the nexus type of the file (one of the DOCTYPE_ constants)
	 * @param string $filename The name of the file
	 * @param string $mimeType The mime type (if available)
	 * @return string One of the DOCTYPE_ constants (DOCTYPE_UNKNOWN if not found)
	 */
	public function determineNexusType($filename, $mimeType='')
	{
        $fileType = $this->fileTypeObjectFromFile($filename,$mimeType);
        if ($fileType)
        {
            return $fileType->getNexusFileType();
        }
        return DOCTYPE_UNKNOWN;		
	}	  
    
    /**
    * @desc Gets the filetype object that applies to the given 
    *       file.  Use the mimetype to more correctly identify the
    *       the file type.  If no mimetype available then the extension
    *       is used.
    * @param string $filename The name of the file
    * @param string $mimeType The mime type of the file (if available).
    * @return shFileType The file type object or null if the file type is not found.
    */
    public function fileTypeObjectFromFile($filename, $mimeType='')
    {
        foreach ($this->_fileTypes as $fileType)
        {            
            if ($mimeType)
            {                
                $mtArray = $fileType->getMimeType($filename);
                if (in_array('*',$mtArray) || 
                    in_array(strtolower($mimeType),$mtArray))
                {                    
                    return $fileType;
                }
            }
            
            $ext = pathinfo($filename,PATHINFO_EXTENSION);
            $extensions = $fileType->getFilenameExtensions();
            foreach($extensions as $ce)
            {                   
                if (strtolower($ce) == strtolower($ext))
                {                    
                    return $fileType;
                }
            }                    
        }
        
        return null;        
    }
}

?>