<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

define ('SETTING_DATE_TIME_LONG', 'time_date_format_long');
define ('SETTING_DATE_TIME_SHORT', 'time_date_format_short');
define ('SETTING_DATE_LONG', 'date_format_long');
define ('SETTING_DATE_SHORT', 'date_format_short');
define ('SETTING_TIME', 'time_format');
define ('SETTING_LISTVIEW_DEFAULT', 'list_view_default');
define ('SETTING_SITE_TAGLINE', 'site_tagline');
define ('SETTING_COMPANY_NAME', 'company_name');
define ('SETTING_ADMIN_EMAIL', 'admin_email');
define ('SETTING_ALLOW_SELF_REG', 'allow_self_reg');
define ('SETTING_SELF_REG_GROUP', 'self_reg_group');
define ('SETTING_SELF_REG_COMPANY', 'self_reg_company');
define ('SETTING_TIMEZONE', 'timezone');

/**
 * The settings object acts as a mediator between the web application and the
 * settings table.  Using PHP5 setter and getter calls, the user can set and 
 * get the various properties stored in the database.  If there is no database
 * connection then default values are loaded.
 */    
class shSettings
{
	/** Associative array for storing the settings by setting ID */
	protected $_settings;
	
	/**
	 * Loads all settings from the database.  If database connection fails, then
	 * default settings are loaded.
	 */	 	 	
	public function __construct()
	{
		$query = 'select setting_id,setting_value from sh_settings';
        
        app()->flushCacheIfInvalid($query);
		$results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);

		// start with defaults in case there's a problem retrieving them
		//	from the database.
		$this->_settings[SETTING_DATE_TIME_LONG] = 'F j, Y, g:i a';
		$this->_settings[SETTING_DATE_TIME_SHORT] = 'm/d/y g:i a';
		$this->_settings[SETTING_DATE_LONG] = 'F j, Y';
		$this->_settings[SETTING_DATE_SHORT] = 'm/d/y';
		$this->_settings[SETTING_TIME] = 'g:i a';
		$this->_settings[SETTING_LISTVIEW_DEFAULT] = '0';
		$this->_settings[SETTING_COMPANY_NAME] = 'My Company';
		$this->_settings[SETTING_SITE_TAGLINE] = 'Nexus Library of Documents';
		$this->_settings[SETTING_ADMIN_EMAIL] = 'nexus@iwonderdesigns.com';
		$this->_settings[SETTING_ALLOW_SELF_REG] = '0';
		$this->_settings[SETTING_SELF_REG_GROUP] = '1';
		$this->_settings[SETTING_SELF_REG_COMPANY] = '1';
        $this->_settings[SETTING_TIMEZONE] = 'America/New_York';
		       
        nx_fire_event('EVENT_GET_SETTINGS',array('settings'=>&$this->_settings));
                
		if ($results !== false)
		{
			foreach ($results as $row)
			{							
				$this->_settings[$row['setting_id']] = $row['setting_value'];												
			}
		}	
	}
	
	public function __set ($name, $value)
	{
		$settingExists = app()->getDatabase()->GetOne ('select count(*) from sh_settings where setting_id="'.$name.'"');
		if ($settingExists)
		{
			$query = 'update sh_settings set setting_value="'.$value.'" where setting_id="'.$name.'"';
		}
		else
		{
			$query = 'insert into sh_settings values("'.$name.'","'.$value.'")';
		}

		$result = app()->getDatabase()->query ($query);
		
		if (($result))
		{			
			$this->_settings[$name] = $value;
		}		
	}
	
	public function __get ($name)
	{
		if (isset ($this->_settings[$name]))
		{
			return $this->_settings[$name];
		}
		
		return '';
	}
	
	/**
	 * Gets the settings as an associative array.  This can be used for 
	 * smarty templates
	 * @return An associative array of settings
	 */	 
	public function getAssocArray ()
	{
		return $this->_settings;
	}
}

?>
