<?php

function smarty_init_obj ($template, &$smarty)
{
	$relPathToTemplateFile = $template;
	
    $dir = app()->get('template_dir');
    $customTemplate = $dir.'/'.$relPathToTemplateFile;
    if (!file_exists($customTemplate))
    {
    	$dir = 'themes/default';
    	$defaultTemplate = $dir.'/'.$relPathToTemplateFile;
    	if (file_exists($defaultTemplate))
    	{    		 
    		$smarty->template_dir = $dir;
    	}
    }
}

function smarty_deinit_obj (&$smarty)
{
	$smarty->template_dir = app()->get('template_dir');
}

function smarty_templ_fetch ($template, &$smarty)
{
	$relPathToTemplateFile = $template;
	
	smarty_init_obj($template,$smarty);
	$html = $smarty->fetch($relPathToTemplateFile);
	smarty_deinit_obj($smarty);		    		
    
    return $html;
}

function smarty_templ_display ($template, &$smarty)
{
	$old = error_reporting(E_ALL ^ E_NOTICE);
	smarty_init_obj($template,$smarty);
	$smarty->display($template);
	smarty_deinit_obj($smarty);
	error_reporting($old);
}

function smarty_templ_file_path($relPathToFile)
{
    $dir = app()->get('template_dir');
    $customTemplate = $dir.'/'.$relPathToFile;
    if (!file_exists($customTemplate))
    {
    	$dir = 'themes/default';
    	$defaultTemplate = $dir.'/'.$relPathToFile;
    	if (file_exists($defaultTemplate))
    	{
    		return $defaultTemplate;    		 
    	}
    	else
    	{
    		return '';
    	}
    }
    else
    {
    	return $customTemplate;
    }
}

function smarty_function_byte_format($params,&$smarty)
{
    if (!in_array('bytes', array_keys($params)))
    {
        $smarty->trigger_error("templ_path: missing 'bytes' parameter");
        return;
    }
    
    return iwUtilities::byteFormat($params['bytes']);
}

function smarty_function_templ_path($params, &$smarty)
{
    if (!in_array('file', array_keys($params))) 
    {
        $smarty->trigger_error("templ_path: missing 'file' parameter");
        return;
    }	

    $path = smarty_templ_file_path($params['file']);
    if ($path == '')
    {
    	$smarty->trigger_error("templ_path: no such file ($path) in templating system");
    }
    
    return $path;
}

function smarty_function_includealt($params, &$smarty)
{
    if (!in_array('file', array_keys($params))) 
    {
        $smarty->trigger_error("includealt: missing 'file' parameter");
        return;
    }
    $relPathToTemplateFile = $params['file']; 
    
    foreach ($params as $key=>$value) 
    {
    	if ($key != 'file')
    	{
    		$smarty->assign($key,$value);
    	}
    }	
    	
    return smarty_templ_fetch($relPathToTemplateFile,$smarty);
}

/**
 * Retrieves the html needed to display the requested form
 *
 * @param array $params One or more of the following
 * 			'formtype' - must be one of the FORMTYPE_X
 * 			'formname' - The name to use in the form
 * 			'async' - True for async false for synchronous 
 * @param unknown_type $smarty The smarty object
 */
function smarty_function_getform($params,&$smarty)
{	
	if (!in_array('formtype', array_keys($params)))
	{
		$smarty->trigger_error("getform: missing 'formtype' parameter");
		return;			
	}
	
	
	$type = $params['formtype'];
	$name = isset($params['formname']) ? $params['formname'] : '';
	$async = isset($params['async']) ? $params['async'] : false;

	$factory = new shFormFactory;
	$form = $factory->createFormFromType($type);
	
	if ($form)
	{
		$form->addParams($params);
		$form->setFormName($name);	
		$form->setAsync($async);
			
		return $form->render();
	}
	
	return '[FORM NOT FOUND]';
}

/**
 * Populates the smarty object with documents that 
 * available to the public (non-registered users)
 *
 */
function smarty_function_public_documents($params,&$smarty)
{
	$docSmarty = app()->getSmartyObject();
	$docs = new shItemDocumentCollection();
	$params = new iwParameterCollection;
	$params->add(new iwParameterArray(FILTER_COMPANY, COMPANY_UNREGISTERED));
	$params->add(new iwParameterArray(FILTER_GROUP, GROUP_ALL));	
	$docs->load($params);
	$docSmarty->assign('documents',$docs->toArray());
	
	return smarty_templ_fetch('controls/control.view.documents.flat.simple.tpl.html',$docSmarty);
}

/**
* @desc This return a translation for the given string code
* @param array $params The following parameter is required:
*           's' = The string code (e.g. STR_MY_STRING)
*           'a' = A comma delimited list of arguments for string that contain formatting codes (e.g. %s)
* @return string The translated string.
*/
function smarty_function_str($params,&$smarty)
{
    if (isset($params['s']))
    {
        $args = array();
        if (isset($params['a']))
        {
            $args = split(',',$params['a']);
        }
                        
        return str_templates($params['s'],$args);
    }
    else
    {
        return '';
    }
}

function smarty_function_navbar($params, &$smarty)
{
    $vertical = true;
    if (isset($params['vertical']))
    {
        $vertical = $params['vertical'];
    }
    
    $blocks = nx_get_nav_blocks($vertical);
    
    $html = '';
    
    $classPrefix = $vertical ? 'nav_block_vert': 'nav_block_horz';
    foreach($blocks as $block)
    {
        $html .= "<div class='$classPrefix'>\r\n";
        $html .= "<div class='{$classPrefix}_title'>$block->title</div>\r\n";
        
        if ($block->html != '')
        {
            $html .= $block->html;
        }
        
        foreach ($block->items as $item)
        {
            $html .= "<div class='{$classPrefix}_item'>";

            if ($item->html == '')
            {            
                $icon = '';
                if (isset($item->icon) && $item->icon != '')
                {
                    $img = new iwImage($item->icon);
                    if ($img->isValid())
                    {
                        $padding = $img->getWidth() + 4;
                    }
                    else
                    {
                        $padding=0;
                    }
                    
                    $icon = "style='display:block; padding-left: {$padding}px; background-image:url({$item->icon});background-repeat:no-repeat;'";
                }
                
                $html .= "<a $icon href='{$item->link}'>{$item->title}</a>";                
            }
            else
            {
                $html .= $item->html;
            }
            $html .= "</div>\r\n";
        }    
        
        
        $html .= "</div>";
    }
    
    return $html;
}

?>