<?php

class shLocalStorage extends shDocumentStorage
{
    public function __construct()
    {
        $this->_storageLocationId = 1;        
    }
    
    public function isLocalStorageSupported()
    {
        return true;
    }
    
    public function getFileSize(shItemDocument $doc)
    {
        return filesize($this->getStorageLocation($doc));
    }
    
    public function retrieveDocument(shItemDocument $doc)
    {
        $location = $this->getStorageLocation($doc);
        if (file_exists($location))
        {
            $this->setLastError(DS_ERR_NONE);
            return $location;
        }        
        else
        {
            $this->setLastError(DS_ERR_DOC_NOT_FOUND);
            return false;
        }        
    }

    public function getDocumentInfo(shItemDocument $doc)
    {
        $file = new shFileInfo;
        
        $documentPath = $this->getStorageLocation($doc);
        @list ($file->path, $file->name, $file->extension) = pathinfo ($documentPath);                    
        if (file_exists($documentPath))
        {        
            $file->lastAccessTime = @fileatime ($documentPath);
            $file->creationTime = @filectime ($documentPath);
            $file->lastModifiedTime = @filemtime ($documentPath);
            $file->size = @filesize ($documentPath);
            $type = typeman()->fileTypeObjectFromFile($documentPath);
            if ($type == null)
            {
                $file->type = 'shFileTypeBinary';                
            }
            else
            {
                $file->type = get_class($type);
            }        
        }
        else
        {
            $file->lastAccessTime = 0;
            $file->creationTime = 0;
            $file->lastModifiedTime = 0;
            $file->size = 0;
            $file->type = 'shFileTypeBinary';            
        }
        
        return $file;            
    }
    
    public function storeDocument(shItemDocument $doc)
    {
        $pathFrom = $doc->getTemporaryLocation();
        $pathTo = $this->getStorageLocation($doc);
        
        // rename the original
        if (file_exists($pathTo))
        {
            if (!@unlink($pathTo))
            {
                $this->setLastError(DS_ERR_EXISTING_DOC_DELETE_FAILED);
                return false;
            }
        }
        
		$result = false;
		if (is_uploaded_file($pathFrom))
		{
			$result = @move_uploaded_file($pathFrom,$pathTo);
		}
		else
		{
			$result = @copy($pathFrom,$pathTo);
		}
		
        if (!$result)
        {
            $this->setLastError(DS_ERR_LOCAL_COPY_FAILED);
            return false;
        }
        else                                              
        {           
            $this->setLastError(DS_ERR_NONE);
            return true;
        }
        
    }

    public function deleteDocument(shItemDocument $doc)
    {
        $loc = $this->getStorageLocation($doc);
        if (file_exists($loc))
        {
            return @unlink($loc);
        }
        
        return false;
    }    
    
    public function getStorageLocation(shItemDocument $doc)
    {
        $dir = '';
        if ($doc instanceof shPoolItem)
        {
            $dir = app()->get(PROP_FILEBOX_DIR);
        }
        else
        {
            $dir = app()->get (PROP_LIBRARY_DIR);
        }
        
        $rel = $doc->getInternalFileName();
        
        if ($rel != '')
        {
            return $dir.'/'.$rel;
        }        
        
        return '';
    }  
    
    /**
    * @desc The temporary file location is the actual location since
    *       it's local.  There's no need to copy it somewhere else.
    */
    protected function generateTemporaryFileLocation(shItemDocument $doc)
    {
        return $this->getStorageLocation($doc);
    }      
    
    /**
    * @desc The temp location is the final location so we do NOT want
    *       to delete it.  
    */
    public function removeTempDocument(shItemDocument $doc, $tempFilePath)
    {
        return true;
    }    
}

?>
