<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


/**
 * Starts the micro-second timer.  Call stopTimer at the end of the script
 */
function startTimer()
{
	$GLOBALS['perf_start_time'] = microtime(true);
}

/**
 * Stops the timer after startTimer was called and returns the number
 *	of seconds (fractional) between the two.
 *	@return float The number of seconds between now and the call to startTimer
 */
function stopTimer()
{
	$time_end = microtime(true);
	$time = $time_end - $GLOBALS['perf_start_time'];
	
	return $time;
}

startTimer();

/////////////////////////////////////////////////////
// Validate Installation
/////////////////////////////////////////////////////
// Redirect to installer page if there is no 
//  dbconfig file.  This means that either 
//  the installation got corrupted somehow
//  or this is the first time that they've 
//  visited the site since it was installed.
if (!file_exists ('config/dbconfig.php'))
{
    header ("Location: ./include/install/");
    exit ();
}

$revision = @file_get_contents('include/revision.txt');
if ($revision == '')
{
	die ('Invalid version of Nexus');
}
else
{
	@list($revision,$ignore) = split(':',$revision,2);
}

if (file_exists('config/user.config.php'))
{
	require_once('config/user.config.php');
}

define ('SHARE_VERSION','1.7.4.'.$revision);
define ('APPLICATION_NAME', 'Nexus');

/////////////////////////////////////////////////////
// Gather Code
/////////////////////////////////////////////////////

require_once ('engine/classes.inc.php');	
require_once ('engine/globals.inc.php');    
require_once ('config/share.config.php');
require_once ('include/engine/core/iwUtilities.class.php');
require_once ('api/api.php');

/////////////////////////////////////////////////////
// Secure Execution of Script
/////////////////////////////////////////////////////                            
unregister_GLOBALS();

define ('IW_HIDDEN_HASH','TudH1O39;*fda');

if (!defined('IW_DEBUG_STRICT'))
{
	define ('IW_DEBUG_STRICT', false);
}

/////////////////////////////////////////////////////
// Store Site Paths
/////////////////////////////////////////////////////
$path = $url = '';
 
getPaths (IWONDER_LIB_SOURCE_FILE,$path,$url);
define ('IW_BASE_DIR', $path);
define ('IW_BASE_URL', $url);

// For servers that have setup an include directory automatically in the php.ini file, this
//    will turn off that include directory so that there are not any conflicts with the included
//    PEAR library
$includePath = ini_get ('include_path');
ini_set ('include_path', IW_BASE_DIR.
                            PATH_SEPARATOR.
                            IW_BASE_DIR.'include/components/pear/'.
                            PATH_SEPARATOR.
                            IW_BASE_DIR.IWONDER_LIBRARY_REL_PATH.
                            PATH_SEPARATOR.
                            IW_BASE_DIR.'/include/');


/////////////////////////////////////////////////////
// Prepare Components
/////////////////////////////////////////////////////
                            
// ADODB
require_once ('components/adodb5/adodb.inc.php');
    
// PEAR
require_once ('PEAR.php');
require_once ('Zip.php');
require_once ('HTML_Safe.php');

// Xajax
require_once ('components/xajax_0.5/xajax_core/xajax.inc.php');

// phpMailer
require_once ('components/phpmailer/class.phpmailer.php');

// JSON Support (when not native)
require_once ('components/json/Services_JSON.class.php');

// Smarty
require_once ('components/smarty/libs/Smarty.class.php');
require_once ('engine/smarty/plugins.smarty.php');

// Nexus
require_once ('engine/const.inc.php');

/////////////////////////////////////////////////////
// Start PHP Session
/////////////////////////////////////////////////////

if (session_id() == "")
{
	session_name ('nexus_'.DEPLOYMENT_NAME);
	if (isset ($_GET['sessid']))
	{
		session_id(base64_decode($_GET['sessid']));
	}
	session_start ();
}

define ('SESSION_ID',session_id());

initializeGlobals ();

//////////////////////////////////////////////////////////////
// Prepare Ajax Object
//////////////////////////////////////////////////////////////
ajax()->processRequest();

//////////////////////////////////////////////////////////////
// Prepare Application object
//////////////////////////////////////////////////////////////
app()->set ('page_dir', 'include/engine/pages');
app()->set ('template_dir', 'themes/'.DEFAULT_THEME);
app()->set ('admin_email', settings()->admin_email);

define ('NEXUS_CURRENT_URL', app()->_getUrlWithParameters());

set_error_handler('handleError');

app()->registerPlugins();

?>