<?php
/**
* name: Automatic File Lock
* description: Lock files after a specified period of time.
* date: 4/1/6/2008
* version: 1.0
* author: iWonder Designs
*/
  
define ('GLOBAL_SETTING_NAME','filelock_timeout_days');

nx_register_handler('EVENT_PLUGIN_INIT','filelock_initialize');
nx_register_handler('EVENT_GET_SETTINGS','filelock_get_settings');  
nx_register_handler('EVENT_SETTINGS_FORM_ADD_FIELDS','filelock_get_settings_fields');
nx_register_handler('EVENT_SETTINGS_FORM_APPLY_FIELDS','filelock_apply_settings_fields');
nx_register_handler('EVENT_BROWSER_PRE_DOCUMENT_ADD','filelock_browser_pre_doc_add');
nx_register_handler('EVENT_DOCUMENT_ACTIONS_NEEDED','filelock_document_actions');
nx_register_handler('EVENT_FORMS_NEEDED','filelock_get_forms');
nx_register_handler('EVENT_DOCUMENT_VIEW_SINGLE_HEADER','filelock_get_document_header');
nx_register_handler('EVENT_BROWSER_PRE_DOCUMENT_DISPLAY','filelock_alter_browser_display');

require_once('FileLockForm.class.php');

function filelock_initialize()
{    
    // check to see that the file lock table is present
    $result = db()->Execute('select count(filelock_id) from sh_filelock_files');
    if (!$result)
    {
        // create the table.
        $result = db()->Execute('
            CREATE TABLE `sh_filelock_files` (
                `filelock_id` int(11) NOT NULL auto_increment,
                `expiration_date` datetime NOT NULL,
                `document_id` int(11) NOT NULL,
                `comment` varchar(255) NOT NULL,
                PRIMARY KEY  (`filelock_id`)
                ) ENGINE=InnoDB AUTO_INCREMENT=1 ;
            ');
            
        if (!$result)
        {
            $errObj = new iwError ('File Locking Plugin: Unable to create File Locking table in database');
            app()->addGlobalError($errObj);
            return false;
        }        
    }
    
    return true;
}

function filelock_get_settings($params)
{
    $settings = &$params['settings']; 
    $settings[GLOBAL_SETTING_NAME] = 2; // two days    
}

function filelock_apply_settings_fields($params)
{    
    settings()->__set(GLOBAL_SETTING_NAME,ps(GLOBAL_SETTING_NAME));
}

function filelock_get_settings_fields($params)
{
    $fields = &$params['fields'];    
    $fields[] = new shFormField(FIELD_TYPE_TEXT,settings()->__get(GLOBAL_SETTING_NAME),GLOBAL_SETTING_NAME,'Default File Lock Expiration(days)');    
}

function filelock_has_doc_expired($docId)
{
    $result = db()->GetOne('select expiration_date from sh_filelock_files where document_id='.$docId);
    if ($result !== false)
    {
        $expirationDate = strtotime($result);
        $currentDate = time();
        
        if ($expirationDate < $currentDate)
        {
            return true;
        }
    }         
    
    return false;
}

function filelock_browser_pre_doc_add($params)
{
    $icon = &$params['icon'];
    $doc = &$params['doc'];
    $show = &$params['visible'];   
    
    // Administrators can see files even if they are expired.
    if (!app()->isAdminLoggedIn())
    {
        if (filelock_has_doc_expired($doc->get(PROP_ID)))
        {
            $icon = 'plugins/filelock/expired_doc.gif';
            $doc->set(PROP_NAME,$doc->get(PROP_NAME).' (Expired)');            
        }     
    }
}

function filelock_get_expiration_info($docId)
{
    $query = 'select * from sh_filelock_files where document_id='.$docId;
    $result = db()->GetRow($query);
    return $result;
}

function filelock_document_actions ($params)
{                                          
    $actions = &$params['actions'];
    $doc = &$params['doc'];
    
    if (app()->isAdminLoggedIn())
    {
        $actions[] =     
        shPage::makeActionArray ('Set Timeout', 
                         'plugins/filelock/expire.png',
                         'index.php?a=filelock_settimeout&id='.$doc->get(PROP_ID),
                         '', 
                         'Set Timeout');            
    }
    else
    {
        // don't let those who are not admins view the document if it has
        //  expired. 
        
        if (filelock_has_doc_expired($doc->get(PROP_ID)))
        {            
            for ($i = count($actions)-1; $i >= 0; $i--)
            {
                $actionId = $actions[$i]['id'];
                if (($actionId == ACTION_VIEW) || 
                    ($actionId == ACTION_EDIT) ||
                    ($actionId == ACTION_TRASH) ||
                    ($actionId == ACTION_BRIEFCASE))                
                {
                    unset($actions[$i]);
                }
            }
        }
    }
}

function filelock_get_forms($params)
{
    $forms = &$params['forms'];
    $forms[] = 'FileLockForm';
}

function filelock_get_document_header($params)
{
    $doc = &$params['doc'];
    $expirationInfo = filelock_get_expiration_info($doc->get(PROP_ID));
    if ($expirationInfo)
    {
        $header = &$params['header'];

		if (strtotime($expirationInfo['expiration_date']) > time())
		{
        	$header .= '<div class="warning">This file is set to expire on '.iwUtilities::dateTimeFormat($expirationInfo['expiration_date'],true).'</div>';
		}
		else
		{
			$header .= '<div class="error">This file expired on '.iwUtilities::dateTimeFormat($expirationInfo['expiration_date'],true).'.  Access denied.</div>';
		}
    }
}

function filelock_alter_browser_display($params)
{
    $doc = &$params['doc'];
    $info = &$params['info'];
    $expirationInfo = filelock_get_expiration_info($doc->get(PROP_ID));
    if ($expirationInfo)
    {
        if (strtotime($expirationInfo['expiration_date']) > time())
        {
            $info['description'] .= addslashes('  [<font color="#008800">This file </b>is set to expire on '.iwUtilities::dateTimeFormat($expirationInfo['expiration_date'],true).'</font>]');
        }
        else
        {
            $info['description'] .= addslashes('  [<font color="#880000">This file expired on '.iwUtilities::dateTimeFormat($expirationInfo['expiration_date'],true).'</font>]');
        }        
        
    }    
    
}

?>
